@@ -1,7 +1,7 @@
 #!/bin/bash
 # Get the index files for each LDAT file
 
-for i in `ls --color=no *.ldat`
+for i in *.ldat
 do
   indexfile=`echo $i | sed 's/ldat/pack/'`
   echo "INDEX: $indexfile"
