@@ -25,9 +25,11 @@
 #define ANIMATION_HEADER
 
 /* Functions */
-extern void init_animation (void);
 extern void reinit_animation (void);
-extern Uint8 pause_game (void);
+extern void recalc_geometry(void);
+extern void rearrange_animation (void);
+extern int pause_game (void);
+extern void kill_plr_screen (int plr);
 
 extern void animate_frame (void);
 
