@@ -22,6 +22,8 @@
  */
 
 #include "defines.h"
+#include "audio.h"
+
 #if HAVE_LIBSDL_MIXER
 #include <stdlib.h>
 #include <stdio.h>
@@ -40,7 +42,6 @@
 
 
 #include "console.h"
-#include "audio.h"
 #include "fs.h"
 #include "stringutil.h"
 #include "startup.h"
@@ -67,25 +68,30 @@ int song_count, original_song_count;
  * in audio.h
 */
 static char *wavs[] = { "blip.wav", "blip2.wav", "fire.wav",
-    "explosion.wav", "laser.wav", "jump.wav",
-    "missile.wav", "zap.wav", "swoosh.wav", "burn.wav",
-    "dart.wav", "snowball.wav",
+    "fire2.wav","explosion.wav", "largexpl.wav","crash.wav",
+    "laser.wav", "jump.wav", "missile.wav", "zap.wav", "swoosh.wav",
+    "burn.wav", "dart.wav", "snowball.wav",
     "cow.wav", "bird.wav", "steam.wav"
 };
 
 static Mix_Chunk *samples[SAMPLE_COUNT];
 static Mix_Music *music;
 
+#endif
+
 /* Music stopped playing, move on to the next track */
 void playlist_forward (void)
 {
+#if HAVE_LIBSDL_MIXER
     music_skip (1);
     music_play ();
+#endif
 }
 
 /* Initialize */
 void init_audio ()
 {
+#if HAVE_LIBSDL_MIXER
     int w;
     Playlist *next;
 
@@ -175,17 +181,21 @@ void init_audio ()
                 break;
         }
     }
+#endif
 }
 
 void playwave (AudioSample sample)
 {
+#if HAVE_LIBSDL_MIXER
     if (!audio_available || !game_settings.sounds)
         return;
     Mix_PlayChannel (-1, samples[sample], 0);
+#endif
 }
 
 void playwave_3d (AudioSample sample, int x, int y)
 {
+#if HAVE_LIBSDL_MIXER
 #ifdef MIXER_HAS_EFFECTS
     Uint8 dist;
     Sint16 angle;
@@ -210,25 +220,47 @@ void playwave_3d (AudioSample sample, int x, int y)
 #else
     Mix_PlayChannel (-1, samples[sample], 0);
 #endif
+#endif
+}
+
+void audio_setsndvolume(int volume)
+{
+#if HAVE_LIBSDL_MIXER
+    if (audio_available)
+        Mix_Volume(-1,volume);
+#endif
+}
+
+void audio_setmusvolume(int volume)
+{
+#if HAVE_LIBSDL_MIXER
+    if (audio_available)
+        Mix_VolumeMusic(volume);
+#endif
 }
 
 void music_play (void)
 {
+#if HAVE_LIBSDL_MIXER
     if (!playlist || game_settings.music == 0)
         return;
     Mix_VolumeMusic (game_settings.music_vol);
     Mix_PlayMusic (music, 1);
+#endif
 }
 
 void music_stop (void)
 {
+#if HAVE_LIBSDL_MIXER
     if (!playlist || game_settings.music == 0)
         return;
     Mix_HaltMusic ();
+#endif
 }
 
 void music_skip (int skips)
 {
+#if HAVE_LIBSDL_MIXER
     int s;
     Playlist *next;
     if (!playlist || game_settings.music == 0)
@@ -261,16 +293,21 @@ void music_skip (int skips)
         if (playlist == NULL)
             break;
     }
+#endif
 }
 
 void music_newplaylist (void)
 {
+#if HAVE_LIBSDL_MIXER
     playlist_original = playlist;
     song_count = 0;
     playlist = NULL;
+#endif
 }
+
 void music_restoreplaylist (void)
 {
+#if HAVE_LIBSDL_MIXER
     Playlist *tmp;
     if (playlist) {
         while (playlist->prev)
@@ -283,9 +320,12 @@ void music_restoreplaylist (void)
     }
     playlist = playlist_original;
     song_count = original_song_count;
+#endif
 }
+
 void music_add_song (char *filename)
 {
+#if HAVE_LIBSDL_MIXER
     if (playlist == NULL) {
         playlist = malloc (sizeof (Playlist));
         memset (playlist, 0, sizeof (Playlist));
@@ -300,6 +340,6 @@ void music_add_song (char *filename)
         playlist->file = filename;
     }
     song_count++;
+#endif
 }
 
-#endif
