@@ -26,17 +26,22 @@
 
 /* Samples */
 typedef enum { WAV_BLIP, WAV_BLIP2, WAV_NORMALWEAP,
-    WAV_EXPLOSION, WAV_LASER, WAV_JUMP, WAV_MISSILE,
+    WAV_SPECIALWEAP, WAV_EXPLOSION, WAV_EXPLOSION2,
+    WAV_CRASH,
+    WAV_LASER, WAV_JUMP, WAV_MISSILE,
     WAV_ZAP, WAV_SWOOSH, WAV_BURN,
     WAV_DART, WAV_FREEZE,
     WAV_CRITTER1, WAV_CRITTER2, WAV_STEAM
 } AudioSample;
 
-#define SAMPLE_COUNT 15
+#define SAMPLE_COUNT 18
 
 /* Initialization */
 extern void init_audio (void);
 
+extern void audio_setsndvolume(int volume);
+extern void audio_setmusvolume(int volume);
+
 /* Playback */
 extern void playwave (AudioSample sample);
 extern void playwave_pan (AudioSample sample, unsigned char left,
