@@ -24,6 +24,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "SDL_endian.h"
+
 #include "defines.h"
 #include "fs.h"
 #include "console.h"
@@ -102,9 +104,27 @@ void init_sdl ()
 
 /** Initialize video **/
 void init_video () {
+    int swidth,sheight;
+    switch(luola_options.videomode) {
+        case VID_640:
+            swidth = 640;
+            sheight = 480;
+            break;
+        case VID_800:
+            swidth = 800;
+            sheight = 600;
+            break;
+        case VID_1024:
+            swidth = 1024;
+            sheight = 768;
+            break;
+        default:
+            fprintf(stderr,"Bug! init_video(): unknown video mode %d!\n",luola_options.videomode);
+            abort();
+    }
     screen =
-        SDL_SetVideoMode (SCREEN_W, SCREEN_H, SCREEN_DEPTH,
-                          luola_options.fullscreen?SDL_FULLSCREEN:0|SDL_SWSURFACE);
+        SDL_SetVideoMode (swidth, sheight, SCREEN_DEPTH,
+                          (luola_options.fullscreen?SDL_FULLSCREEN:0)|SDL_SWSURFACE);
     if (screen == NULL) {
         printf ("Unable to set video mode: %s\n", SDL_GetError ());
         exit (1);
@@ -112,55 +132,47 @@ void init_video () {
     if (luola_options.hidemouse)
         SDL_ShowCursor (SDL_DISABLE);
     /* Set window caption */
-    SDL_WM_SetCaption (PACKAGE " " VERSION, PACKAGE);
+    SDL_WM_SetCaption (PACKAGE_STRING, PACKAGE);
     /* Initialize colours */
+    col_black = map_rgba (0, 0, 0, 255);
+    col_gray = map_rgba (128, 128, 128, 255);
+    col_grenade = map_rgba (160, 160, 160, 255);
+    col_snow = map_rgba (176, 193, 255, 255);
+    col_clay = map_rgba (255, 200, 128, 255);
+    col_default = map_rgba (255, 100, 100, 255);
+    col_yellow = map_rgba (255, 255, 100, 255);
+    col_red = map_rgba (255, 0, 0, 255);
+    col_blue = map_rgba (0, 0, 255, 255);
+    col_white = map_rgba (255, 255, 255, 255);
+    col_rope = map_rgba (178, 159, 103, 255);
+    col_plrs[0] = map_rgba (255, 0, 0, 255);
+    col_plrs[1] = map_rgba (0, 0, 255, 255);
+    col_plrs[2] = map_rgba (0, 255, 0, 255);
+    col_plrs[3] = map_rgba (255, 255, 0, 255);
+    col_pause_backg = map_rgba (128, 128, 128, 128);
+    col_green = map_rgba (0, 158, 0, 255);
+    col_transculent = map_rgba(255,255,255,128);
+}
+
+/* Map RGBA colors using either plain SDL or SDL_gfx */
+Uint32 map_rgba(Uint8 r,Uint8 g,Uint8 b,Uint8 a)
+{
 #if HAVE_LIBSDL_GFX
-    col_black = 0x000000FF;
-    col_gray = 0x808080FF;
-    col_grenade = 0xA0A0A0FF;
-    col_snow = 0xB0C1FFFF;
-    col_clay = 0xFFC880FF;
-    col_default = 0xFF6464FF;
-    col_yellow = 0xFFFF64FF;
-    col_red = 0xFF0000FF;
-    col_white = 0xFFFFFFFF;
-    col_rope = 0xB29F67FF;
-    col_plrs[0] = 0xFF0000FF;
-    col_plrs[1] = 0x0000FFFF;
-    col_plrs[2] = 0x00FF00FF;
-    col_plrs[3] = 0xFFFF00FF;
-    col_pause_backg = 0x80808080;
-    col_green = 0x009E00FF;
-    col_blue = 0x0000FFFF;
-    col_transculent = 0xFFFFFF80;
+        return (r << 24) + (g << 16) + (b << 8) + a;
 #else
-    col_black = SDL_MapRGB (screen->format, 0, 0, 0);
-    col_gray = SDL_MapRGB (screen->format, 128, 128, 128);
-    col_grenade = SDL_MapRGB (screen->format, 160, 160, 160);
-    col_snow = SDL_MapRGB (screen->format, 176, 193, 255);
-    col_clay = SDL_MapRGB (screen->format, 255, 200, 128);
-    col_default = SDL_MapRGB (screen->format, 255, 100, 100);
-    col_yellow = SDL_MapRGB (screen->format, 255, 255, 100);
-    col_red = SDL_MapRGB (screen->format, 255, 0, 0);
-    col_blue = SDL_MapRGB (screen->format, 0, 0, 255);
-    col_white = SDL_MapRGB (screen->format, 255, 255, 255);
-    col_rope = SDL_MapRGB (screen->format, 178, 159, 103);
-    col_plrs[0] = SDL_MapRGB (screen->format, 255, 0, 0);
-    col_plrs[1] = SDL_MapRGB (screen->format, 0, 0, 255);
-    col_plrs[2] = SDL_MapRGB (screen->format, 0, 255, 0);
-    col_plrs[3] = SDL_MapRGB (screen->format, 255, 255, 0);
-    col_pause_backg = SDL_MapRGB (screen->format, 128, 128, 128);
-    col_green = SDL_MapRGB (screen->format, 0, 158, 0);
-    col_transculent = col_gray;
+        r = r/255.0 * a;
+        g = g/255.0 * a;
+        b = b/255.0 * a;
+        return SDL_MapRGB (screen->format, r, g, b);
 #endif
 }
 
-/**** Draw a box *****/
-void draw_box (int x, int y, int w, int h, char width, Uint32 color) {
+/* Draw a box */
+void draw_box (int x, int y, int w, int h, int width, Uint32 color) {
 #if HAVE_LIBSDL_GFX
     int r;
     for (r = 0; r < width; r++)
-        rectangleColor (screen, x + r, y + r, x + w - r, y + h - r, color);
+        rectangleColor (screen, x + r, y + r, x + w - r-1, y + h - r-1, color);
 #else
     SDL_Rect rect;
     rect.x = x;
@@ -180,18 +192,26 @@ void draw_box (int x, int y, int w, int h, char width, Uint32 color) {
 #endif
 }
 
+/* Draw a filled box */
+void fill_box (SDL_Surface *surface,int x, int y, int w, int h, Uint32 color) {
+#if HAVE_LIBSDL_GFX
+    boxColor(surface, x, y, x+w, y+h, color);
+#else
+    SDL_Rect rect;
+    rect.x = x; rect.y = y;
+    rect.w = w; rect.h = h;
+    SDL_FillRect(surface, &rect, color);
+#endif
+}
+
 /* Do a raw 32bit pixel copy */
 void pixelcopy (Uint32 * srcpix, Uint32 * pixels, int w, int h, int srcpitch,
                 int pitch) {
-    int x, y;
+    int y;
     for (y = 0; y < h; y++) {
-        for (x = 0; x < w; x++) {
-            *pixels = *srcpix;
-            srcpix++;
-            pixels++;
-        }
-        pixels += pitch - w;
-        srcpix += srcpitch - w;
+        memcpy(pixels,srcpix,w*sizeof(Uint32));
+        pixels += pitch;
+        srcpix += srcpitch;
     }
 }
 
@@ -328,8 +348,7 @@ void recolor (SDL_Surface * surface, float red, float green, float blue,
               float alpha) {
     Uint8 *pos, *end;
     if (surface->format->BytesPerPixel != 4) {
-        printf
-            ("Error ! Unsupported color depth (%d bytes per pixel), currently only 4 bps is supported ! (fix this at recolor(), console.c)\n",
+        printf("Error! Unsupported color depth (%d bytes per pixel), currently only 4 bps is supported. (fix this at recolor(), console.c)\n",
              screen->format->BytesPerPixel);
         exit (1);
     }
@@ -337,65 +356,69 @@ void recolor (SDL_Surface * surface, float red, float green, float blue,
     pos = (Uint8 *) surface->pixels;
     end = pos + 4 * surface->w * surface->h;
     while (pos < end) {
+#if SDL_BYTEORDER == SDL_LIL_ENDIAN
         (*pos++) *= blue;
         (*pos++) *= green;
         (*pos++) *= red;
         (*pos++) *= alpha;
+#else
+        (*pos++) *= alpha;
+        (*pos++) *= red;
+        (*pos++) *= green;
+        (*pos++) *= blue;
+#endif
     }
 }
 
-#ifndef HAVE_LIBSDL_GFX
-SDL_Surface *scale_surface (SDL_Surface * original, int sX, int sY,
-                            int smooth)
-{ /* Note. smooth is unused. It is for API compatability with zoomSurface from SDL_gfx */
+/* Return a scaled version of the surface */
+SDL_Surface *zoom_surface(SDL_Surface *original, float aspect, float zoom) {
     SDL_Surface *scaled;
-    Uint8 *bits, *targ, bpp;
-    int x, y, xscale, yscale;
+    Uint8 *targ, bpp;
+    int x, y;
+
     bpp = original->format->BytesPerPixel;
+    if(bpp!=1 && bpp!=4) {
+        fprintf(stderr,"zoom_surface(): surface has %d bytes per pixel, only 1 and 4 are supported!\n",bpp);
+        return NULL;
+    }
     scaled =
-        SDL_CreateRGBSurface (original->flags, original->w * sX,
-                              original->h * sY,
+        SDL_CreateRGBSurface (original->flags, original->w * aspect * zoom,
+                              original->h * zoom,
                               original->format->BitsPerPixel,
                               original->format->Rmask,
                               original->format->Gmask,
                               original->format->Bmask,
                               original->format->Amask);
-    if (bpp == 1)               /* Copy the palette */
+    if (bpp == 1)
         SDL_SetPalette (scaled, SDL_LOGPAL, original->format->palette->colors,
                         0, original->format->palette->ncolors);
-    bits = ((Uint8 *) original->pixels);
     targ = ((Uint8 *) scaled->pixels);
-    for (y = 0; y < original->h; y++) {
-        for (x = 0; x < original->w; x++) {
+    for (y = 0; y < scaled->h; y++) {
+        for (x = 0; x < scaled->w; x++) {
+#if 1 /* This works with a limited set of bitdepths, but is faster than using memcpy */
             switch (bpp) {
             case 1:
-                for (yscale = 0; yscale < sY; yscale++) {
-                    for (xscale = 0; xscale < sX; xscale++) {
-                        *((Uint8 *) (targ)) = (*(Uint8 *) (bits));
-                        targ++;
-                    }
-                    targ += scaled->pitch - sX;
-                }
+                *((Uint8 *) (targ)) =
+                    *((Uint8 *) (original->pixels)+((int)(y/zoom))*original->w+(int)(x/(aspect*zoom)));
+                targ++;
                 break;
             case 4:
-                for (yscale = 0; yscale < sY; yscale++) {
-                    for (xscale = 0; xscale < sX; xscale++) {
-                        *((Uint32 *) (targ)) = (*(Uint32 *) (bits));
-                        targ += 4;
-                    }
-                    targ += scaled->pitch - 4 * sX;
-                }
+                *((Uint32 *) (targ)) =
+                    *((Uint32 *) (original->pixels)+((int)(y/zoom))*original->w+(int)(x/(aspect*zoom)));
+                targ += 4;
+                break;
             }
-            bits += bpp;
-            targ -= (scaled->pitch * sY) - bpp * sX;
+#else /* This works with all bitdepths, but is a bit slower */
+            memcpy(targ,original->pixels+(int)(y/zoom)*original->pitch+
+                    (int)(x/(aspect*zoom))*bpp,bpp);
+            targ+=bpp;
+#endif
         }
-        targ += scaled->pitch * (sY - 1);
+        targ += scaled->pitch - scaled->w*bpp;
     }
     return scaled;
 }
 
-#endif
-
 /* Wait for a keypress */
 /* Also handles screenshot and toggling between fullscreen
  * and windowed mode.
@@ -405,9 +428,9 @@ void wait_for_enter(void) {
     while(1) {
         SDL_WaitEvent(&e);
         if(e.type==SDL_KEYDOWN) {
-            if(e.key.keysym.sym == SDLK_RETURN) {
+            if(e.key.keysym.sym == SDLK_RETURN || e.key.keysym.sym == SDLK_KP_ENTER) {
                 if((e.key.keysym.mod & (KMOD_LALT|KMOD_RALT)))
-                    SDL_WM_ToggleFullScreen(screen);
+                    toggle_fullscreen();
                 else
                     break;
             } else if(e.key.keysym.sym == SDLK_F11) {
@@ -418,10 +441,44 @@ void wait_for_enter(void) {
     }
 }
 
+/* Toggle between fullscreen and windowed mode */
+void toggle_fullscreen(void) {
+#if (defined(unix) || defined(__unix__) || defined(_AIX) || \
+     defined(__OpenBSD__)) && (!defined(DISABLE_X11) && \
+     !defined(__CYGWIN32__) && !defined(ENABLE_NANOX) && \
+     !defined(__QNXNTO__))
+    /* This is the easy way, but it only works under X11 */
+    SDL_WM_ToggleFullScreen(screen);
+#else
+    int fullscreen = !(screen->flags & SDL_FULLSCREEN);
+    int w=screen->w,h=screen->h;
+    char *pixels;
+    /* The pixel data on screen will be lost, so copy it first */
+    pixels = malloc(screen->h*screen->pitch);
+    memcpy(pixels,screen->pixels,screen->h*screen->pitch);
+
+    /* Create a new screen */
+    SDL_FreeSurface(screen);
+    screen =
+        SDL_SetVideoMode (w, h, SCREEN_DEPTH,
+                          (fullscreen?SDL_FULLSCREEN:0)|SDL_SWSURFACE);
+    if (screen == NULL) {
+        printf ("Unable to set video mode: %s\n", SDL_GetError ());
+        exit (1);
+    }
+
+    /* Copy back the pixel data */
+    memcpy(screen->pixels,pixels,screen->h*screen->pitch);
+    free(pixels);
+    SDL_UpdateRect(screen,0,0,0,0);
+#endif
+}
+
 /* Translate joystick button press to keypress */
 void joystick_button (SDL_JoyButtonEvent * btn) {
     SDL_Event event;
     event.key.state = btn->state;
+    event.key.keysym.mod = 0;
     event.key.type =
         (btn->type == SDL_JOYBUTTONDOWN) ? SDL_KEYDOWN : SDL_KEYUP;
     if (btn->button == 0 || btn->button == 3)
@@ -476,6 +533,7 @@ void joystick_motion (SDL_JoyAxisEvent * axis, char plrmode) {
         else
             event.key.keysym.sym = SDLK_RIGHT;
     }
+    event.key.keysym.mod = 0;
     SDL_PushEvent (&event);
 }
 
@@ -487,7 +545,7 @@ void joystick_motion (SDL_JoyAxisEvent * axis, char plrmode) {
 
 inline void putpixel (SDL_Surface * surface, int x, int y, Uint32 color) {
 #if SCREEN_DEPTH != 32
-#error update putpixel() with bitdepths other than 32 !
+#error update putpixel() with bitdepths other than 32!
 #endif
     Uint8 *bits, bpp;
     if (x < 0 || y < 0 || x >= surface->w || y >= surface->h)
@@ -521,6 +579,54 @@ inline void putpixel (SDL_Surface * surface, int x, int y, Uint32 color) {
 }
 #endif
 
+/*
+ * Get a single pixel
+ */
+Uint32 getpixel (SDL_Surface * surface, int x, int y)
+{
+    Uint32 color = 0, temp;
+    SDL_Color scolor;
+    switch (surface->format->BytesPerPixel) {
+    case 1:
+        color = *((Uint8 *) surface->pixels + y * surface->pitch + x);
+        scolor = surface->format->palette->colors[color];
+        color = (scolor.r << 24) | (scolor.g << 16) | (scolor.b << 8);
+        break;
+    case 2:
+        temp = *((Uint16 *) surface->pixels + y * surface->pitch / 2 + x * 2);
+        color = temp & surface->format->Rmask;
+        color = temp >> surface->format->Rshift;
+        color = temp << surface->format->Rloss;
+        scolor.r = color;
+        color = temp & surface->format->Gmask;
+        color = temp >> surface->format->Gshift;
+        color = temp << surface->format->Gloss;
+        scolor.g = color;
+        color = temp & surface->format->Bmask;
+        color = temp >> surface->format->Bshift;
+        color = temp << surface->format->Bloss;
+        scolor.b = color;
+        color = scolor.r << 24 | scolor.g << 16 | scolor.b << 8;
+        break;
+    case 3:                    /* Format/endian independent */
+        scolor.r =
+            *(((Uint8 *) surface->pixels + y * surface->pitch + x * 3) +
+              surface->format->Rshift / 8);
+        scolor.g =
+            *(((Uint8 *) surface->pixels + y * surface->pitch + x * 3) +
+              surface->format->Gshift / 8);
+        scolor.b =
+            *(((Uint8 *) surface->pixels + y * surface->pitch + x * 3) +
+              surface->format->Bshift / 8);
+        color = scolor.r << 24 | scolor.g << 16 | scolor.b << 8;
+        break;
+    case 4:
+        color = *((Uint32*)surface->pixels + y * (surface->pitch/4) + x);
+        break;
+    }
+    return color;
+}
+
 #ifndef HAVE_LIBSDL_GFX
 /* Draw a line from point A to point B */
 void draw_line (SDL_Surface * screen, int x1, int y1, int x2, int y2,
@@ -561,7 +667,7 @@ void draw_line (SDL_Surface * screen, int x1, int y1, int x2, int y2,
     x = x1;
     y = y1;
 #if SCREEN_DEPTH != 32
-#error update draw_line() with bitdepths other than 32 !
+#error update draw_line() with bitdepths other than 32!
 #endif
     bpp = screen->format->BytesPerPixel;
     if (ax > ay) {
@@ -670,17 +776,3 @@ void draw_line (SDL_Surface * screen, int x1, int y1, int x2, int y2,
 
 #endif
 
-void kill_plr_screen (unsigned char plr)
-{
-    SDL_Rect rect, rect2;
-    rect.x = (plr == 1 || plr == 3) * 320;
-    rect2.x = rect.x + 160 - plr_messages[plr]->w / 2;
-    rect.y = (plr > 1) * 240;
-    rect2.y = rect.y + 120 - plr_messages[plr]->h / 2;
-    rect.w = 320;
-    rect.h = 240;
-    SDL_FillRect (screen, &rect, 0);
-    SDL_BlitSurface (plr_messages[plr], NULL, screen, &rect2);
-    SDL_UpdateRect (screen, rect.x, rect.y, rect.w, rect.h);
-}
-
