@@ -23,7 +23,6 @@
 
 #include <stdlib.h>
 #include <math.h>
-#include <SDL/SDL.h>
 
 #include "defines.h"
 #include "console.h"
@@ -37,9 +36,7 @@
 #include "ship.h"
 #include "fs.h"
 
-#if HAVE_LIBSDL_MIXER
 #include "audio.h"
-#endif
 
 static struct dllist *critters;
 
@@ -154,8 +151,8 @@ void prepare_critters (LevelSettings * settings)
     }
     for (r = 0; r < level_settings.bats; r++) {
         crit_bat_attack[r].src.y = 0;
-        crit_bat_attack[r].src.w = 320;
-        crit_bat_attack[r].src.h = 240;
+        crit_bat_attack[r].src.w = screen->w/2;
+        crit_bat_attack[r].src.h = screen->h/2;
         crit_bat_attack[r].end = 0;
     }
     /* Add manually placed critters */
@@ -389,7 +386,7 @@ void cow_storm (int x, int y)
 /** Animate critters **/
 void animate_critters (void) {
     struct dllist *list = critters;
-    struct dllist *shiplist;
+    struct dllist *ships;
     int maxframes;
     int i1;
     double f1;
@@ -423,9 +420,9 @@ void animate_critters (void) {
         if (critter->angry)
             critter->angry--;
         /* Gravity effects, check if someone is operating a force field nearby */
-        shiplist = getshiplist ();
-        while (shiplist) {
-            struct Ship *ship=shiplist->data;
+        ships = ship_list;
+        while (ships) {
+            struct Ship *ship=ships->data;
             if (ship->shieldup) {
                 f1 = hypot (abs (ship->x - critter->x),
                             abs (ship->y - critter->y));
@@ -446,7 +443,7 @@ void animate_critters (void) {
                     critter->vector.y -= 0.6;
                 }
             }
-            shiplist = shiplist->next;
+            ships = ships->next;
         }
         /* Critters don't go thru walls */
         if (critter->x > lev_level.width - 20)
@@ -482,8 +479,8 @@ void animate_critters (void) {
             critter->vector.x /= 1.5;
         }
         /* Apply vectors */
-        critter->y += round (critter->vector.y);
-        critter->x += round (critter->vector.x);
+        critter->y += Round (critter->vector.y);
+        critter->x += Round (critter->vector.x);
         /* Make sure the critter doesn't leave the level boundaries */
         if (critter->x < 0)
             critter->x = 0;
@@ -737,33 +734,35 @@ static void bat_movement (Critter * critter, char hitwall)
     if (f < 60) {
         dx = players[p].ship->x - rx;
         dy = players[p].ship->y - ry;
-        if (f < 20 && critter->carried == 0 && bat_attack && players[p].ship) { /* Bat attack ! */
+        if (f < 20 && critter->carried == 0 && bat_attack && players[p].ship) {
+            /* Bat attack! */
             for (b = 0; b < level_settings.bats; b++) {
                 if (crit_bat_attack[b].me == critter
                     && crit_bat_attack[b].end)
                     break;
                 if (crit_bat_attack[b].end == 0) {
                     int dx, dy;
-                    dx = 60 - rand () % 120;
-                    dy = 80 - rand () % 160;
-                    crit_bat_attack[b].targ.x =
-                        ((p == 1 || p == 3) ? 320 : 0) + dx;
-                    crit_bat_attack[b].targ.y = ((p > 1) ? 240 : 0) + dy;
-                    crit_bat_attack[b].src.x = 320 * (rand () % 3);
+                    dx = cam_rects[p].w/2 - rand () % cam_rects[p].w;
+                    dy = cam_rects[p].h/2 - rand () % cam_rects[p].h;
+                    crit_bat_attack[b].targ.x = lev_rects[p].x + dx;
+                    crit_bat_attack[b].targ.y = lev_rects[p].y + dy;
+                    crit_bat_attack[b].src.x = 0;
+                    crit_bat_attack[b].src.y = 0;
+                    crit_bat_attack[b].src.w = bat_attack->w;
+                    crit_bat_attack[b].src.h = bat_attack->h;
                     if (dx < 0) {
                         crit_bat_attack[b].src.x -= dx;
-                        crit_bat_attack[b].src.w = 320 + dx;
+                        crit_bat_attack[b].src.w = bat_attack->w + dx;
                         crit_bat_attack[b].targ.x -= dx;
-                    } else {
-                        crit_bat_attack[b].src.w = 320 - dx;
+                    } else if(dx+bat_attack->w>cam_rects[p].w) {
+                        crit_bat_attack[b].src.w -= dx + bat_attack->w - cam_rects[p].w;
                     }
                     if (dy < 0) {
                         crit_bat_attack[b].src.y = -dy;
-                        crit_bat_attack[b].src.h = 240 + dy;
+                        crit_bat_attack[b].src.h = bat_attack->h + dy;
                         crit_bat_attack[b].targ.y -= dy;
-                    } else {
-                        crit_bat_attack[b].src.y = 0;
-                        crit_bat_attack[b].src.h = 240 - dy;
+                    } else if(dy+bat_attack->h>cam_rects[p].h) {
+                        crit_bat_attack[b].src.h -= dy + bat_attack->h - cam_rects[p].h;
                     }
                     crit_bat_attack[b].me = critter;
                     crit_bat_attack[b].end = 10;
@@ -905,18 +904,18 @@ static inline void draw_critter (Critter * critter)
     int px, py, p;
     sprite = critter_gfx[critter->species][critter->frame];
     for (p = 0; p < 4; p++) {
-        if (players[p].state==ALIVE) {
+        if (players[p].state==ALIVE || players[p].state==DEAD) {
             rect.x = critter->x - cam_rects[p].x + lev_rects[p].x;
             rect.y = critter->y - cam_rects[p].y + lev_rects[p].y;
             if ((rect.x > lev_rects[p].x - critter->rect.w
-                 && rect.x < lev_rects[p].x + lev_rects[p].w)
+                 && rect.x < lev_rects[p].x + cam_rects[p].w)
                 && (rect.y > lev_rects[p].y - critter->rect.h
-                    && rect.y < lev_rects[p].y + lev_rects[p].h)) {
+                    && rect.y < lev_rects[p].y + cam_rects[p].h)) {
                 rect2 =
                     cliprect (rect.x, rect.y, critter->rect.w,
                               critter->rect.h, lev_rects[p].x, lev_rects[p].y,
-                              lev_rects[p].x + lev_rects[p].w,
-                              lev_rects[p].y + lev_rects[p].h);
+                              lev_rects[p].x + cam_rects[p].w,
+                              lev_rects[p].y + cam_rects[p].h);
                 rect2.x += critter->rect.x;
                 /*rect2.y+=critter->rect.y; */
                 if (rect.x < lev_rects[p].x)
@@ -1067,12 +1066,10 @@ static struct dllist *kill_critter (struct dllist *list) {
                       critter_gfx[critter->species][0]->h / 2,
                       LotsOfFeathers);
     }
-#if HAVE_LIBSDL_MIXER
     if (critter->species == Cow)
         playwave_3d (WAV_CRITTER1, critter->x, critter->y);
     else if (critter->species == Bird)
         playwave_3d (WAV_CRITTER2, critter->x, critter->y);
-#endif
     if (critter->species == Infantry)
         soldier_count[critter->owner]--;
     if (critter->species == Helicopter)
