@@ -25,12 +25,12 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 #include "defines.h"
 #include "console.h"
 #include "demo.h"
 
 #define STARFIELD_COUNT 256
-#define FADE_STEP	35
 #define FADE_STEPS	255/FADE_STEP
 
 /* Star structure */
@@ -56,20 +56,18 @@ void init_demos (void)
     for (r = 0; r < STARFIELD_COUNT; r++)
         init_star (&stars[r], r + 1);
     for (r = 0; r < 256; r++)
-#if HAVE_LIBSDL_GFX
-        starcolors[r] =
-            ((Uint32) r * 3 << 24) | ((Uint32) r * 3 << 16) | ((Uint32) r *
-                                                               3 << 8) | 0xff;
-#else
-        starcolors[r] = SDL_MapRGB (screen->format, r * 3, r * 3, r * 3);
-#endif
+        starcolors[r] = map_rgba(r*3, r*3, r*3, 255);
     demo_black =
         SDL_CreateRGBSurface (screen->flags, screen->w, screen->h,
                               screen->format->BitsPerPixel,
                               screen->format->Rmask, screen->format->Gmask,
                               screen->format->Bmask, screen->format->Amask);
-    memset (demo_black->pixels, 0, demo_black->pitch * SCREEN_H);
-    SDL_SetAlpha (demo_black, SDL_SRCALPHA | SDL_RLEACCEL, FADE_STEP);
+    if(demo_black) {
+        memset (demo_black->pixels, 0, demo_black->pitch * screen->h);
+        SDL_SetAlpha (demo_black, SDL_SRCALPHA | SDL_RLEACCEL, FADE_STEP);
+    } else {
+        fprintf(stderr,"init_demos(): %s\n",SDL_GetError());
+    }
 }
 
 /* Initialize a single star */
@@ -87,8 +85,8 @@ static void init_star (Star * star, short i)
     star->py = 0;
 
     star->color = i >> 2;       /*the closer to the viewer the brighter */
-    star->px = (star->x / star->z) + SCREEN_W / 2;
-    star->py = (star->y / star->z) + SCREEN_H / 2;
+    star->px = (star->x / star->z) + screen->w / 2;
+    star->py = (star->y / star->z) + screen->h / 2;
 
 }
 
@@ -105,9 +103,9 @@ void draw_starfield (void)
         }
 
         /*compute 3D position */
-        tempx = (stars[i].x / stars[i].z) + SCREEN_W / 2;
-        tempy = (stars[i].y / stars[i].z) + SCREEN_H / 2;
-        if (tempx < 0 || tempx > SCREEN_W - 1 || tempy < 0 || tempy > SCREEN_H - 1) {   /*check if a star leaves the screen */
+        tempx = (stars[i].x / stars[i].z) + screen->w / 2;
+        tempy = (stars[i].y / stars[i].z) + screen->h / 2;
+        if (tempx < 0 || tempx > screen->w - 1 || tempy < 0 || tempy > screen->h - 1) {   /* check if a star leaves the screen */
             init_star (stars + i, i + 1);
             continue;
         }
@@ -138,6 +136,12 @@ void fade_to_black (void) {
                               screen->format->BitsPerPixel,
                               screen->format->Rmask, screen->format->Gmask,
                               screen->format->Bmask, screen->format->Amask);
+    if(tmpsurface==NULL || demo_black==NULL) {
+        if(!tmpsurface)
+            fprintf(stderr,"fade_to_black(): %s\n",SDL_GetError());
+        SDL_FillRect(screen,NULL,0);
+        return;
+    }
     memcpy(tmpsurface->pixels,screen->pixels,screen->pitch*screen->h);
     for (r = 0; r <= FADE_STEPS; r++) {
         lasttime=SDL_GetTicks();
