@@ -28,6 +28,7 @@
 #include <SDL.h>
 
 #include "defines.h"
+#include "startup.h"
 #include "stringutil.h"
 #include "console.h"
 #include "fs.h"
@@ -39,10 +40,7 @@
 #include "animation.h"
 #include "startup.h"
 #include "parser.h"
-
-#if HAVE_LIBSDL_MIXER
 #include "audio.h"
-#endif
 
 /* Some globals */
 static SDL_Surface *gam_filler;
@@ -59,25 +57,25 @@ int init_game (void)
         ldat_open_file (getfullpath (GFX_DIRECTORY, "misc.ldat"));
     if(!datafile) return 1;
 
-    gam_filler = load_image_ldat (datafile, 1, 0, "FILLER", 0);
+    gam_filler = load_image_ldat (datafile, 1, 0, "FILLER", luola_options.videomode);
     ldat_free (datafile);
     if (gam_filler)
-        recolor (gam_filler, 0.4, 0.4, 0.4, 1.0);       /* Think the background logo is too bright or dark ? Then adjust these values ! */
+        recolor (gam_filler, 0.4, 0.4, 0.4, 1.0);
 
     /* Set up the default keys */
-    game_settings.buttons[0][0] = SDLK_UP;
-    game_settings.buttons[0][1] = SDLK_DOWN;
-    game_settings.buttons[0][2] = SDLK_LEFT;
-    game_settings.buttons[0][3] = SDLK_RIGHT;
-    game_settings.buttons[0][4] = SDLK_RSHIFT;
-    game_settings.buttons[0][5] = SDLK_RCTRL;
+    game_settings.buttons[0][0] = SDLK_w;
+    game_settings.buttons[0][1] = SDLK_s;
+    game_settings.buttons[0][2] = SDLK_a;
+    game_settings.buttons[0][3] = SDLK_d;
+    game_settings.buttons[0][4] = SDLK_LSHIFT;
+    game_settings.buttons[0][5] = SDLK_LCTRL;
 
-    game_settings.buttons[1][0] = SDLK_w;
-    game_settings.buttons[1][1] = SDLK_s;
-    game_settings.buttons[1][2] = SDLK_a;
-    game_settings.buttons[1][3] = SDLK_d;
-    game_settings.buttons[1][4] = SDLK_LSHIFT;
-    game_settings.buttons[1][5] = SDLK_LCTRL;
+    game_settings.buttons[1][0] = SDLK_UP;
+    game_settings.buttons[1][1] = SDLK_DOWN;
+    game_settings.buttons[1][2] = SDLK_LEFT;
+    game_settings.buttons[1][3] = SDLK_RIGHT;
+    game_settings.buttons[1][4] = SDLK_RSHIFT;
+    game_settings.buttons[1][5] = SDLK_RCTRL;
 
     game_settings.buttons[2][0] = SDLK_i;
     game_settings.buttons[2][1] = SDLK_k;
@@ -120,16 +118,19 @@ int init_game (void)
     game_settings.last_level = NULL;
     game_settings.gravity_bullets = 1;
     game_settings.wind_bullets = 0;
+    game_settings.large_bullets = 0;
     game_settings.weapon_switch = 0;
     game_settings.eject = 1;
     game_settings.explosions = 1;
     game_settings.holesize = 2;
     game_settings.recall = 0;
     game_settings.criticals = 0;
+    game_settings.bigscreens = 1;
 
     game_settings.sounds = 0;
     game_settings.music = 0;
     game_settings.playlist = 0;
+    game_settings.sound_vol = 128;
     game_settings.music_vol = 128;
 
     /* Set the temporary settings */
@@ -184,10 +185,13 @@ void apply_per_level_settings (LevelSettings * settings)
     }
 }
 
+/*
+ * Show game statistics screen
+ */
 void game_statistics (void)
 {
-    Uint32 color, sep_col, team_col, plr_col;
-    SDL_Rect rect1, rect2;
+    Uint32 sep_col, team_col, plr_col;
+    SDL_Rect rect;
     int t, p, teams[4];
     char tmps[256];
     char data[8][3][32];    /* Four teams and four players */
@@ -198,40 +202,23 @@ void game_statistics (void)
     int mostwins;
     char tiegame;
     int x, y, x2, y2;       /* Variables used when outputting data to screen */
-#if HAVE_LIBSDL_GFX
-    color = 0xC85050FF;
-    sep_col = 0x5050FFF0;
-    team_col = 0x0A0A40F0;
-    plr_col = 0x000032A0;
-#else
-    color = SDL_MapRGB (screen->format, 200, 80, 80);
-    sep_col = SDL_MapRGB (screen->format, 80, 80, 255);
-    team_col = SDL_MapRGB (screen->format, 10, 10, 80);
-    plr_col = SDL_MapRGB (screen->format, 0, 0, 50);
-#endif
+    sep_col = map_rgba(80, 80, 255, 240);
+    team_col = map_rgba(10, 10, 80, 240);
+    plr_col = map_rgba(0, 0, 50, 160);
     /* Draw background */
-    rect1.x = SCREEN_W / 2 - 300;
-    rect1.y = SCREEN_H / 2 - 200;
-    rect1.w = 600;
-    rect1.h = 400;
-    rect2.x = rect1.x + 2;
-    rect2.y = rect1.y + 2;
-    rect2.w = rect1.w - 4;
-    rect2.h = rect1.h - 4;
-#if HAVE_LIBSDL_GFX
-    draw_box (rect1.x, rect1.y, rect1.w, rect1.h, 2, 0xC85050BF);
-    boxColor (screen, rect2.x, rect2.y, rect2.x + rect2.w - 1,
-              rect2.y + rect2.h - 1, 0x000000BF);
-#else
-    SDL_FillRect (screen, &rect1, color);
-    SDL_FillRect (screen, &rect2, col_black);
-#endif
+    rect.x = screen->w / 2 - 300;
+    rect.y = screen->h / 2 - 200;
+    rect.w = 600;
+    rect.h = 400;
+    fill_box(screen, rect.x, rect.y, rect.w, rect.h, map_rgba(0,0,0,191));
+    draw_box(rect.x, rect.y, rect.w, rect.h,2, map_rgba(200,80,80,255));
+
     if (game_status.total_rounds == 1)
         sprintf (tmps, "Game over after one round");
     else
         sprintf (tmps, "Game over after %d rounds",
                  game_status.total_rounds);
-    centered_string (screen, Bigfont, rect2.y + 10, tmps, font_color_white);
+    centered_string (screen, Bigfont, rect.y + 10, tmps, font_color_white);
     /* Generate game statistics */
     memset (teams, 0, sizeof (int) * 4);
     data_line = 0;
@@ -291,18 +278,18 @@ void game_statistics (void)
     } else {
         for (t = 0; t < 4; t++)
             if (teams[t] == mostwins) {
-                sprintf (summary, "Team %d wins !", t + 1);
+                sprintf (summary, "Team %d wins!", t + 1);
                 break;
             }
     }
     /* Draw game statistics */
-    x = rect2.x + 10;
-    x2 = rect2.x + rect2.w - 10;
-    y = rect2.y + 75;
+    x = rect.x + 10;
+    x2 = rect.x + rect.w - 10;
+    y = rect.y + 75;
     y2 = y + 30;
-    putstring_direct (screen, Bigfont, rect2.x + 200, rect2.y + 50, "Wins",
+    putstring_direct (screen, Bigfont, rect.x + 200, rect.y + 50, "Wins",
                       font_color_green);
-    putstring_direct (screen, Bigfont, rect2.x + 300, rect2.y + 50,
+    putstring_direct (screen, Bigfont, rect.x + 300, rect.y + 50,
                       "Average lifetime", font_color_green);
     for (p = 0; p < data_line; p++) {
         if (data_format[p] == 0) {      /* Draw team line */
@@ -311,11 +298,7 @@ void game_statistics (void)
             fr.y = y;
             fr.w = x2 - x;
             fr.h = y2 - y;
-#if HAVE_LIBSDL_GFX
-            boxColor (screen, fr.x, fr.y, fr.x + fr.w, fr.y + fr.h, team_col);
-#else
-            SDL_FillRect (screen, &fr, team_col);
-#endif
+            fill_box(screen,fr.x, fr.y, fr.w, fr.h, team_col);
             draw_line (screen, x, y, x2, y, sep_col);
             putstring_direct (screen, Bigfont, x, y, data[p][0],
                               font_color_white);
@@ -325,17 +308,13 @@ void game_statistics (void)
             fr.y = y;
             fr.w = x2 - x;
             fr.h = y2 - y;
-#if HAVE_LIBSDL_GFX
-            boxColor (screen, fr.x, fr.y, fr.x + fr.w, fr.y + fr.h, plr_col);
-#else
-            SDL_FillRect (screen, &fr, plr_col);
-#endif
+            fill_box(screen,fr.x, fr.y, fr.w, fr.h, plr_col);
             putstring_direct (screen, Bigfont, x + 40, y, data[p][0],
                               font_color_white);
         }
-        putstring_direct (screen, Bigfont, rect2.x + 200, y, data[p][1],
+        putstring_direct (screen, Bigfont, rect.x + 200, y, data[p][1],
                           font_color_white);
-        putstring_direct (screen, Bigfont, rect2.x + 300, y, data[p][2],
+        putstring_direct (screen, Bigfont, rect.x + 300, y, data[p][2],
                           font_color_white);
         y += 30;
         y2 += 30;
@@ -343,30 +322,31 @@ void game_statistics (void)
     /* Draw summary line */
     putstring_direct (screen, Bigfont, x, y, summary, font_color_white);
     /* Draw the "press enter to continue" message */
-    centered_string (screen, Bigfont, rect2.y + rect2.h - 30,
+    centered_string (screen, Bigfont, rect.y + rect.h - 30,
                      "Press enter to continue", font_color_white);
-    SDL_UpdateRect (screen, rect1.x, rect1.y, rect1.w, rect1.h);
+    SDL_UpdateRect (screen, rect.x, rect.y, rect.w, rect.h);
     /* Wait for Enter to be pressed */
     wait_for_enter();
 }
 
+/* Draw the luola logo on all screen quarters.
+ * Active player screens will just draw over it. */
 void fill_unused_player_screens (void)
 {
-    SDL_Rect rect;
-    int p;
-    rect.w = 320;
-    rect.h = 320;
-    rect.x = 0;
-    rect.y = 0;
-    for (p = 0; p < 4; p++) {
-        if (gam_filler)
+    SDL_FillRect (screen, NULL, 0);
+
+    if (gam_filler) {
+        SDL_Rect rect;
+        int p;
+        rect.x = (screen->w/2 - gam_filler->w) / 2;
+        rect.y = (screen->h/2 - gam_filler->h) / 2;
+        for (p = 0; p < 4; p++) {
             SDL_BlitSurface (gam_filler, NULL, screen, &rect);
-        else
-            SDL_FillRect (screen, &rect, 0);
-        rect.x += 320;
-        if (rect.x == 640) {
-            rect.y += 240;
-            rect.x = 0;
+            rect.x += screen->w/2;
+            if (p == 1) {
+                rect.y += screen->h/2;
+                rect.x -= screen->w;
+            }
         }
     }
     SDL_UpdateRect (screen, 0, 0, 0, 0);
@@ -382,27 +362,44 @@ void game_eventloop (void) {
         while (SDL_PollEvent (&Event)) {
             switch (Event.type) {
             case SDL_KEYDOWN:
-            case SDL_KEYUP:
+                /* Key down event */
 #ifdef CHEAT_POSSIBLE
-                if (cheat && Event.type == SDL_KEYDOWN)
-                    cheatcode (Event.key.keysym.sym);
+                if (cheat) cheatcode (Event.key.keysym.sym);
 #endif
-                if (Event.key.keysym.sym == SDLK_ESCAPE
-                    && Event.type == SDL_KEYDOWN)
-                    return;
-                else if (Event.key.keysym.sym == SDLK_F1
-                         && Event.type == SDL_KEYDOWN)
+                if (Event.key.keysym.sym == SDLK_ESCAPE) return;
+                else if (Event.key.keysym.sym == SDLK_F1)
                     radars_visible = !radars_visible;
-                else if (Event.key.keysym.sym == SDLK_F11
-                         && Event.type == SDL_KEYDOWN)
+                else if (Event.key.keysym.sym == SDLK_F5) {
+                    game_settings.sound_vol -= 10;
+                    if(game_settings.sound_vol<0) game_settings.sound_vol=0;
+                    audio_setsndvolume(game_settings.sound_vol);
+                    playwave(WAV_BLIP);
+                } else if (Event.key.keysym.sym == SDLK_F6) {
+                    game_settings.sound_vol += 10;
+                    if(game_settings.sound_vol>128) game_settings.sound_vol=128;
+                    audio_setsndvolume(game_settings.sound_vol);
+                    playwave(WAV_BLIP);
+                } else if (Event.key.keysym.sym == SDLK_F7) {
+                    game_settings.music_vol -= 10;
+                    if(game_settings.music_vol<0) game_settings.music_vol=0;
+                    audio_setmusvolume(game_settings.music_vol);
+                    playwave(WAV_BLIP2);
+                } else if (Event.key.keysym.sym == SDLK_F8) {
+                    game_settings.music_vol += 10;
+                    if(game_settings.music_vol>128) game_settings.music_vol=128;
+                    audio_setmusvolume(game_settings.music_vol);
+                    playwave(WAV_BLIP2);
+                } else if (Event.key.keysym.sym == SDLK_F11)
                     screenshot ();
-                else if (Event.key.keysym.sym == SDLK_RETURN
+                else if (Event.key.keysym.sym == SDLK_PAUSE)
+                    is_not_paused = pause_game ();
+
+            case SDL_KEYUP:
+                /* Key up event. Fall through from key down */
+                if (Event.key.keysym.sym == SDLK_RETURN
                          && Event.type == SDL_KEYUP
                          && (Event.key.keysym.mod & (KMOD_LALT|KMOD_RALT)))
-                    SDL_WM_ToggleFullScreen (screen);
-                else if (Event.key.keysym.sym == SDLK_PAUSE
-                         && Event.type == SDL_KEYUP)
-                    is_not_paused = pause_game ();
+                    toggle_fullscreen();
                 else
                     player_keyhandler (&Event.key, Event.type);
                 break;
@@ -439,7 +436,7 @@ void save_game_config (void)
     filename = getfullpath (HOME_DIRECTORY, CONF_FILE);
     fp = fopen (filename, "w");
     if (!fp) {
-        printf ("Error ! Cannot open file \"%s\" for writing !\n", filename);
+        printf ("Error! Cannot open file \"%s\" for writing\n", filename);
         exit (1);
     }
     /* Write controllers */
@@ -473,6 +470,8 @@ void save_game_config (void)
     fprintf (fp, "smoke=%d\n", game_settings.enable_smoke);
     fprintf (fp, "bullet_gravity=%d\n", game_settings.gravity_bullets);
     fprintf (fp, "bullet_wind=%d\n", game_settings.wind_bullets);
+    fprintf (fp, "large_bullets=%d\n", game_settings.large_bullets);
+    fprintf (fp, "bigscreens=%d\n", game_settings.bigscreens);
     fprintf (fp, "snowfall=%d\n", game_settings.ls.snowfall);
     fprintf (fp, "stars=%d\n", game_settings.ls.stars);
     fprintf (fp, "endmode=%d\n", game_settings.endmode);
@@ -486,6 +485,7 @@ void save_game_config (void)
     fprintf (fp, "sounds=%d\n", game_settings.sounds);
     fprintf (fp, "music=%d\n", game_settings.music);
     fprintf (fp, "playlist=%d\n", game_settings.playlist);
+    fprintf (fp, "sound_volume=%d\n", game_settings.sound_vol);
     fprintf (fp, "music_volume=%d\n", game_settings.music_vol);
 
     fclose (fp);
@@ -493,9 +493,9 @@ void save_game_config (void)
 
 /* Parse configuration file settings block */
 static void parse_settings_block(struct dllist *values) {
-    CfgPtrType types[30];
-    void *pointers[30];
-    char *keys[30];
+    CfgPtrType types[33];
+    void *pointers[33];
+    char *keys[33];
 
     keys[0]="ship_collisions";      types[0]=CFG_INT; pointers[0]=&game_settings.ship_collisions;
     keys[1]="indestructable_base";  types[1]=CFG_INT; pointers[1]=&game_settings.ls.indstr_base;
@@ -527,6 +527,9 @@ static void parse_settings_block(struct dllist *values) {
     keys[27]="music";               types[27]=CFG_INT;pointers[27]=&game_settings.music;
     keys[28]="playlist";            types[28]=CFG_INT;pointers[28]=&game_settings.playlist;
     keys[29]="music_volume";        types[29]=CFG_INT;pointers[29]=&game_settings.music_vol;
+    keys[30]="sound_volume";        types[30]=CFG_INT;pointers[30]=&game_settings.sound_vol;
+    keys[31]="large_bullets";       types[31]=CFG_INT;pointers[31]=&game_settings.large_bullets;
+    keys[32]="bigscreens";          types[32]=CFG_INT;pointers[32]=&game_settings.bigscreens;
     translate_config(values,sizeof(types)/sizeof(CfgPtrType),keys,types,pointers,0);
 }
 
@@ -559,7 +562,7 @@ static void parse_keys_block(struct dllist *values,int player) {
 void load_game_config (void) {
     struct dllist *gamecfg,*cfgptr;
     
-    gamecfg=read_config_file(getfullpath(HOME_DIRECTORY,CONF_FILE));
+    gamecfg=read_config_file(getfullpath(HOME_DIRECTORY,CONF_FILE),0);
     if(!gamecfg) {
         printf("Couldn't load configuration file. Using built in defaults.\n");
         return;
@@ -577,4 +580,5 @@ void load_game_config (void) {
         else printf("Unknown block \"%s\"\n",block->title);
         cfgptr=cfgptr->next;
     }
+    dllist_free(gamecfg,free_config_file);
 }
