@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "defines.h"
+#include "startup.h"
 #include "console.h"
 #include "fs.h"
 #include "stringutil.h"
@@ -41,9 +42,7 @@
 #include "critter.h"
 #include "particle.h"
 #include "ldat.h"
-#if HAVE_LIBSDL_MIXER
 #include "audio.h"
-#endif
 #include "font.h"
 #include "menu.h"
 #include "demo.h"
@@ -56,6 +55,8 @@
 #define CHANGE_WEA_BOX_W	500
 #define CHANGE_WEA_BOX_H	250
 
+#define CURSORPOS 2
+
 /* Internally used globals */
 static ToplevelMenu *hotseat_startup_menu;
 static SDL_Surface *trophy_gfx[4];
@@ -66,7 +67,7 @@ static struct LCache {
     SDL_Surface *name;
     struct LCache *next;
 } *weaponsel_levelcache;
-static Uint8 hotseat_weaponsel_mode;
+static enum {LEVELSEL,WEAPONSEL} hotseat_weaponsel_mode;
 static MenuItem *startgame_item;
 
 /* Internally used functions */
@@ -76,7 +77,7 @@ static void draw_hotseat_startup (void);
 static void draw_hotseat_weaponsel (SDL_Surface * surface);
 
 static Uint8 hotseat_startup_eventloop (void);
-static Uint8 hotseat_weaponsel_eventloop (void);
+static Uint8 hotseat_weaponsel_eventloop (int levelcount);
 
 /* Functions called by menu callbacks */
 static SDL_Surface *ship_icon (Uint8 mid, MenuItem * item);
@@ -90,12 +91,25 @@ int init_hotseat (void) {
     LDAT *ldat;
     Menu *m;
     int r;
-    dopts = make_menu_drawingoptions (0, 100, SCREEN_W, SCREEN_H);
+    int yoffset=100;
+    switch(luola_options.videomode) {
+        case VID_640:
+            yoffset=100;
+            break;
+        case VID_800:
+            yoffset=120;
+            break;
+        case VID_1024:
+            yoffset=200;
+            break;
+    }
+
+    dopts = make_menu_drawingoptions (0, yoffset, screen->w, screen->h);
     dopts.spacing = MENU_SPACING;
-    dopts.left_offset = 240;
+    dopts.left_offset = screen->h/2;
     dopts.selection_color = SDL_MapRGB (screen->format, 0, 128, 255);
     m = create_menu (MENU_ID_HOTSEAT, MNU_DRAWING_OPTIONS, dopts,
-                     MNU_ITEM_SEP, 0, MNU_OPT_LABEL, "Who's in ?",
+                     MNU_ITEM_SEP, 0, MNU_OPT_LABEL, "Who's in?",
                      MNU_OPT_ALIGN, MNU_ALIGN_CENTER, MNU_OPT_COLOR,
                      font_color_green, 0, MNU_ITEM_LABEL, 1, MNU_OPT_LABEL,
                      "Player 1", MNU_OPT_ACTION, menu_toggle_player,
@@ -140,7 +154,7 @@ int init_hotseat (void) {
                      MNU_OPT_VALUE_TYPE, MNU_TYP_INT, MNU_OPT_VALUE,
                      &game_settings.playmode, MNU_OPT_MINVALUE, 0,
                      MNU_OPT_MAXVALUE, PLAYMODE_COUNT - 1, 0, MNU_ITEM_RETURN,
-                     8, MNU_OPT_LABEL, "Go !", MNU_OPT_COLOR,
+                     8, MNU_OPT_LABEL, "Go!", MNU_OPT_COLOR,
                      font_color_green, MNU_OPT_RVAL, MENU_RVALUE_START,
                      MNU_OPT_ALIGN, MNU_ALIGN_CENTER, MNU_OPT_DISABLED, 1, 0,
                      MNU_ITEM_RETURN, 9, MNU_OPT_LABEL, "Cancel",
@@ -164,9 +178,7 @@ int init_hotseat (void) {
 void hotseat_game (void)
 {
     int rval, r;
-#if HAVE_LIBSDL_MIXER
     LevelBgMusic *bgm;
-#endif
     reset_game ();
     for (r = 1; r <= 4; r++)
         find_item (hotseat_startup_menu->menu, r)->color = font_color_gray;
@@ -199,7 +211,6 @@ void hotseat_game (void)
         reinit_players ();
         reinit_ships (curlevel->settings);
         prepare_weather ();
-#if HAVE_LIBSDL_MIXER
         bgm = curlevel->settings->mainblock->music;
         if (bgm) {
             music_newplaylist ();
@@ -211,7 +222,6 @@ void hotseat_game (void)
             music_skip (-1);
         }
         music_play ();
-#endif
         if(game_settings.playmode==RndWeapon) {
             for(r=0;r<4;r++) {
                 if(players[r].state==ALIVE) {
@@ -220,10 +230,14 @@ void hotseat_game (void)
                 }
             }
         }
+        /* Game starts */
+        SDL_EnableKeyRepeat(0,0);
         game_eventloop ();
+        SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
+
+        /* Game finished, clean up */
         if (game_settings.mbg_anim)
             fade_to_black ();
-#if HAVE_LIBSDL_MIXER
         music_stop ();
         if (bgm) {
             music_restoreplaylist ();
@@ -232,7 +246,6 @@ void hotseat_game (void)
         } else {
             music_skip (1);
         }
-#endif
         /* Check who won */
         game_status.lastwin = -1;
         for (r = 0; r < 4; r++) {
@@ -267,7 +280,7 @@ static void draw_hotseat_startup (void)
 /* Internal */
 /* Hotseat game startup eventloop */
 /* this is where you choose who joins the game and such */
-Uint8 hotseat_startup_eventloop (void)
+static Uint8 hotseat_startup_eventloop (void)
 {
     SDL_Event Event;
     Uint8 needupdate, rval, isevent;
@@ -302,19 +315,17 @@ Uint8 hotseat_startup_eventloop (void)
                     command = MenuRight;
                 else if (Event.key.keysym.sym == SDLK_RETURN) {
                     if((Event.key.keysym.mod & (KMOD_LALT|KMOD_RALT)))
-                        SDL_WM_ToggleFullScreen (screen);
+                        toggle_fullscreen();
                     else
                         command = MenuEnter;
                 } else if (Event.key.keysym.sym == SDLK_ESCAPE)
                     command = MenuESC;
                 if (command >= 0) {
-#if HAVE_LIBSDL_MIXER
                     if (command != MenuEnter && command != MenuESC
                         && command != MenuToggleLock)
                         playwave (WAV_BLIP);
                     else
                         playwave (WAV_BLIP2);
-#endif
                     rval = menu_control (hotseat_startup_menu, command);
                     if (rval == MENU_RVALUE_START
                         || rval == MENU_RVALUE_CANCEL)
@@ -351,7 +362,7 @@ Uint8 hotseat_startup_eventloop (void)
 }
 
 /* Called by a menu callback */
-SDL_Surface *ship_icon (Uint8 mid, MenuItem * item)
+static SDL_Surface *ship_icon (Uint8 mid, MenuItem * item)
 {
     Uint8 plr;
     plr = item->ID - 1;
@@ -363,13 +374,13 @@ SDL_Surface *ship_icon (Uint8 mid, MenuItem * item)
 
 /* Called by a menu callback */
 /* A dummy icon */
-int menu_icon_spacer (int x, int y, Uint8 align, int mid, MenuItem * item)
+static int menu_icon_spacer (int x, int y, Uint8 align, int mid, MenuItem * item)
 {
     return 10;
 }
 
 /* Called by a menu callback */
-Uint8 menu_toggle_player (MenuCommand cmd, int mid, MenuItem * item)
+static Uint8 menu_toggle_player (MenuCommand cmd, int mid, MenuItem * item)
 {
     Uint8 plr, newstate;
     plr = item->ID - 1;
@@ -384,7 +395,7 @@ Uint8 menu_toggle_player (MenuCommand cmd, int mid, MenuItem * item)
         item->color = newstate ? font_color_white : font_color_gray;
     } else if (cmd == MenuLeft) {
         player_teams[plr]--;
-        if (player_teams[plr] > 3)
+        if (player_teams[plr] < 0)
             player_teams[plr] = 3;
     } else if (cmd == MenuRight) {
         player_teams[plr]++;
@@ -399,7 +410,7 @@ Uint8 menu_toggle_player (MenuCommand cmd, int mid, MenuItem * item)
 }
 
 /* Called by a menu callback */
-char *hotseat_label_function (int mid, MenuItem * item)
+static char *hotseat_label_function (int mid, MenuItem * item)
 {
     if (item->ID == MENU_ITEM_PLAYMODE) {
         switch (game_settings.playmode) {
@@ -425,9 +436,9 @@ static Uint8 hotseat_weaponsel (void)
     struct LCache *lc, *prev = NULL;
     struct LevelFile *level;
     struct dllist *lev;
-    int r, rval;
+    int r, rval,levelcount=5;
     /* Do some preparations */
-    hotseat_weaponsel_mode = 0;
+    hotseat_weaponsel_mode = LEVELSEL;
     weaponsel_title = NULL;
     weaponsel_prompt = NULL;
     weaponsel_levelcache = NULL;
@@ -436,12 +447,19 @@ static Uint8 hotseat_weaponsel (void)
         weaponsel_weapons[1][r] = NULL;
     }
     lev = game_settings.levels;
-    for (r = 0; r < 2; r++) {
+    /* Rewind the level pointer back. (levels above the cursor) */
+    for (r = 0; r < CURSORPOS; r++) {
         lev = lev->prev;
         if (lev == NULL)
             lev = game_settings.last_level;
     }
-    for (r = 0; r < 5; r++) {
+    switch(luola_options.videomode) {
+        case VID_640: levelcount=5; break;
+        case VID_800: levelcount=8; break;
+        case VID_1024: levelcount= 12; break;
+    }
+    /* Make a cache of rendered level names */
+    for (r = 0; r < levelcount; r++) {
         level = lev->data;
         lc = malloc (sizeof (struct LCache));
         if (weaponsel_levelcache == NULL)
@@ -473,7 +491,7 @@ static Uint8 hotseat_weaponsel (void)
         draw_hotseat_weaponsel (screen);
         SDL_UpdateRect (screen, 0, 0, 0, 0);
     }
-    rval = hotseat_weaponsel_eventloop ();
+    rval = hotseat_weaponsel_eventloop (levelcount);
     /* Clean up */
     SDL_FreeSurface (weaponsel_title);
     SDL_FreeSurface (weaponsel_prompt);
@@ -498,43 +516,23 @@ static void draw_hotseat_weaponsel (SDL_Surface * surface)
     static SDL_Rect title_rect = { 100, 64, 0, 0 };
     struct LCache *lnames = weaponsel_levelcache;
     SDL_Rect rect, rect2;
+    Uint32 cursorcol;
     char str[256];
     Uint32 colors[4];
     int r;
-#ifndef HAVE_LIBSDL_GFX
-    SDL_Rect verticalb, horizontalb;
-    Uint32 barcol, cursorcol;
-    barcol = SDL_MapRGB (surface->format, 220, 220, 220);
-    cursorcol = SDL_MapRGB (surface->format, 211, 165, 255);
-    verticalb.x = 32;
-    verticalb.y = 0;
-    verticalb.w = 32;
-    verticalb.h = surface->h;
-    horizontalb.x = 0;
-    horizontalb.y = 32;
-    horizontalb.w = surface->w;
-    horizontalb.h = 64;
-    colors[0] = SDL_MapRGB (surface->format, 200, 0, 0);
-    colors[1] = SDL_MapRGB (surface->format, 0, 0, 200);
-    colors[2] = SDL_MapRGB (surface->format, 0, 200, 0);
-    colors[3] = SDL_MapRGB (surface->format, 200, 200, 0);
-#else
-    colors[0] = 0xC80000C8;
-    colors[1] = 0x0000C8C8;
-    colors[2] = 0x00C800C8;
-    colors[3] = 0xC8C800C8;
-#endif
+    cursorcol = map_rgba(211,165,255,200);
+    colors[0] = map_rgba(200,0,0,200);
+    colors[1] = map_rgba(0,0,200,200);
+    colors[2] = map_rgba(0,200,0,200);
+    colors[3] = map_rgba(200,200,0,200);
+
     /* Draw common background */
     SDL_FillRect (surface, NULL, 0);
-#if HAVE_LIBSDL_GFX
-    boxColor (surface, 32, 0, 64, surface->h, 0xFFFFFFC8);
-    boxColor (surface, 0, 32, surface->w, 96, 0xFFFFFFC8);
-#else
-    SDL_FillRect (surface, &verticalb, barcol);
-    SDL_FillRect (surface, &horizontalb, barcol);
-#endif
+    fill_box(surface,32,0,32,surface->h,map_rgba(255,255,255,220));
+    fill_box(surface,0,32,surface->w,64,map_rgba(255,255,255,220));
+
     /* Level selection mode */
-    if (hotseat_weaponsel_mode == 0) {
+    if (hotseat_weaponsel_mode == LEVELSEL) {
         if (!weaponsel_title) {
             sprintf (str, "Round %d of %d", game_status.total_rounds + 1,
                      game_settings.rounds + game_status.total_rounds);
@@ -555,18 +553,8 @@ static void draw_hotseat_weaponsel (SDL_Surface * surface)
         r = 0;
         while (lnames) {
             SDL_Rect namerect;
-            if (r == 2) {
-#ifndef HAVE_LIBSDL_GFX
-                SDL_Rect cr;
-                cr.x = 48;
-                cr.y = rect.y;
-                cr.w = surface->w - cr.x;
-                cr.h = rect.h;
-                SDL_FillRect (surface, &cr, cursorcol);
-#else
-                boxColor (surface, 48, rect.y, surface->w, rect.y + rect.h,
-                          0xD3A5FFC8);
-#endif
+            if (r == CURSORPOS) {
+                fill_box(surface,48,rect.y,surface->w-48,rect.h,cursorcol);
             }
             namerect = rect;
             if (lnames->icon) {
@@ -611,15 +599,8 @@ static void draw_hotseat_weaponsel (SDL_Surface * surface)
         for (r = 0; r < 4; r++) {
             if (game_settings.players_in[r] != ' ') {
                 SDL_Rect r1, r2;
-#if HAVE_LIBSDL_GFX
-                boxColor (surface, rect.x, rect.y, rect.x + rect.w,
-                          rect.y + rect.h, colors[r]);
-                boxColor (surface, rect2.x, rect2.y, rect2.x + rect2.w,
-                          rect2.y + rect2.h, colors[r]);
-#else
-                SDL_FillRect (surface, &rect, colors[r]);
-                SDL_FillRect (surface, &rect2, colors[r]);
-#endif
+                fill_box(surface,rect.x, rect.y, rect.w, rect.h, colors[r]);
+                fill_box(surface,rect2.x, rect2.y, rect2.w, rect2.h, colors[r]);
                 if (!weaponsel_weapons[0][r])
                     weaponsel_weapons[0][r] =
                         renderstring (Bigfont,
@@ -645,18 +626,19 @@ static void draw_hotseat_weaponsel (SDL_Surface * surface)
     SDL_BlitSurface (weaponsel_title, NULL, surface, &title_rect);
     if (game_status.lastwin > 0) {
         SDL_Rect trophy_rect;
-        trophy_rect.x = 520;
+        trophy_rect.x = screen->w - trophy_gfx[game_status.lastwin-1]->w - 20;
         trophy_rect.y = 16;
         SDL_BlitSurface (trophy_gfx[game_status.lastwin - 1], NULL, surface,
                          &trophy_rect);
     }
 }
 
-/* Weapon selection screen eventloop */
-static Uint8 hotseat_weaponsel_eventloop ()
+/* Weapon/level selection screen eventloop */
+static Uint8 hotseat_weaponsel_eventloop (int levelcount)
 {
     SDL_Event Event;
     Uint8 p, oldmode, needupdate;
+    int cursor;
     while (1) {
         if (!SDL_WaitEvent (&Event)) {
             printf ("Error occured while waiting for an event: %s\n",
@@ -669,40 +651,47 @@ static Uint8 hotseat_weaponsel_eventloop ()
         case SDL_KEYDOWN:
             if (Event.key.keysym.sym == SDLK_F11)
                 screenshot ();
-            else if (hotseat_weaponsel_mode == 0) {     /* Level selection mode */
+            else if (hotseat_weaponsel_mode == LEVELSEL) {
                 if (Event.key.keysym.sym == SDLK_ESCAPE)
                     return 1;
                 else if (Event.key.keysym.sym == SDLK_RETURN
                          || Event.key.keysym.sym == SDLK_KP_ENTER) {
                     if((Event.key.keysym.mod & (KMOD_LALT|KMOD_RALT))) {
-                        SDL_WM_ToggleFullScreen (screen);
+                        toggle_fullscreen();
                     } else {
+                        playwave (WAV_BLIP2);
                         if(game_settings.playmode==RndWeapon)
                             return 0;
                         else
-                            hotseat_weaponsel_mode++;
+                            hotseat_weaponsel_mode=WEAPONSEL;
                     }
                 }
-                else if (Event.key.keysym.sym == SDLK_LEFT || Event.key.keysym.sym == SDLK_UP) {        /* Level-- */
+                else if (Event.key.keysym.sym == SDLK_LEFT ||
+                        Event.key.keysym.sym == SDLK_UP) { /* Level-- */
                     struct LCache *lcache = weaponsel_levelcache;
+                    struct LCache *tmpcache;
                     struct dllist *lev;
+                    playwave (WAV_BLIP);
+                    /* Set level pointer to previous level */
                     if (game_settings.levels->prev == NULL)
                         game_settings.levels = game_settings.last_level;
                     else
                         game_settings.levels = game_settings.levels->prev;
-                    lev = game_settings.levels->prev;
-                    if (lev == NULL)
-                        lev = game_settings.last_level;
-                    lev = lev->prev;
-                    if (lev == NULL)
-                        lev = game_settings.last_level;
+                    lev = game_settings.levels;
+                    /* Get the level on the top of the list */
+                    for(cursor=CURSORPOS;cursor>0;cursor--) {
+                        lev = lev->prev;
+                        if (lev == NULL)
+                            lev = game_settings.last_level;
+                    }
+                    /* Update level cache */
                     while (lcache->next->next)
                         lcache = lcache->next;
                     SDL_FreeSurface (lcache->next->name);
-                    free (lcache->next);
+                    tmpcache = lcache->next;
                     lcache->next = NULL;
                     lcache = weaponsel_levelcache;
-                    weaponsel_levelcache = malloc (sizeof (struct LCache));
+                    weaponsel_levelcache = tmpcache;
                     weaponsel_levelcache->next = lcache;
                     weaponsel_levelcache->icon =
                         ((struct LevelFile*)lev->data)->settings->icon;
@@ -712,25 +701,31 @@ static Uint8 hotseat_weaponsel_eventloop ()
                             font_color_white);
                     needupdate = 1;
 
-                } else if (Event.key.keysym.sym == SDLK_RIGHT || Event.key.keysym.sym == SDLK_DOWN) {   /* Level++ */
+                } else if (Event.key.keysym.sym == SDLK_RIGHT ||
+                        Event.key.keysym.sym == SDLK_DOWN) { /* Level++ */
                     struct LCache *lcache = weaponsel_levelcache->next;
+                    struct LCache *tmpcache;
                     struct dllist *lev;
+                    playwave (WAV_BLIP);
+                    /* Set level pointer to next level */
                     if (game_settings.levels->next == NULL)
                         game_settings.levels = game_settings.first_level;
                     else
                         game_settings.levels = game_settings.levels->next;
-                    lev = game_settings.levels->next;
-                    if (lev == NULL)
-                        lev = game_settings.first_level;
-                    lev = lev->next;
-                    if (lev == NULL)
-                        lev = game_settings.first_level;
+                    /* Get the level on the bottom of the list */
+                    lev = game_settings.levels;
+                    for(cursor=CURSORPOS;cursor<levelcount-1;cursor++) {
+                        lev = lev->next;
+                        if (lev == NULL)
+                            lev = game_settings.first_level;
+                    }
+                    /* Update level cache */
                     SDL_FreeSurface (weaponsel_levelcache->name);
-                    free (weaponsel_levelcache);
+                    tmpcache = weaponsel_levelcache;
                     weaponsel_levelcache = lcache;
                     while (lcache->next)
                         lcache = lcache->next;
-                    lcache->next = malloc (sizeof (struct LCache));
+                    lcache->next = tmpcache;
                     lcache = lcache->next;
                     lcache->next = NULL;
                     lcache->icon = ((struct LevelFile*)lev->data)->settings->icon;
@@ -740,32 +735,43 @@ static Uint8 hotseat_weaponsel_eventloop ()
                                 font_color_white);
                     needupdate = 1;
                 }
-            } else {            /* Weapon selection mode */
+            } else {
+                /* Weapon selection mode */
                 if (Event.key.keysym.sym == SDLK_ESCAPE)
-                    hotseat_weaponsel_mode = 0;
-                else if (Event.key.keysym.sym == SDLK_RETURN
-                         || Event.key.keysym.sym == SDLK_KP_ENTER)
-                    return 0;
+                    hotseat_weaponsel_mode = LEVELSEL;
+                else if (Event.key.keysym.sym == SDLK_RETURN ||
+                        Event.key.keysym.sym == SDLK_KP_ENTER) {
+                    if((Event.key.keysym.mod & (KMOD_LALT|KMOD_RALT))) {
+                        toggle_fullscreen();
+                    } else {
+                        return 0;
+                    }
+                }
                 for (p = 0; p < 4; p++) {
+                    if (game_settings.players_in[p] == ' ') continue;
                     if (Event.key.keysym.sym == game_settings.buttons[p][2]) {
                         players[p].specialWeapon--;
                         SDL_FreeSurface (weaponsel_weapons[1][p]);
                         weaponsel_weapons[1][p] = 0;
+                        playwave (WAV_BLIP);
                     } else if (Event.key.keysym.sym ==
                                game_settings.buttons[p][3]) {
                         players[p].specialWeapon++;
                         SDL_FreeSurface (weaponsel_weapons[1][p]);
                         weaponsel_weapons[1][p] = 0;
+                        playwave (WAV_BLIP);
                     } else if (Event.key.keysym.sym ==
                                game_settings.buttons[p][1]) {
                         players[p].standardWeapon--;
                         SDL_FreeSurface (weaponsel_weapons[0][p]);
                         weaponsel_weapons[0][p] = 0;
+                        playwave (WAV_BLIP);
                     } else if (Event.key.keysym.sym ==
                                game_settings.buttons[p][0]) {
                         players[p].standardWeapon++;
                         SDL_FreeSurface (weaponsel_weapons[0][p]);
                         weaponsel_weapons[0][p] = 0;
+                        playwave (WAV_BLIP);
                     }
                     if (players[p].specialWeapon < 1)
                         players[p].specialWeapon = WeaponCount - 1;
@@ -793,7 +799,7 @@ static Uint8 hotseat_weaponsel_eventloop ()
             weaponsel_title = NULL;
             SDL_FreeSurface (weaponsel_prompt);
             weaponsel_prompt = NULL;
-            if (hotseat_weaponsel_mode == 0) {
+            if (hotseat_weaponsel_mode == LEVELSEL) {
                 int r;
                 for (r = 0; r < 4; r++) {
                     if (weaponsel_weapons[0][r]) {
@@ -815,3 +821,4 @@ static Uint8 hotseat_weaponsel_eventloop ()
     }
     return 0;
 }
+
