@@ -37,9 +37,7 @@
 #include "startup.h"
 #include "demo.h"
 
-#if HAVE_LIBSDL_MIXER
 #include "audio.h"
-#endif
 
 #include "number.h"
 #define MENU_ID_TOPLEVEL		0x00
@@ -65,6 +63,7 @@
 #define MENU_ITEM_ID_JUMPLIFE		0xa0
 #define MENU_ITEM_ID_HOLESIZE		0xa1
 #define MENU_ITEM_ID_MUSVOL		0xa1
+#define MENU_ITEM_ID_SNDVOL		0xa2
 #define MENU_ITEM_ID_RETURN		0xFF
 
 /* Internally used globals */
@@ -91,11 +90,12 @@ static void draw_intro_screen (void);
 static void intro_draw_message (void);
 
 /* Functions called by menu callbacks */
-void draw_logo (int menu_id);
-Uint8 set_player_key (MenuCommand cmd, int menu_id, MenuItem * item);
-Uint8 save_settings (MenuCommand cmd, int menu_id, MenuItem * item);
-Uint8 save_startup_settings (MenuCommand cmd, int menu_id, MenuItem * item);
-char *intro_label_function (int mid, MenuItem * item);
+static void draw_logo (int menu_id);
+static Uint8 update_sndvolume (MenuCommand cmd, int menu_id, MenuItem * item);
+static Uint8 set_player_key (MenuCommand cmd, int menu_id, MenuItem * item);
+static Uint8 save_settings (MenuCommand cmd, int menu_id, MenuItem * item);
+static Uint8 save_startup_settings (MenuCommand cmd, int menu_id, MenuItem * item);
+static char *intro_label_function (int mid, MenuItem * item);
 
 
 /* Initialize. Load menus and such */
@@ -103,6 +103,9 @@ int init_intro (void) {
     Menu *menulist[13];
     MenuDrawingOptions mainmenu, optionsmenu;
     LDAT *datafile;
+    int logoy=10;
+    int yoffset=250;
+    int optionsy=80;
     int p;
     datafile =
         ldat_open_file (getfullpath (GFX_DIRECTORY, "misc.ldat"));
@@ -112,36 +115,50 @@ int init_intro (void) {
         keyb_icon[p] = load_image_ldat (datafile, 1, 2, "KEYBOARD", 0);
         pad_icon[p] = load_image_ldat (datafile, 1, 2, "PAD", 0);
     }
-    if (keyb_icon[0])
+    if (keyb_icon[0]) {
         recolor (keyb_icon[0], 0.5, 0.5, 0.6, 1);
-    if (pad_icon[0])
-        recolor (pad_icon[0], 0.5, 0.5, 0.6, 1);
-    if (keyb_icon[1])
         recolor (keyb_icon[1], 0.6, 0.5, 0.5, 1);
-    if (pad_icon[1])
-        recolor (pad_icon[1], 0.6, 0.5, 0.5, 1);
-    if (keyb_icon[2])
         recolor (keyb_icon[2], 0.5, 0.6, 0.5, 1);
-    if (pad_icon[2])
-        recolor (pad_icon[2], 0.5, 0.6, 0.5, 1);
-    if (keyb_icon[3])
         recolor (keyb_icon[3], 0.8, 0.8, 0.5, 1);
-    if (pad_icon[3])
+    }
+    if (pad_icon[0]) {
+        recolor (pad_icon[0], 0.5, 0.5, 0.6, 1);
+        recolor (pad_icon[1], 0.6, 0.5, 0.5, 1);
+        recolor (pad_icon[2], 0.5, 0.6, 0.5, 1);
         recolor (pad_icon[3], 0.8, 0.8, 0.5, 1);
+    }
+    switch(luola_options.videomode) {
+        case VID_640:
+            yoffset = 250;
+            logoy = 10;
+            optionsy = 80;
+            break;
+        case VID_800:
+            yoffset = 280;
+            logoy = 40;
+            optionsy = 120;
+            break;
+        case VID_1024:
+            yoffset = 380;
+            logoy = 90;
+            optionsy = 220;
+            break;
+    }
+
     /* Menu drawing options */
-    mainmenu = make_menu_drawingoptions (0, 250, SCREEN_W, SCREEN_H);
+    mainmenu = make_menu_drawingoptions (0, yoffset, screen->w, screen->h);
     mainmenu.spacing = MENU_SPACING;
     mainmenu.selection_color = SDL_MapRGB (screen->format, 0, 128, 255);
-    mainmenu.left_offset = 240;
-    optionsmenu = make_menu_drawingoptions (0, 80, SCREEN_W, SCREEN_H);
+    mainmenu.left_offset = screen->h/2;
+    optionsmenu = make_menu_drawingoptions (0, optionsy, screen->w, screen->h);
     optionsmenu.spacing = mainmenu.spacing;
     optionsmenu.selection_color = mainmenu.selection_color;
     optionsmenu.left_offset = 130;
     /* Load graphics */
     intr_logo = load_image_ldat (datafile, 1, 2, "LOGO", 0);
     if (intr_logo) {
-        intr_logo_rect.x = SCREEN_W / 2 - intr_logo->w / 2;
-        intr_logo_rect.y = 10;
+        intr_logo_rect.x = screen->w / 2 - intr_logo->w / 2;
+        intr_logo_rect.y = logoy;
         centered_string (intr_logo, Smallfont, intr_logo->h - 20, VERSION,
                          font_color_red);
     }
@@ -241,9 +258,15 @@ int init_intro (void) {
                                MNU_ITEM_TOGGLE, 6, MNU_OPT_LABEL,
                                "Ship recall is %s", MNU_OPT_VALUE_TYPE,
                                MNU_TYP_INT, MNU_OPT_VALUE,
-                               &game_settings.recall, 0, MNU_ITEM_RETURN,
-                               MENU_ITEM_ID_RETURN, MNU_OPT_LABEL, "Ok",
-                               MNU_OPT_COLOR, font_color_green, 0, 0);
+                               &game_settings.recall, 0,
+                               MNU_ITEM_TOGGLE, 7, MNU_OPT_LABEL,
+                               "Player screen size: %s", MNU_OPT_VALUE_TYPE,
+                               MNU_TYP_INT, MNU_OPT_VALUE,
+                               &game_settings.bigscreens, MNU_OPT_TOGGLE_TEXTS,
+                               "maximum", "quarter", 0,
+                               MNU_ITEM_RETURN, MENU_ITEM_ID_RETURN,
+                               MNU_OPT_LABEL, "Ok", MNU_OPT_COLOR,
+                               font_color_green, 0, 0);
     /* Create weapon menu */
     menulist[4] = create_menu (MENU_ID_WEAPON,
                                MNU_ITEM_SEP, 0, MNU_OPT_LABEL,
@@ -261,6 +284,11 @@ int init_intro (void) {
                                MNU_OPT_VALUE, &game_settings.wind_bullets,
                                MNU_OPT_TOGGLE_TEXTS, "are", "are not",
                                0, MNU_ITEM_TOGGLE, 3, MNU_OPT_LABEL,
+                               "Projectiles are drawn %s",
+                               MNU_OPT_VALUE_TYPE, MNU_TYP_INT,
+                               MNU_OPT_VALUE, &game_settings.large_bullets,
+                               MNU_OPT_TOGGLE_TEXTS, "large", "normal",
+                               0, MNU_ITEM_TOGGLE, 3, MNU_OPT_LABEL,
                                "Ingame weapon switching is %s",
                                MNU_OPT_VALUE_TYPE, MNU_TYP_INT,
                                MNU_OPT_VALUE, &game_settings.weapon_switch, 0,
@@ -372,28 +400,45 @@ int init_intro (void) {
                                MNU_OPT_COLOR, font_color_green, 0, 0);
     /* Build audio menu */
     menulist[7] = create_menu (MENU_ID_AUDIO,
-                               MNU_ITEM_SEP, 0, MNU_OPT_LABEL,
-                               "Audio settings", MNU_OPT_ALIGN,
-                               MNU_ALIGN_CENTER, MNU_OPT_COLOR,
-                               font_color_gray, 0, MNU_ITEM_SEP, 0, 0,
-                               MNU_ITEM_TOGGLE, 1, MNU_OPT_LABEL,
-                               "Sounds are %s", MNU_OPT_VALUE_TYPE,
-                               MNU_TYP_INT, MNU_OPT_VALUE,
-                               &game_settings.sounds, 0, MNU_ITEM_TOGGLE, 2,
+                               MNU_ITEM_SEP, 0,
+                               MNU_OPT_LABEL, "Audio settings",
+                               MNU_OPT_ALIGN,MNU_ALIGN_CENTER,
+                               MNU_OPT_COLOR, font_color_gray, 0,
+
+                               MNU_ITEM_SEP, 0, 0,
+
+                               MNU_ITEM_TOGGLE, 1,
+                               MNU_OPT_LABEL,"Sounds are %s",
+                               MNU_OPT_VALUE_TYPE, MNU_TYP_INT,
+                               MNU_OPT_VALUE, &game_settings.sounds, 0,
+
+                               MNU_ITEM_TOGGLE, 2,
                                MNU_OPT_LABEL, "Music is %s",
                                MNU_OPT_VALUE_TYPE, MNU_TYP_INT,
                                MNU_OPT_VALUE, &game_settings.music, 0,
+
                                MNU_ITEM_TOGGLE, 3, MNU_OPT_LABEL,
                                "Playlist is %s", MNU_OPT_VALUE_TYPE,
                                MNU_TYP_INT, MNU_OPT_VALUE,
                                &game_settings.playlist, MNU_OPT_TOGGLE_TEXTS,
-                               "shuffled", "ordered", 0, MNU_ITEM_VALUE,
-                               MENU_ITEM_ID_MUSVOL, MNU_OPT_LABELF,
-                               intro_label_function, MNU_OPT_VALUE_TYPE,
-                               MNU_TYP_INT, MNU_OPT_VALUE,
-                               &game_settings.music_vol, MNU_OPT_MINVALUE, 0,
-                               MNU_OPT_MAXVALUE, 128, MNU_OPT_INCREMENT, 10,
-                               0, MNU_ITEM_RETURN, MENU_ITEM_ID_RETURN,
+                               "shuffled", "ordered", 0,
+
+                               MNU_ITEM_VALUE, MENU_ITEM_ID_SNDVOL,
+                               MNU_OPT_LABELF,intro_label_function,
+                               MNU_OPT_VALUE_TYPE,MNU_TYP_INT,
+                               MNU_OPT_VALUE,&game_settings.sound_vol,
+                               MNU_OPT_MINVALUE, 0,MNU_OPT_MAXVALUE, 128,
+                               MNU_OPT_ACTION,update_sndvolume,
+                               MNU_OPT_INCREMENT, 10, 0,
+
+                               MNU_ITEM_VALUE, MENU_ITEM_ID_MUSVOL,
+                               MNU_OPT_LABELF,intro_label_function,
+                               MNU_OPT_VALUE_TYPE,MNU_TYP_INT,
+                               MNU_OPT_VALUE,&game_settings.music_vol,
+                               MNU_OPT_MINVALUE, 0,MNU_OPT_MAXVALUE, 128,
+                               MNU_OPT_INCREMENT, 10, 0,
+
+                               MNU_ITEM_RETURN, MENU_ITEM_ID_RETURN,
                                MNU_OPT_LABEL, "Ok", MNU_OPT_COLOR,
                                font_color_green, 0, 0);
     /* Create player input settings menus */
@@ -468,6 +513,11 @@ int init_intro (void) {
                                 "Menu background animations are %s",
                                 MNU_OPT_VALUE_TYPE, MNU_TYP_INT,
                                 MNU_OPT_VALUE, &luola_options.mbg_anim, 0,
+                                MNU_ITEM_VALUE, 6,
+                                MNU_OPT_LABELF, intro_label_function,
+                                MNU_OPT_VALUE_TYPE, MNU_TYP_INT,
+                                MNU_OPT_VALUE, &luola_options.videomode,
+                                MNU_OPT_MINVALUE, 0, MNU_OPT_MAXVALUE, 2, 0,
                                 MNU_ITEM_SEP, 0, MNU_OPT_LABEL, "- - -", 0,
                                 MNU_ITEM_LABEL, 10, MNU_OPT_LABEL,
                                 "Save startup settings", MNU_OPT_COLOR,
@@ -483,13 +533,8 @@ int init_intro (void) {
     intr_message.show = 0;
     intr_message.text = NULL;
     intr_message.setkey = NULL;
-#if HAVE_LIBSDL_GFX
-    intr_message.framecolor = 0xC85050DC;
-    intr_message.fillcolor = 0x000000DC;
-#else
-    intr_message.framecolor = SDL_MapRGB (screen->format, 200, 80, 80);
-    intr_message.fillcolor = SDL_MapRGB (screen->format, 0, 0, 0);
-#endif
+    intr_message.framecolor = map_rgba(200,80,80,220);
+    intr_message.fillcolor = map_rgba(0,0,0,220);
     ldat_free (datafile);
     return 0;
 }
@@ -498,7 +543,6 @@ int init_intro (void) {
 int game_menu_screen (void) {
     draw_intro_screen ();
     return intro_event_loop ();
-    return 0;
 }
 
 /* Internal. */
@@ -546,7 +590,7 @@ static Uint8 intro_event_loop (void) {
                         command = MenuRight;
                     else if (Event.key.keysym.sym == SDLK_RETURN) {
                         if((Event.key.keysym.mod & (KMOD_LALT|KMOD_RALT)))
-                            SDL_WM_ToggleFullScreen(screen);
+                            toggle_fullscreen();
                         else
                             command = MenuEnter;
                     }
@@ -554,13 +598,11 @@ static Uint8 intro_event_loop (void) {
                         command = MenuESC;
                 }
                 if (command >= 0) {
-#if HAVE_LIBSDL_MIXER
                     if (command != MenuEnter && command != MenuESC
                         && command != MenuToggleLock)
                         playwave (WAV_BLIP);
                     else
                         playwave (WAV_BLIP2);
-#endif
                     rval = menu_control (intro_menu, command);
                     if (rval == INTRO_RVAL_STARTGAME
                         || rval == INTRO_RVAL_EXIT)
@@ -613,7 +655,7 @@ void draw_intro_screen (void)
 void draw_logo (int menu_id)
 {
     if (menu_id != MENU_ID_TOPLEVEL) {
-        printf ("Warning ! draw_logo(%d) was called from a wrong menu 1\n",
+        printf ("Warning! draw_logo(%d) was called from a wrong menu\n",
                 (int) menu_id);
         return;
     }
@@ -627,7 +669,7 @@ Uint8 save_settings (MenuCommand cmd, int menu_id, MenuItem * item)
 {
     if (menu_id != MENU_ID_SETTINGS) {
         printf
-            ("Warning ! save_settings(%d,%d,%x) was not called from %d !\n",
+            ("Warning! save_settings(%d,%d,%x) was not called from %d\n",
              cmd, menu_id, MENU_ID_SETTINGS, (int) item);
         return 0;
     }
@@ -643,7 +685,7 @@ Uint8 save_settings (MenuCommand cmd, int menu_id, MenuItem * item)
 Uint8 save_startup_settings (MenuCommand cmd, int menu_id, MenuItem * item)
 {
     if (menu_id != MENU_ID_STARTUP) {
-        printf ("save_startup_settings(%d,%d,%x) was not called from %d !\n",
+        printf ("save_startup_settings(%d,%d,%x) was not called from %d\n",
                 cmd, menu_id, MENU_ID_STARTUP, (int) item);
         return 0;
     }
@@ -670,10 +712,14 @@ char *intro_label_function (int id, MenuItem * item)
         }
     } else if (id == MENU_ID_AUDIO) {
         switch (item->ID) {
-        case MENU_ITEM_ID_MUSVOL:
-            sprintf (buf, "Music volume: %d%%",
-                     (int) ((game_settings.music_vol / 128.0) * 100.0));
-            break;
+            case MENU_ITEM_ID_SNDVOL:
+                sprintf(buf, "Effect volume: %d%%",
+                        (int) ((game_settings.sound_vol / 128.0) * 100.0));
+                break;
+            case MENU_ITEM_ID_MUSVOL:
+                sprintf (buf, "Music volume: %d%%",
+                         (int) ((game_settings.music_vol / 128.0) * 100.0));
+                break;
         }
     } else if (id == MENU_ID_WEAPON) {
         char sizestr[32] = "foo";
@@ -693,6 +739,14 @@ char *intro_label_function (int id, MenuItem * item)
         }
         sprintf (buf, "Explosion hole size: %s", sizestr);
         set_hole_size (game_settings.holesize);
+    } else if (id == MENU_ID_STARTUP) {
+        char vidstr[32] = "foo";
+        switch(luola_options.videomode) {
+            case VID_640: strcpy(vidstr,"640x480"); break;
+            case VID_800: strcpy(vidstr,"800x600"); break;
+            case VID_1024: strcpy(vidstr,"1024x768"); break;
+        }
+        sprintf (buf, "Video mode: %s", vidstr);
     } else if (id >= MENU_ID_INPUT1 && id <= MENU_ID_INPUT4) {
         Uint8 p;
         p = id - MENU_ID_INPUT1;
@@ -749,7 +803,7 @@ int draw_input_icon (int x, int y, Uint8 align, int mid, MenuItem * item)
         id--;
     }
     if (id > 3) {
-        printf ("Warning ! Didn't find the icon for menu %d item %d !\n", mid,
+        printf ("Warning! Didn't find the icon for menu %d item %d\n", mid,
                 id);
         return 0;
     }
@@ -793,7 +847,7 @@ int draw_team_icon (int x, int y, Uint8 align, int mid, MenuItem * item)
         id--;
     }
     if (id > 3) {
-        printf ("Warning ! Didn't find the icon for menu %d item %d !\n", mid,
+        printf ("Warning! Didn't find the icon for menu %d item %d\n", mid,
                 id);
         return 0;
     }
@@ -813,6 +867,14 @@ int draw_team_icon (int x, int y, Uint8 align, int mid, MenuItem * item)
     return 12;
 }
 
+/* Called by a menu callback */
+/* Set sound volume */
+Uint8 update_sndvolume (MenuCommand cmd, int menu_id, MenuItem * item)
+{
+    audio_setsndvolume(game_settings.sound_vol);
+    return 1;
+}
+
 /* Called by a menu callback */
 /* Ask a new key */
 Uint8 set_player_key (MenuCommand cmd, int menu_id, MenuItem * item)
@@ -838,8 +900,8 @@ static void intro_message (char *msg)
     if (intr_message.text)
         SDL_FreeSurface (intr_message.text);
     intr_message.text = renderstring (Bigfont, msg, font_color_white);
-    intr_message.x = SCREEN_W / 2 - intr_message.text->w / 2 - 25;
-    intr_message.y = SCREEN_H / 2 - intr_message.text->h / 2 - 10;
+    intr_message.x = screen->w / 2 - intr_message.text->w / 2 - 25;
+    intr_message.y = screen->h / 2 - intr_message.text->h / 2 - 10;
     intr_message.w = intr_message.text->w + 50;
     intr_message.h = intr_message.text->h + 20;
     intr_message.textrect.x =
@@ -852,18 +914,8 @@ static void intro_message (char *msg)
 /* Draw the intro messagebox */
 static void intro_draw_message (void)
 {
-#if HAVE_LIBSDL_GFX
-    boxColor (screen, intr_message.x + 2, intr_message.y + 2,
-              intr_message.x + intr_message.w - 2,
-              intr_message.y + intr_message.h - 2, intr_message.fillcolor);
-#else
-    SDL_Rect rect;
-    rect.x = intr_message.x + 2;
-    rect.y = intr_message.y + 2;
-    rect.w = intr_message.w - 4;
-    rect.h = intr_message.h - 4;
-    SDL_FillRect (screen, &rect, intr_message.fillcolor);
-#endif
+    fill_box(screen,intr_message.x+2,intr_message.y+2,intr_message.w-4,intr_message.h-4,intr_message.fillcolor);
+
     draw_box (intr_message.x, intr_message.y, intr_message.w, intr_message.h,
               2, intr_message.framecolor);
     SDL_BlitSurface (intr_message.text, NULL, screen, &intr_message.textrect);
