@@ -23,11 +23,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <SDL_image.h>
+
+#include "SDL_image.h"
+#include "SDL_endian.h"
 
 #include "stringutil.h"
 #include "lconf.h"
-#include "game.h"               /* Use game_settings.ls to get default values */
 #include "sbmp.h"
 
 /* Internally used functions */
@@ -45,10 +46,10 @@ static int readObjectsBlockRW (LevelSettings * settings, SDL_RWops * rw,
                                int len);
 static int readPaletteBlockRW (LevelSettings * settings, SDL_RWops * rw,
                                int len);
-static int readIconRW (LevelInfo * info, SDL_RWops * rw, int len);
+static int readIconRW (LevelSettings * info, SDL_RWops * rw, int len);
 
 /* Load the configuration file from file */
-LevelSettings *load_level_config (const char *filename, int loadextra)
+LevelSettings *load_level_config (const char *filename)
 {
     FILE *fp;
     char *line;
@@ -59,7 +60,7 @@ LevelSettings *load_level_config (const char *filename, int loadextra)
     /* Open file */
     fp = fopen (filename, "r");
     if (!fp) {
-        printf ("Error ! Could not open configuration file \"%s\" !\n",
+        printf ("Error! Could not open configuration file \"%s\"\n",
                 filename);
         exit (1);
     }
@@ -75,7 +76,7 @@ LevelSettings *load_level_config (const char *filename, int loadextra)
             settings->objects = readObjectsBlock (fp);
         else if (strcmp (line, "[palette]") == 0)
             settings->palette = readPaletteBlock (fp);
-        else if (strcmp (line, "[icon]") == 0 && loadextra)
+        else if (strcmp (line, "[icon]") == 0)
             settings->icon = readIconBlock (fp);
         free (line);
     }
@@ -90,7 +91,7 @@ LevelSettings *load_level_config_rw (SDL_RWops * rw, int len)
     LevelSettings *settings;
     int read = 0;
     Uint16 l;
-    Uint8 type;
+    Uint8 type,ver;
     /* Initialize */
     settings = malloc (sizeof (LevelSettings));
     memset (settings, 0, sizeof (LevelSettings));
@@ -98,26 +99,34 @@ LevelSettings *load_level_config_rw (SDL_RWops * rw, int len)
     while (read < len) {
         read += SDL_RWread (rw, &l, 2, 1) * 2;
         read += SDL_RWread (rw, &type, 1, 1);
+        l = SDL_SwapLE16(l);
         switch (type) {
         case 0x01:
-            read += readMainBlockRW (settings, rw, l);
+            read += SDL_RWread(rw,&ver,1,1);
+            if(ver!=LCONF_REVISION) {
+                fprintf(stderr,"Error: incompatible level file\n");
+                free(settings);
+                return NULL;
+            }
             break;
         case 0x02:
-            read += readOverrideBlockRW (settings, rw, l);
+            read += readMainBlockRW (settings, rw, l);
             break;
         case 0x03:
-            read += readObjectsBlockRW (settings, rw, l);
+            read += readOverrideBlockRW (settings, rw, l);
             break;
         case 0x04:
-            read += readPaletteBlockRW (settings, rw, l);
+            read += readObjectsBlockRW (settings, rw, l);
             break;
         case 0x05:
-            read += l;
-            SDL_RWseek (rw, l, SEEK_CUR);
-            break;              /* We (currently) don't need to load the icon here */
+            read += readPaletteBlockRW (settings, rw, l);
+            break;
+        case 0x06:
+            read += readIconRW(settings,rw,l);
+            break;
         default:
-            printf
-                ("Warning: Unrecognized toplevel block 0x%x ! (skipping %d bytes...)\n",
+            fprintf(stderr,
+                "Warning: Unrecognized toplevel block 0x%x ! (skipping %d bytes...)\n",
                  type, l);
             SDL_RWseek (rw, l, SEEK_CUR);
             read += l;
@@ -125,8 +134,8 @@ LevelSettings *load_level_config_rw (SDL_RWops * rw, int len)
         }
     }
     if (read > len)
-        printf
-            ("Warning: read %d bytes past the limit in load_level_config_rw() !\n",
+        fprintf(stderr,
+            "Warning: read %d bytes past the limit in load_level_config_rw()!\n",
              read - len);
     /* Finished */
     return settings;
@@ -140,18 +149,18 @@ static SDL_Surface *readIconBlock (FILE * fp)
     rw = SDL_RWFromFP (fp, 0);
     icon = IMG_LoadTyped_RW (rw, 0, "XPM");
     if (icon == NULL)
-        printf ("Error occured while tried to read level icon:\n%s\n",
+        fprintf (stderr,"Error occured while tried to read level icon:\n%s\n",
                 SDL_GetError ());
     SDL_FreeRW (rw);
     return icon;
 }
 
 /* Read the icon from a binary level settings file */
-static int readIconRW (LevelInfo * info, SDL_RWops * rw, int len)
+static int readIconRW (LevelSettings *settings, SDL_RWops * rw, int len)
 {
     Uint32 colorkey;
     Uint8 hascolorkey;
-    char *data;
+    Uint8 *data;
     int read;
     data = malloc (len);
     read = SDL_RWread (rw, &hascolorkey, 1, 1);
@@ -160,83 +169,25 @@ static int readIconRW (LevelInfo * info, SDL_RWops * rw, int len)
     len -= read;
     if (len)
         SDL_RWseek (rw, len, SEEK_CUR);
-    info->icon = sbmp_to_surface (data);
+    settings->icon = sbmp_to_surface (data);
     if (hascolorkey)
-        SDL_SetColorKey (info->icon, SDL_SRCCOLORKEY, colorkey);
+        SDL_SetColorKey (settings->icon, SDL_SRCCOLORKEY, SDL_SwapLE32(colorkey));
     free (data);
     return read + len;
 }
 
-
-/* Load just the main block and icon from a text mode configuration file */
-LevelInfo *quickload_levelsettings (const char *filename)
-{
-    LevelInfo *info;
-    FILE *fp;
-    char *line;
-    fp = fopen (filename, "r");
-    if (!fp) {
-        printf ("Error ! Could not open configuration file \"%s\" !\n",
-                filename);
-        exit (1);
-    }
-    info = malloc (sizeof (LevelInfo));
-    memset (info, 0, sizeof (LevelInfo));
-    /* Read */
-    while ((line = read_line (fp))) {
-        if (line[0] == '\0')
-            continue;
-        if (strcmp (line, "[main]") == 0)
-            info->mainblock = readMainBlock (fp);
-        else if (strcmp (line, "[icon]") == 0)
-            info->icon = readIconBlock (fp);
-        free (line);
-    }
-    fclose (fp);
-    return info;
-}
-
-/* Load just the main block and icon from a binary configuration file */
-LevelInfo *quickload_levelsettings_rw (SDL_RWops * rw, int len)
-{
-    LevelSettings settings;
-    LevelInfo *info;
-    Uint16 l;
-    Uint8 type;
-    int read = 0;
-    info = malloc (sizeof (LevelInfo));
-    memset (info, 0, sizeof (LevelInfo));
-    /* Read */
-    while (read < len) {
-        read += SDL_RWread (rw, &l, 2, 1) * 2;
-        read += SDL_RWread (rw, &type, 1, 1);
-        switch (type) {
-        case 0x01:
-            read += readMainBlockRW (&settings, rw, l);
-            info->mainblock = settings.mainblock;
-            break;
-        case 0x05:
-            read += readIconRW (info, rw, l);
-            break;
-        default:
-            SDL_RWseek (rw, l, SEEK_CUR);
-            read += l;
-            break;
-        }
-    }
-    /* Finished */
-    return info;
-}
-
 /* Read the main block */
 static LSB_Main *readMainBlock (FILE * fp)
 {
     LSB_Main *mblock;
     char *left, *right;
     char *line;
+    /* Initialize main block */
     mblock = malloc (sizeof (LSB_Main));
     memset (mblock, 0, sizeof (LSB_Main));
-    mblock->smooth_scale = 1;
+    mblock->aspect = 1.0;
+    mblock->zoom = 1.0;
+    /* Read main block values */
     while ((line = read_line (fp))) {
         if (line[0] == '\0')
             continue;
@@ -246,29 +197,20 @@ static LSB_Main *readMainBlock (FILE * fp)
         }
         split_string (line, '=', &left, &right);
         if (left == NULL || right == NULL) {
-            printf ("Warning: Malformed line:\n \"%s\"\n", line);
+            fprintf (stderr,"Warning: Malformed line:\n \"%s\"\n", line);
             continue;
         }
-        if (strcmp (left, "modified") == 0) {
-            if (strlen (right) != 12)
-                printf ("Warning: illegal modification date !\n");
-            else
-                strncpy (mblock->modified, right, 12);
-            free (right);
-        } else if (strcmp (left, "collisionmap") == 0)
+        if (strcmp (left, "collisionmap") == 0)
             mblock->collmap = right;
         else if (strcmp (left, "artwork") == 0)
             mblock->artwork = right;
         else if (strcmp (left, "name") == 0)
             mblock->name = right;
-        else if (strcmp (left, "smoothscale") == 0) {
-            mblock->smooth_scale = atoi (right);
+        else if (strcmp (left, "aspect") == 0) {
+            mblock->aspect= atof (right);
             free (right);
-        } else if (strcmp (left, "scalex") == 0) {
-            mblock->scalex = atoi (right);
-            free (right);
-        } else if (strcmp (left, "scaley") == 0) {
-            mblock->scaley = atoi (right);
+        } else if (strcmp (left, "zoom") == 0) {
+            mblock->zoom = atof (right);
             free (right);
         } else if (strcmp (left, "music") == 0) {
             LevelBgMusic *mentry = NULL;
@@ -284,6 +226,9 @@ static LSB_Main *readMainBlock (FILE * fp)
                     mtmp = mtmp->next;
                 mtmp->next = mentry;
             }
+        } else {
+            fprintf(stderr,"Warning: unknown option \"%s\"\n",left);
+            free(right);
         }
         free (line);
         free (left);
@@ -295,27 +240,19 @@ static LSB_Main *readMainBlock (FILE * fp)
 static int readMainBlockRW (LevelSettings * settings, SDL_RWops * rw, int len)
 {
     LSB_Main *mblock;
+    Uint32 aspect,zoom;
     int read = 0;
     Uint8 type;
+    /* Initialize mainblock */
     mblock = malloc (sizeof (LSB_Main));
     memset (mblock, 0, sizeof (LSB_Main));
-    mblock->smooth_scale = 1;
+    mblock->aspect = 1.0;
+    mblock->zoom = 1.0;
+    /* Read mainblock */
     while (read < len) {
         read += SDL_RWread (rw, &type, 1, 1);
         switch (type) {
-        case 0x01:{            /* Modification date */
-                Uint16 year;
-                Uint8 m, d, hour, min;
-                read += SDL_RWread (rw, &year, 2, 1) * 2;
-                read += SDL_RWread (rw, &m, 1, 1);
-                read += SDL_RWread (rw, &d, 1, 1);
-                read += SDL_RWread (rw, &hour, 1, 1);
-                read += SDL_RWread (rw, &min, 1, 1);
-                sprintf (mblock->modified, "%d%d%d%d%d", year, m, d, hour,
-                         min);
-            }
-            break;
-        case 0x02:{            /* Level name */
+        case 0x01:{ /* Level name */
                 char tmps[512], chr = 'a';
                 int l = 0;
                 while (chr != '\0' && l < 512) {
@@ -327,22 +264,17 @@ static int readMainBlockRW (LevelSettings * settings, SDL_RWops * rw, int len)
                 strncpy (mblock->name, tmps, l);
             }
             break;
-        case 0x03:             /* Level type (deprecated) */
-            SDL_RWseek (rw, 1, SEEK_CUR); read++;
-            printf ("Warning ! Deprecated option 0x3 \"level type\"\n");
+        case 0x02: /* Aspect ratio */
+            read += SDL_RWread (rw, &aspect, 4, 1) * 4;
+            mblock->aspect = SDL_SwapLE32(aspect)/1000.0;
             break;
-        case 0x04:             /* Horizontal scaling */
-            read += SDL_RWread (rw, &mblock->scalex, 1, 1);
-            break;
-        case 0x05:             /* Vertical scaling */
-            read += SDL_RWread (rw, &mblock->scaley, 1, 1);
-            break;
-        case 0x06:             /* Smooth scaling */
-            read += SDL_RWread (rw, &mblock->smooth_scale, 1, 1);
+        case 0x03: /* Zoom */
+            read += SDL_RWread (rw, &zoom, 4, 1) * 4;
+            mblock->zoom = SDL_SwapLE32(zoom)/1000.0;
             break;
         default:
-            printf
-                ("WARNING !! Unknown main block item 0x%x ! (skipping one byte...)\n",
+            fprintf(stderr,
+                "WARNING! Unknown main block item 0x%x! (skipping one byte...)\n",
                  type);
             SDL_RWseek (rw, 1, SEEK_CUR);
             read++;
@@ -353,6 +285,21 @@ static int readMainBlockRW (LevelSettings * settings, SDL_RWops * rw, int len)
     return read;
 }
 
+static void init_override_block(LSB_Override *block) {
+    block->indstr_base = -1;
+    block->critters = -1;
+    block->stars = -1;
+    block->snowfall = -1;
+    block->turrets = -1;
+    block->jumpgates = -1;
+    block->cows = -1;
+    block->fish = -1;
+    block->birds = -1;
+    block->bats = -1;
+    block->soldiers = -1;
+    block->helicopters = -1;
+}
+
 /* Read the override block */
 static LSB_Override *readOverrideBlock (FILE * fp)
 {
@@ -360,18 +307,7 @@ static LSB_Override *readOverrideBlock (FILE * fp)
     char *left, *right;
     char *line;
     override = malloc (sizeof (LSB_Override));
-    override->indstr_base = game_settings.ls.indstr_base;
-    override->critters_enabled = 0;
-    override->stars = game_settings.ls.stars;
-    override->snowfall = game_settings.ls.snowfall;
-    override->turrets = game_settings.ls.turrets;
-    override->jumpgates = game_settings.ls.jumpgates;
-    override->cows = game_settings.ls.cows;
-    override->fish = game_settings.ls.birds;
-    override->birds = game_settings.ls.fish;
-    override->bats = game_settings.ls.bats;
-    override->soldiers = game_settings.ls.soldiers;
-    override->helicopters = game_settings.ls.helicopters;
+    init_override_block(override);
     while ((line = read_line (fp))) {
         if (line[0] == '\0')
             continue;
@@ -381,11 +317,11 @@ static LSB_Override *readOverrideBlock (FILE * fp)
         }
         split_string (line, '=', &left, &right);
         if (left == NULL || right == NULL) {
-            printf ("Warning: Malformed line:\n \"%s\"\n", line);
+            fprintf (stderr,"Warning: Malformed line:\n \"%s\"\n", line);
             continue;
         }
-        if (strcmp (left, "critters_enabled") == 0)
-            override->critters_enabled = atoi (right);
+        if (strcmp (left, "critters") == 0)
+            override->critters = atoi (right);
         else if (strcmp (left, "bases_indestructable") == 0)
             override->indstr_base = atoi (right);
         else if(strcmp (left,"snowfall")==0)
@@ -407,9 +343,9 @@ static LSB_Override *readOverrideBlock (FILE * fp)
         else if (strcmp (left, "soldiers") == 0)
             override->soldiers = atoi (right);
         else if (strcmp (left, "helicopters") == 0)
-            override->soldiers = atoi (right);
+            override->helicopters = atoi (right);
         else
-            printf ("Unidentified options \"%s\" with parameter \"%s\"\n",
+            fprintf (stderr,"Unidentified option \"%s\" with parameter \"%s\"\n",
                     left, right);
         free (line);
         free (left);
@@ -423,60 +359,47 @@ static int readOverrideBlockRW (LevelSettings * settings, SDL_RWops * rw,
                                 int len)
 {
     LSB_Override *override;
+    Uint8 type,byte;
     int read = 0;
-    Uint8 type;
     override = malloc (sizeof (LSB_Override));
-    override->indstr_base = game_settings.ls.indstr_base;
-    override->critters_enabled = 0;
-    override->stars = game_settings.ls.stars;
-    override->snowfall = game_settings.ls.snowfall;
-    override->turrets = game_settings.ls.turrets;
-    override->jumpgates = game_settings.ls.jumpgates;
-    override->cows = game_settings.ls.cows;
-    override->fish = game_settings.ls.birds;
-    override->birds = game_settings.ls.fish;
-    override->bats = game_settings.ls.bats;
-    override->soldiers = game_settings.ls.soldiers;
-    override->helicopters = game_settings.ls.helicopters;
+    init_override_block(override);
     while (read < len) {
         read += SDL_RWread (rw, &type, 1, 1);
+        read += SDL_RWread (rw, &byte, 1, 1);
         switch (type) {
-        case 0x01:             /* Critters enabled */
-            read += SDL_RWread (rw, &override->critters_enabled, 1, 1);
+        case 0x01:             /* Critters */
+            override->critters=byte;
             break;
         case 0x02:             /* Bases indestructable */
-            read += SDL_RWread (rw, &override->indstr_base, 1, 1);
+            override->indstr_base=byte;
             break;
         case 0x03:             /* Stars */
-            read += SDL_RWread (rw, &override->stars, 1, 1);
+            override->stars=byte;
             break;
         case 0x04:             /* Snowfall */
-            read += SDL_RWread (rw, &override->snowfall, 1, 1);
+            override->snowfall=byte;
             break;
         case 0x05:             /* Turrets */
-            read += SDL_RWread (rw, &override->turrets, 1, 1);
+            override->turrets=byte;
             break;
         case 0x06:             /* Jumpgates */
-            read += SDL_RWread (rw, &override->jumpgates, 1, 1);
+            override->jumpgates=byte;
             break;
         case 0x07:             /* Cows */
-            read += SDL_RWread (rw, &override->cows, 1, 1);
+            override->cows=byte;
             break;
         case 0x08:             /* Fish */
-            read += SDL_RWread (rw, &override->fish, 1, 1);
+            override->fish=byte;
             break;
         case 0x09:             /* Birds */
-            read += SDL_RWread (rw, &override->birds, 1, 1);
+            override->birds=byte;
             break;
         case 0x0a:             /* Bats */
-            read += SDL_RWread (rw, &override->bats, 1, 1);
+            override->bats=byte;
             break;
         default:
-            printf
-                ("WARNING !! Unknown override block item 0x%x ! (skipping one byte...)\n",
-                 type);
-            SDL_RWseek (rw, 1, SEEK_CUR);
-            read++;
+            printf("Warning! Unknown override block item 0x%x=0x%x!\n",
+                 type,byte);
             break;
         }
     }
@@ -503,7 +426,7 @@ static LSB_Objects *readObjectsBlock (FILE * fp)
             free (line);
             objects = malloc (sizeof (LSB_Objects));
             if (objects == NULL) {
-                printf ("Malloc error at readObjectsBlock !\n");
+                perror("readObjectsBlock()");
                 exit (1);
             }
             memset (objects, 0, sizeof (LSB_Objects));
@@ -519,7 +442,7 @@ static LSB_Objects *readObjectsBlock (FILE * fp)
         }
         split_string (line, '=', &left, &right);
         if (left == NULL || right == NULL) {
-            printf ("Warning: Malformed line:\n \"%s\"\n", line);
+            fprintf (stderr,"Warning: Malformed line:\n \"%s\"\n", line);
             continue;
         }
         if (strcmp (left, "type") == 0) {
@@ -538,7 +461,7 @@ static LSB_Objects *readObjectsBlock (FILE * fp)
             else if (strcmp (right, "ship") == 0)
                 objects->type = 0x20;
             else
-                printf ("Warning: Unidentified type \"%s\"\n", right);
+                fprintf (stderr,"Warning: Unidentified type \"%s\"\n", right);
         } else if (strcmp (left, "x") == 0)
             objects->x = atoi (right);
         else if (strcmp (left, "y") == 0)
@@ -552,8 +475,8 @@ static LSB_Objects *readObjectsBlock (FILE * fp)
         else if (strcmp (left, "id") == 0)
             objects->id = atoi (right);
         else
-            printf ("Unidenified option \"%s\" with parameter \"%s\"\n", left,
-                    right);
+            fprintf (stderr,"Unidenified option \"%s\" with parameter \"%s\"\n",
+                    left, right);
         free (left);
         free (right);
         free (line);
@@ -561,84 +484,57 @@ static LSB_Objects *readObjectsBlock (FILE * fp)
     return first;
 }
 
-static LSB_Palette *readPaletteBlock (FILE * fp)
-{
+static LSB_Palette *readPaletteBlock (FILE * fp) {
     LSB_Palette *palette;
-    char *left, *right;
     char *line;
-    char mode = 0;
-    Uint8 value = 0;
-    int begin = -1, end = -1;
+    int mapto;
 
     palette = malloc (sizeof (LSB_Palette));
-    memset (palette->entries, 0, 256);  /* In case the levelmaker forgot to put in the default value */
+    memset (palette->entries, 0, 256);
     while ((line = read_line (fp))) {
-        if (line[0] == '\0')
+        char *left=NULL,*right=NULL;
+        if (line[0] == '\0') {
             continue;
-        if (strcmp (line, "[endsub]") == 0) {
-            if (mode == 0) {
-                printf ("Misplaced [endsub] in palette block !\n");
-            } else {
-                mode = 0;
-                if (begin == -1) {
-                    printf ("Error ! No first colour !\n");
-                    continue;
-                }
-                if (end == -1)
-                    end = begin;
-                memset (palette->entries + begin, value, end - begin + 1);
-            }
-            continue;
-        } else if (strcmp (line, "[end]") == 0) {
+        }
+        if (strcmp (line, "[end]") == 0) {
             free (line);
             break;
-        } else if (strcmp (line, "[terrain]") == 0) {
-            if (mode)
-                printf ("Nested [terrains] !\n");
-            mode = 1;
-            end = -1;
-            begin = -1;
-            continue;
         }
         split_string (line, '=', &left, &right);
+        free(line);
         if (left == NULL || right == NULL) {
-            printf ("Warning: Malformed line:\n \"%s\"\n", line);
+            fprintf (stderr,"Warning: Malformed line:\n \"%s\"\n", line);
+            free(left);
+            free(right);
             continue;
         }
-        if (mode == 0) {
-            if (strcmp (left, "default") == 0) {
-                int def = atoi (right);
-                if (def > 255) {
-                    printf
-                        ("Warning, default terrain value is %d, but the 255 is the maximium !\n",
-                         def);
-                    def = 255;
-                }
-                memset (palette->entries, def, 256);
+        mapto = name2terrain(right);
+        if(strchr(left,'-')) { /* Range */
+            int from,to;
+            sscanf(left,"%d - %d",&from,&to);
+            if(from<0||from>255||to<0||to>255) {
+                fprintf(stderr,"Error: palette map range %d-%d out of bounds (0-255!",
+                    from,to);
+                free(left);
+                free(right);
+                continue;
+            } else if(to<from) {
+                int tmp;
+                tmp=to; to=from; from=tmp;
+            }
+            for(;from<=to;from++) palette->entries[from]=mapto;
+        } else { /* Single value */
+            int index;
+            index=atoi(left);
+            if(index<0 || index>255) {
+                fprintf(stderr,"Palette map index %d out of bounds (0-255)!\n",
+                        index);
             } else {
-                printf ("Unrecognized option \"%s\"\n", left);
+                palette->entries[index]=mapto;
             }
-        } else {
-            int val;
-            val = atoi (right);
-            if (val > 255) {
-                printf
-                    ("Warning ! Parameter '%d' for option \"%s\" is too large, maximium is 255 !\n",
-                     val, left);
-                val = 255;
-            }
-            if (strcmp (left, "type") == 0)
-                value = val;
-            else if (strcmp (left, "begin") == 0)
-                begin = val;
-            else if (strcmp (left, "end") == 0)
-                end = val;
-            else
-                printf ("Unrecognized option \"%s\"\n", left);
         }
-        free (left);
-        free (right);
-        free (line);
+        free(left);
+        free(right);
     }
     return palette;
 }
@@ -654,7 +550,7 @@ static int readObjectsBlockRW (LevelSettings * settings, SDL_RWops * rw,
         prev = objects;
         objects = malloc (sizeof (LSB_Objects));
         if (objects == NULL) {
-            printf ("Malloc error at readObjectsBlockRW !\n");
+            perror("readObjectsBlockRW()");
             exit (1);
         }
         memset (objects, 0, sizeof (LSB_Objects));
@@ -663,6 +559,7 @@ static int readObjectsBlockRW (LevelSettings * settings, SDL_RWops * rw,
         if (prev)
             prev->next = (struct LSB_Objects *) objects;
         read += SDL_RWread (rw, &l, 2, 1) * 2;
+        l = SDL_SwapLE16(l);
         read += l;
         l -= SDL_RWread (rw, &objects->type, 1, 1);
         l -= SDL_RWread (rw, &objects->x, 4, 1) * 4;
@@ -673,6 +570,11 @@ static int readObjectsBlockRW (LevelSettings * settings, SDL_RWops * rw,
         l -= SDL_RWread (rw, &objects->link, 4, 1) * 4;
         if (l)
             SDL_RWseek (rw, l, SEEK_CUR);
+
+        objects->x = SDL_SwapLE32(objects->x);
+        objects->y = SDL_SwapLE32(objects->y);
+        objects->id = SDL_SwapLE32(objects->id);
+        objects->link = SDL_SwapLE32(objects->link);
     }
     settings->objects = first;
     return read;
@@ -686,16 +588,14 @@ static int readPaletteBlockRW (LevelSettings * settings, SDL_RWops * rw,
     int read;
     palette = malloc (sizeof (LSB_Palette));
     if (palette == NULL) {
-        printf ("Malloc error in readPaletteBlockRW !\n");
+        perror("readPaletteBlockRW()");
         exit (1);
     }
     if (len != 256)
-        printf ("Warning: Palette block has strange length (%d)\n", len);
-    if (len < 256)
-        printf ("Error: Palette block length is less than 256 bytes !\n");
+        fprintf (stderr,"Warning: Palette block has strange length (%d)\n",len);
     read = SDL_RWread (rw, &palette->entries, 1, 256);
     if (read != 256)
-        printf ("Warning ! Tried to read 256 bytes but read only %d !\n",
+        fprintf (stderr,"Warning! Tried to read 256 bytes but read only %d!\n",
                 read);
     len -= read;
     if (len)
