@@ -1,6 +1,6 @@
 /*
  * LDAT - Luola Datafile format
- * Copyright (C) 2003 Calle Laakkonen
+ * Copyright (C) 2003-2005 Calle Laakkonen
  *
  * File        : ldat.h
  * Description : Handle Luola datafiles
@@ -24,14 +24,14 @@
 #ifndef LDAT_H
 #define LDAT_H
 
-#include <SDL/SDL_rwops.h>
+#include "SDL_rwops.h"
 
 #define LDAT_MAJOR	0x01
 #define LDAT_MINOR	0x00
 
 /* A structure to hold a catalog item */
 typedef struct LDAT_Block {
-    Uint8 *ID;
+    char *ID;
     Uint16 index;
     Uint32 size;
     Uint32 pos;
@@ -58,13 +58,13 @@ extern LDAT *ldat_open_rw (SDL_RWops * rw);
 extern LDAT *ldat_open_file (const char *filename);
 
 /* Creates an empty LDAT structure */
-extern LDAT *ldat_create ();
+extern LDAT *ldat_create (void);
 
 /* Save LDAT into an SDL_RWops */
 extern int ldat_save_rw (LDAT * ldat, SDL_RWops * rw);
 
 /* Save LDAT into a file */
-extern int ldat_save_file (LDAT * ldat, char *filename);
+extern int ldat_save_file (LDAT * ldat,const char *filename);
 
 /* Close the LDAT (read/write only to cache) */
 extern void ldat_close (LDAT * ldat);
@@ -74,16 +74,16 @@ extern void ldat_free (LDAT * ldat);
 
 /** LDAT reading functions **/
 /* Get a file from the archive */
-extern SDL_RWops *ldat_get_item (LDAT * ldat, char *id, int item);
+extern SDL_RWops *ldat_get_item (const LDAT * ldat, const char *id, int item);
 
 /* Get the length of the item */
-extern int ldat_get_item_length (LDAT * ldat, char *id, int item);
+extern int ldat_get_item_length (const LDAT * ldat, const char *id, int item);
 
 /* Find an item */
-extern LDAT_Block *ldat_find_item (LDAT * ldat, char *id, int item);
+extern LDAT_Block *ldat_find_item (const LDAT * ldat, const char *id, int item);
 /** LDAT writing functions **/
 /* Put a file into the archive */
-void ldat_put_item (LDAT * ldat, char *id, int item, SDL_RWops * data,
+extern void ldat_put_item (LDAT * ldat, char *id, int item, SDL_RWops * data,
                     Uint32 len);
 
 #endif
