@@ -34,6 +34,7 @@
 #include "levelfile.h"
 #include "particle.h"
 #include "weather.h"
+#include "ship.h"   /* for bump_ship() */
 
 /* Level effects */
 typedef struct {
@@ -65,18 +66,11 @@ static Star lev_stars[STARCOUNT];
 Uint32 burncolor[FIRE_FRAMES];
 Uint32 lev_watercolrgb[3];
 Uint32 lev_watercol;
-Uint32 lev_basecol;
 SDL_Rect cam_rects[4];
 SDL_Rect lev_rects[4];
 Level lev_level;
 Vector gravity;
 
-/* The default palette for Luola levelfiles */
-static int default_palette (int col) {
-    if(col>=TER_FREE && col<=LAST_TER) return col;
-    return TER_FREE;
-}
-
 static int touch_wall (int x, int y)
 {
     int x1, y1;
@@ -109,8 +103,8 @@ static void draw_stars (SDL_Rect * cam, SDL_Rect * targ)
             (Uint8 *) lev_level.terrain->pixels +
             y * lev_level.terrain->pitch +
             x * lev_level.terrain->format->BytesPerPixel;
-        if (lev_level.solid[x][y] == TER_FREE && col[0] < 5 && col[1] < 5
-            && col[2] < 5)
+        if (x < lev_level.width && y < lev_level.height)
+          if (lev_level.solid[x][y] == TER_FREE && col[0] < 5 && col[1] < 5 && col[2] < 5)
             putpixel (screen, targ->x + lev_stars[r].x,
                       targ->y + lev_stars[r].y, col_white);
     }
@@ -121,7 +115,7 @@ static inline void draw_level (void)
 {
     int p;
     for (p = 0; p < 4; p++) {
-        if (players[p].state==ALIVE) {
+        if (players[p].state==ALIVE || players[p].state==DEAD) {
             SDL_BlitSurface (lev_level.terrain, &cam_rects[p], screen,
                              &lev_rects[p]);
             if(level_settings.stars)
@@ -133,29 +127,9 @@ static inline void draw_level (void)
 /* Initialize level subsystem */
 void init_level (void)
 {
-    int r, red, green, blue;
+    int r,red, green, blue;
     gravity = makeVector (0, -GRAVITY);
-    cam_rects[0].w = 320;
-    cam_rects[0].h = 240;
-    cam_rects[1].w = 320;
-    cam_rects[1].h = 240;
-    cam_rects[2].w = 320;
-    cam_rects[2].h = 240;
-    cam_rects[3].w = 320;
-    cam_rects[3].h = 240;
-    lev_rects[0].x = 0;
-    lev_rects[0].y = 0;
-    lev_rects[1].x = 320;
-    lev_rects[1].y = 0;
-    lev_rects[2].x = 0;
-    lev_rects[2].y = 240;
-    lev_rects[3].x = 320;
-    lev_rects[3].y = 240;
-#if HAVE_LIBSDL_GFX
-    lev_watercol = 0x6464FFFF;
-#else
-    lev_watercol = SDL_MapRGB (screen->format, 100, 100, 255);
-#endif
+    lev_watercol = map_rgba(0x64,0x64,0xff,0xff);
     level_effects = NULL;
     lev_lastfx = NULL;
     for (r = 0; r < FIRE_FRAMES; r++) {
@@ -167,15 +141,25 @@ void init_level (void)
         blue = 0;
         if (red > 255)
             red = 255;
-#if HAVE_LIBSDL_GFX
-        burncolor[r] = (red << 24) + (green << 16) + (blue << 8) + 0xFF;
-#else
-        burncolor[r] = SDL_MapRGB (screen->format, red, green, blue);
-#endif
+        burncolor[r] = map_rgba(red,green,blue,0xff);
+    }
+}
+
+/* Calculate star positions. This must be done when player screen */
+/* geometry changes. */
+void reinit_stars(void) {
+    int w=screen->w,h=screen->h,r;
+    /* Find an active player and use its screen geometry */
+    for(r=0;r<4;r++) {
+        if(players[r].state != INACTIVE) {
+            w = cam_rects[r].w;
+            h = cam_rects[r].h;
+            break;
+        }
     }
     for (r = 0; r < STARCOUNT; r++) {
-        lev_stars[r].x = rand () % (SCREEN_W / 2);
-        lev_stars[r].y = rand () % (SCREEN_H / 2);
+        lev_stars[r].x = rand () % w;
+        lev_stars[r].y = rand () % h;
     }
 }
 
@@ -216,8 +200,12 @@ void load_level (struct LevelFile *lev) {
         for (r = 0; r < 256; r++)
             palette[r] = lev->settings->palette->entries[r];
     } else {
-        for (r = 0; r < 256; r++)
-            palette[r] = default_palette(r);
+        for (r = 0; r < 256; r++) {
+            if(r<=LAST_TER)
+                palette[r] = r;
+            else
+                palette[r] = TER_FREE;
+        }
     }
     /* Load level artwork */
     lev_level.terrain = load_level_art (lev);
@@ -231,7 +219,7 @@ void load_level (struct LevelFile *lev) {
     }
     if (collmap->w != lev_level.width || collmap->h != lev_level.height) {
         printf
-            ("Error Collision map image \"%s\" has incorrect size (%dx%d), should be %dx%d !\n",
+            ("Error Collision map image \"%s\" has incorrect size (%dx%d), should be %dx%d!\n",
              lev->settings->mainblock->collmap, collmap->w, collmap->h,
              lev_level.width, lev_level.height);
         exit (1);
@@ -240,13 +228,7 @@ void load_level (struct LevelFile *lev) {
     /* Get water colour */
     tmpcol = &defaultwater;
     find_color(TER_WATER,palette,collmap->format->palette,&tmpcol);
-#if HAVE_LIBSDL_GFX
-    lev_watercol =
-        (tmpcol->r << 24) + (tmpcol->g << 16) + (tmpcol->b << 8) + 0xFF;
-#else
-    lev_watercol =
-        SDL_MapRGB (screen->format, tmpcol->r, tmpcol->g, tmpcol->b);
-#endif
+    lev_watercol = map_rgba(tmpcol->r, tmpcol->g, tmpcol->b,0xff);
     lev_watercolrgb[0] = tmpcol->r;
     lev_watercolrgb[1] = tmpcol->g;
     lev_watercolrgb[2] = tmpcol->b;
@@ -254,35 +236,12 @@ void load_level (struct LevelFile *lev) {
     tmpcol->r = (lev_watercolrgb[0] + 255) / 2;
     tmpcol->g = (lev_watercolrgb[0] + 200) / 2;
     tmpcol->b = (lev_watercolrgb[0] + 128) / 2;
-#if HAVE_LIBSDL_GFX
-    col_clay2 =
-        (tmpcol->r << 24) + (tmpcol->g << 16) + (tmpcol->b << 8) + 0xFF;
-#else
-    col_clay2 = SDL_MapRGB (screen->format, tmpcol->r, tmpcol->g, tmpcol->b);
-#endif
+    col_clay2 = map_rgba(tmpcol->r, tmpcol->g, tmpcol->b, 0xff);
     /* Get snow colour */
     tmpcol = NULL;
     find_color(TER_SNOW,palette,collmap->format->palette,&tmpcol);
     if (tmpcol) {
-#if HAVE_LIBSDL_GFX
-        col_snow =
-            (tmpcol->r << 24) + (tmpcol->g << 16) + (tmpcol->b << 8) + 0xFF;
-#else
-        col_snow =
-            SDL_MapRGB (screen->format, tmpcol->r, tmpcol->g, tmpcol->b);
-#endif
-    }
-    /* Get base colour */
-    tmpcol = NULL;
-    find_color(TER_BASE,palette,collmap->format->palette,&tmpcol);
-    if(tmpcol) {
-#if HAVE_LIBSDL_GFX
-        lev_basecol =
-            (tmpcol->r << 24) + (tmpcol->g << 16) + (tmpcol->b << 8) + 0xFF;
-#else
-        lev_basecol =
-            SDL_MapRGB(screen->format, tmpcol->r, tmpcol->g, tmpcol->b);
-#endif
+        col_snow = map_rgba(tmpcol->r, tmpcol->g, tmpcol->b,0xff);
     }
     /* Prepare base regeneration array */
     if(game_settings.base_regen) {
@@ -295,13 +254,12 @@ void load_level (struct LevelFile *lev) {
     /* Load data map */
     lev_level.base_area = 0;
     lev_level.regen_area = 0;
-    lev_level.solid = malloc (sizeof (unsigned char *) * lev_level.width);
+    lev_level.solid = malloc (sizeof (char *) * lev_level.width);
     freepix = 0;
     otherpix = 0;
     for (x = 0; x < lev_level.width; x++) {
         bits = ((Uint8 *) collmap->pixels)+x;
-        lev_level.solid[x] =
-            malloc (sizeof (unsigned char) * lev_level.height);
+        lev_level.solid[x] = malloc (lev_level.height);
         for (y = 0; y < lev_level.height; y++,bits+=collmap->pitch) {
             lev_level.solid[x][y] = palette[*bits];
             if (lev_level.solid[x][y] == TER_FREE
@@ -311,8 +269,17 @@ void load_level (struct LevelFile *lev) {
                 otherpix++;
                 if (lev_level.solid[x][y] == TER_BASE) {
                     if(lev_level.base) {
+                        Uint8 r,g,b;
                         lev_level.base[lev_level.base_area].x=x;
                         lev_level.base[lev_level.base_area].y=y;
+                        SDL_GetRGB(getpixel(lev_level.terrain,x,y),
+                                screen->format,
+                                &r,
+                                &g,
+                                &b);
+                        lev_level.base[lev_level.base_area].c=
+                            map_rgba(r, g, b, 0xff);
+
                         if(lev_level.base_area==basebufsize-1) {
                             basebufsize+=512;
                             lev_level.base=realloc(lev_level.base,
@@ -352,7 +319,7 @@ void load_level (struct LevelFile *lev) {
                     r++;
                     if (r > 100000) {
                         printf
-                            ("Warning: while searching for player %d startup position, loop counter reached 100000 !\n",
+                            ("Warning: while searching for player %d startup position, loop counter reached 100000!\n",
                              p);
                         printf
                             ("Number of free space (including water) pixels: %d, number of other pixels: %d\n",
@@ -476,7 +443,7 @@ int find_rainy (int x)
 {
     int y, loops = 0, r, c;
     do {
-        y = 15 + rand () % 160;
+        y = 15 + rand () % (screen->w/4);
         loops++;
         if (loops > 100)
             return -1;
@@ -543,6 +510,8 @@ void start_melting (int x, int y, unsigned int recurse)
     fx->type = Melt;
     fx->value = 4;
     fx->brake = recurse;
+    if(game_settings.base_regen && lev_level.solid[x][y]==TER_BASE)
+        lev_level.base_area--;
     lev_level.solid[x][y] = TER_FREE;
     putpixel (lev_level.terrain, x, y, col_green);
     newentry->fx = fx;
@@ -613,8 +582,10 @@ void animate_level (void)
                 x=lev_level.base[r].x;
                 y=lev_level.base[r].y;
                 if(lev_level.solid[x][y]==TER_FREE) {
+                    bump_ship(x,y);
                     lev_level.solid[x][y]=TER_BASE;
-                    putpixel (lev_level.terrain, x, y, lev_basecol);
+                    putpixel (lev_level.terrain, x, y, lev_level.base[r].c);
+                    putpixel (lev_level.terrain, x, y, lev_level.base[r].c);
                     lev_level.base_area++;
                     break;
                 }
@@ -668,8 +639,8 @@ void animate_level (void)
                 }
                 if (list->fx->value == FIRE_SPREAD) {
                     for (f = -M_PI; f < M_PI; f += M_PI / 4.0) {
-                        nx = list->fx->x + round(sin (f) * 3.0);
-                        ny = list->fx->y + round(cos (f) * 3.0);
+                        nx = list->fx->x + Round(sin (f) * 3.0);
+                        ny = list->fx->y + Round(cos (f) * 3.0);
                         if (nx >= lev_level.width || ny >= lev_level.height
                             || nx <= 0 || ny <= 0)
                             continue;
@@ -723,8 +694,8 @@ void animate_level (void)
                 putpixel (lev_level.terrain, list->fx->x, list->fx->y,
                           col_black);
                 for (f = -M_PI; f < M_PI; f += M_PI / 4.0) {
-                    nx = list->fx->x + round(sin (f) * 3.0);
-                    ny = list->fx->y + round(cos (f) * 3.0);
+                    nx = list->fx->x + Round(sin (f) * 3.0);
+                    ny = list->fx->y + Round(cos (f) * 3.0);
                     if (nx >= lev_level.width || ny >= lev_level.height
                         || nx <= 0 || ny <= 0)
                         continue;
