@@ -52,19 +52,14 @@
 
 static void show_version (void) {
     const SDL_version *sdld;
-    const SDL_VideoInfo *vidinfo;
-    SDL_Init (SDL_INIT_VIDEO);  /* So we get info about the video mode */
     sdld = SDL_Linked_Version ();
-    vidinfo = SDL_GetVideoInfo ();
     printf ("%s version %s (Stable branch)\n", PACKAGE, VERSION);
     printf ("Compiled with SDL version %d.%d.%d\n", SDL_MAJOR_VERSION,
             SDL_MINOR_VERSION, SDL_PATCHLEVEL);
     printf ("Dynamically linked SDL version is %d.%d.%d\n", sdld->major,
             sdld->minor, sdld->patch);
-#ifdef PACKAGE_DATA_DIR
+#ifndef WIN32
     printf ("Data directory is \"%s\"\n", PACKAGE_DATA_DIR);
-#else
-    printf ("Test compile, data directory is \"../data\"\n");
 #endif
     printf ("\nCompiled in features:\n");
 #if HAVE_LIBSDL_MIXER
@@ -82,27 +77,6 @@ static void show_version (void) {
 #else
     printf ("SDL_ttf (Truetype fonts): disabled\n");
 #endif
-    if (vidinfo) {
-        printf ("Video information:\n");
-        printf ("Hardware surfaces available: %s\n",
-                vidinfo->hw_available ? "yes" : "no");
-        printf ("Hardware accelerated blits: %s\n",
-                vidinfo->blit_hw ? "yes" : "no");
-        printf ("Hardware accelerated blits with colorkey: %s\n",
-                vidinfo->blit_hw_CC ? "yes" : "no");
-        printf ("Hardware accelerated blits with alpha: %s\n",
-                vidinfo->blit_hw_A ? "yes" : "no");
-        printf ("Accelerated sw->hw blits: %s\n",
-                vidinfo->blit_sw ? "yes" : "no");
-        printf ("Accelerated sw->hw blits with colorkey: %s\n",
-                vidinfo->blit_sw_CC ? "yes" : "no");
-        printf ("Accelerated sw->hw blits with alpha: %s\n",
-                vidinfo->blit_sw_A ? "yes" : "no");
-        printf ("Total ammount of video memory: %d kilobytes\n",
-                vidinfo->video_mem);
-        printf ("Best mode has %d bits per pixel\n",
-                vidinfo->vfmt->BitsPerPixel);
-    }
     exit (0);
 }
 
@@ -159,6 +133,14 @@ int main (int argc, char *argv[]) {
     if (game_settings.levels == NULL)
         no_levels_found ();
     load_game_config ();
+
+#if HAVE_LIBSDL_MIXER
+    /* Set sound effect volume */
+    audio_setsndvolume(game_settings.sound_vol);
+#endif
+    /* Enable key repeat (useful in menus. Will be disabled during game) */
+    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
+
     /* Enter game loop */
     while (1) {
         /* Intro screen */
