@@ -81,8 +81,8 @@ void animate_particles (void)
     while (list) {
         Particle *part = list->data;
         next = list->next;
-        part->x -= round (part->vector.x);
-        part->y -= round (part->vector.y);
+        part->x -= Round (part->vector.x);
+        part->y -= Round (part->vector.y);
         part->color[0] += part->rd;
         part->color[1] += part->gd;
         part->color[2] += part->bd;
@@ -105,21 +105,14 @@ static inline void draw_particle (Particle * part)
     Uint32 color;
     int x, y, p;
 
-#if HAVE_LIBSDL_GFX
-    color =
-        (part->color[0] << 24) + (part->color[1] << 16) +
-        (part->color[2] << 8) + part->color[3];
-#else
-    color =
-        SDL_MapRGB (screen->format, part->color[0], part->color[1],
-                    part->color[2]);
-#endif
+    color = map_rgba(part->color[0], part->color[1], part->color[2],
+            part->color[3]);
     for (p = 0; p < 4; p++) {
-        if (players[p].state==ALIVE) {
+        if (players[p].state==ALIVE || players[p].state==DEAD) {
             x = part->x - cam_rects[p].x;
             y = part->y - cam_rects[p].y;
-            if ((x > 0 && x < lev_rects[p].w)
-                && (y > 0 && y < lev_rects[p].h))
+            if ((x > 0 && x < cam_rects[p].w)
+                && (y > 0 && y < cam_rects[p].h))
                 putpixel (screen, lev_rects[p].x + x, lev_rects[p].y + y,
                           color);
         }
