@@ -43,10 +43,10 @@ char pilot_any_ejected;
 static SDL_Surface *pilot_sprite[4][3];        /* Normal,Normal2, Parachute */
 
 /* Internally used functions */
-static void draw_pilot_crosshair (int plr, Pilot * pilot, SDL_Rect * rect);
+static void draw_pilot_crosshair (Pilot * pilot, SDL_Rect * rect);
 static void pilot_rope_code (Pilot * pilot);
 static void pilot_motion_code (Pilot * pilot);
-static void pilot_rope_state (Pilot * pilot, Uint8 attached);
+static void pilot_rope_state (Pilot * pilot, int attach);
 
 /* Initialize pilot code */
 int init_pilots (void) {
@@ -97,6 +97,8 @@ void init_pilot(Pilot *pilot,int playernum) {
     pilot->maxspeed = MAXSPEED;
     pilot->hy = pilot->sprite[0]->h;
     pilot->hx = pilot->sprite[0]->w / 2;
+    pilot->attack_vector.x = 1.0;
+    pilot->attack_vector.y = 0.0;
 }
 
 /*** PILOT ANIMATION ***/
@@ -118,9 +120,23 @@ void animate_pilots (void) {
         pilot_motion_code (pilot);
         if (pilot->rope)
             pilot_rope_code (pilot);
+        /* Pilot autoaim */
+        if (pilot->lock == 0) {
+            double dist;
+            int nearest = find_nearest_player(Round(pilot->x),Round(pilot->y), p, &dist);
+            if (nearest>=0 && dist < 150) {
+                pilot->attack_vector.x = -(players[nearest].ship->x - pilot->x)/dist;
+                pilot->attack_vector.y = -(players[nearest].ship->y - pilot->y)/dist;
+                pilot->crosshair_color = col_plrs[nearest];
+            } else {
+                pilot->crosshair_color = col_white;
+            }
+        } else {
+            pilot->crosshair_color = col_white;
+        }
         /* Pilot physics */
         tmpv = pilot->vector;
-        solid2 = hit_solid(round(pilot->x)+pilot->hx,round(pilot->y)+pilot->hy);
+        solid2 = hit_solid(Round(pilot->x)+pilot->hx,Round(pilot->y)+pilot->hy);
         if (solid2 == TER_WALKWAY)
             solid2 = TER_FREE;
         if (pilot->rope != USEROPE) {
@@ -186,19 +202,26 @@ void animate_pilots (void) {
             newy = pilot->rope_y +
                 cos (pilot->rope_th) * (double) pilot->rope_len;
         }
-        if (newx < 0)
+        solid=0;
+        if (newx < 0) {
             newx = 0;
-        else if (newx + pilot->hx > lev_level.width - 1)
+            solid=TER_INDESTRUCT;
+        } else if (newx + pilot->hx > lev_level.width - 1) {
             newx = lev_level.width - 1 - pilot->hx;
+            solid=TER_INDESTRUCT;
+        }
         if (newy < 0)
             newy = 0;
         else if (newy + pilot->hy > lev_level.height - 1)
             newy = lev_level.height - 1 - pilot->hy;
-        solid = is_walkable (round(newx) + pilot->hx, round(newy) + pilot->hy);
+        if(!solid)
+            solid = is_walkable (Round(newx) + pilot->hx,
+                    Round(newy) + pilot->hy);
         pilot->onbase = 0;
         if (solid) {
-            if (pilot->toofast>PILOT_TOOFAST && pilot->rope!=USEROPE) { /* Player hit the ground too fast */
-                splatter (round(pilot->x), round(pilot->y), Blood);
+            if (pilot->toofast>PILOT_TOOFAST && pilot->rope!=USEROPE) {
+                /* Player hit the ground too fast */
+                splatter (Round(pilot->x), Round(pilot->y), Blood);
                 buryplayer (p);
             }
             if (solid2 == TER_BASE)
@@ -210,15 +233,17 @@ void animate_pilots (void) {
             if (newy < pilot->y)
                 newy = pilot->y;
             else
-                newy = round(newy);
-            if (pilot->parachuting) {   /* Stop parachuting */
+                newy = Round(newy);
+            if (pilot->parachuting) {
+                /* Stop parachuting */
                 pilot->parachuting = 0;
                 pilot->maxspeed = MAXSPEED;
                 newy += pilot->sprite[2]->h - pilot->sprite[0]->h;
                 newx += pilot->sprite[2]->w / 2 - pilot->sprite[0]->w / 2;
                 pilot->hx = pilot->sprite[0]->w / 2;
                 pilot->hy = pilot->sprite[0]->h;
-            } else if (pilot->rope == USEROPE) {/* Bounce of walls while roping */
+            } else if (pilot->rope == USEROPE) {
+                /* Bounce of walls while roping */
                 pilot->rope_v *= -0.5;
                 pilot->rope_th += pilot->rope_v;
                 newx = pilot->rope_x +
@@ -255,9 +280,14 @@ void animate_pilots (void) {
     }
 }
 
+/* Detach rope. Used from outside this module */
+void pilot_detach_rope(Pilot *pilot) {
+    pilot_rope_state(pilot, 0);
+}
+
 /* Start/stop roping with apropriate physics */
-static void pilot_rope_state (Pilot * pilot, Uint8 attached) {
-    if (attached) {
+static void pilot_rope_state (Pilot * pilot, int attach) {
+    if (attach) {
         pilot->rope = USEROPE;
         pilot->vector.x = 0;
         pilot->vector.y = 0;
@@ -315,7 +345,8 @@ static void pilot_rope_code (Pilot * pilot) {
                 pilot->rope_y = pilot->y - pilot->rope_len;
             }
         }
-    } else {                /* Rope physics */
+    } else {
+        /* Rope physics */
         double dd;
         if (pilot->rope_ship) {
             pilot->rope_x = pilot->rope_ship->x;
@@ -334,12 +365,12 @@ static void pilot_rope_code (Pilot * pilot) {
 static void pilot_motion_code (Pilot * pilot) {
     signed char solid;
     int i1;
-    if (round(pilot->y) + pilot->hy >= lev_level.height - 1)
+    if (Round(pilot->y) + pilot->hy >= lev_level.height - 1)
         solid = TER_INDESTRUCT;
-    else if (round(pilot->x) + pilot->hx >= lev_level.width - 1)
+    else if (Round(pilot->x) + pilot->hx >= lev_level.width - 1)
         solid = TER_INDESTRUCT;
     else
-        solid = lev_level.solid[round(pilot->x)+pilot->hx][round(pilot->y)+pilot->hy];
+        solid = lev_level.solid[Round(pilot->x)+pilot->hx][Round(pilot->y)+pilot->hy];
     /* Directional controls */
     if (pilot->walking && !pilot->parachuting) {
         if (pilot->walking < -1)
@@ -358,17 +389,17 @@ static void pilot_motion_code (Pilot * pilot) {
                 pilot->x -= 3;
                 if (pilot->x < 0)
                     pilot->x = 0;
-                i1 = find_nearest_terrain (round(pilot->x) + pilot->hx,
-                        round(pilot->y), pilot->hy);
-                if(round(pilot->x)<0) pilot->x=0;
-                else if(round(pilot->x)>=lev_level.width) pilot->x=lev_level.width-1;
+                i1 = find_nearest_terrain (Round(pilot->x) + pilot->hx,
+                        Round(pilot->y), pilot->hy);
+                if(Round(pilot->x)<0) pilot->x=0;
+                else if(Round(pilot->x)>=lev_level.width) pilot->x=lev_level.width-1;
                 if (i1 != -1)
                     pilot->y = i1;
-                else if (lev_level.solid[round(pilot->x)][round(pilot->y)]!=
+                else if (lev_level.solid[Round(pilot->x)][Round(pilot->y)]!=
                         TER_FREE
-                        && lev_level.solid[round(pilot->x)][round(pilot->y)] !=
+                        && lev_level.solid[Round(pilot->x)][Round(pilot->y)] !=
                         TER_TUNNEL
-                        && lev_level.solid[round(pilot->x)][round(pilot->y)] !=
+                        && lev_level.solid[Round(pilot->x)][Round(pilot->y)] !=
                         TER_WALKWAY)
                     pilot->x += 3;
             }
@@ -389,19 +420,19 @@ static void pilot_motion_code (Pilot * pilot) {
                 pilot->vector.x -= 1;
             } else {
                 pilot->x += 3;
-                if (round(pilot->x) + pilot->hx >= lev_level.width)
+                if (Round(pilot->x) + pilot->hx >= lev_level.width)
                     pilot->x = lev_level.width - pilot->hx - 1;
-                i1 = find_nearest_terrain (round(pilot->x) + pilot->hx,
-                        round(pilot->y), pilot->hy);
-                if(round(pilot->x)<0) pilot->x=0;
-                else if(round(pilot->x)>=lev_level.width) pilot->x=lev_level.width-1;
+                i1 = find_nearest_terrain (Round(pilot->x) + pilot->hx,
+                        Round(pilot->y), pilot->hy);
+                if(Round(pilot->x)<0) pilot->x=0;
+                else if(Round(pilot->x)>=lev_level.width) pilot->x=lev_level.width-1;
                 if (i1 != -1)
                     pilot->y = i1;
-                else if (lev_level.solid[round(pilot->x)][round(pilot->y)]!=
+                else if (lev_level.solid[Round(pilot->x)][Round(pilot->y)]!=
                         TER_FREE
-                        && lev_level.solid[round(pilot->x)][round(pilot->y)]!=
+                        && lev_level.solid[Round(pilot->x)][Round(pilot->y)]!=
                         TER_TUNNEL
-                        && lev_level.solid[round(pilot->x)][round(pilot->y)]!=
+                        && lev_level.solid[Round(pilot->x)][Round(pilot->y)]!=
                         TER_WALKWAY)
                     pilot->x -= 3;
             }
@@ -440,14 +471,14 @@ static void pilot_motion_code (Pilot * pilot) {
             pilot->rope_ship = NULL;
             pilot->updown = 0;
             pilot->rope_x =
-                round(pilot->x) + (pilot->rope_dir > 0) * pilot->sprite[0]->w;
-            pilot->rope_y = round(pilot->y);
+                Round(pilot->x) + (pilot->rope_dir > 0) * pilot->sprite[0]->w;
+            pilot->rope_y = Round(pilot->y);
         }
     } else if (pilot->updown == 1 && pilot->lock == 0) { /* Extend rope,dive or shoot the rope straight up */
         if (pilot->rope == USEROPE) {
             if (pilot->rope_len < MAX_ROPE_LEN - 4
-                && hit_solid (round(pilot->x) + pilot->hx,
-                              round(pilot->y) + pilot->hy) == 0) {
+                && hit_solid (Round(pilot->x) + pilot->hx,
+                              Round(pilot->y) + pilot->hy) == 0) {
                 int newx,newy;
                 pilot->rope_len += 4;
                 newx = pilot->rope_x +
@@ -466,8 +497,8 @@ static void pilot_motion_code (Pilot * pilot) {
             pilot->rope_len = 0;
             pilot->rope_ship = NULL;
             pilot->updown = 0;
-            pilot->rope_x = round(pilot->x);
-            pilot->rope_y = round(pilot->y);
+            pilot->rope_x = Round(pilot->x);
+            pilot->rope_y = Round(pilot->y);
         }
     }
 }
@@ -490,12 +521,12 @@ void draw_pilots (void)
         if (sn > 2)
             sn = 0;
         for (plr = 0; plr < 4; plr++) {
-            if (players[plr].state==ALIVE) {
+            if (players[plr].state==ALIVE || players[plr].state==DEAD) {
                 rect.x =
-                    round(players[p].pilot.x) - cam_rects[plr].x +
+                    Round(players[p].pilot.x) - cam_rects[plr].x +
                     lev_rects[plr].x;
                 rect.y =
-                    round(players[p].pilot.y) - cam_rects[plr].y +
+                    Round(players[p].pilot.y) - cam_rects[plr].y +
                     lev_rects[plr].y;
                 if ((rect.x >= lev_rects[plr].x
                      && rect.x < lev_rects[plr].x + cam_rects[plr].w)
@@ -503,7 +534,7 @@ void draw_pilots (void)
                         && rect.y < lev_rects[plr].y + cam_rects[plr].h)) {
                     SDL_BlitSurface (players[p].pilot.sprite[sn], NULL,
                                      screen, &rect);
-                    draw_pilot_crosshair (p, &players[p].pilot, &rect);
+                    draw_pilot_crosshair (&players[p].pilot, &rect);
                 }
                 if (players[p].pilot.rope) {
                     rx = rect.x;
@@ -530,60 +561,55 @@ void draw_pilots (void)
     }
 }
 
-static void draw_pilot_crosshair (int plr, Pilot *pilot, SDL_Rect *rect)
+static void draw_pilot_crosshair (Pilot *pilot, SDL_Rect *rect)
 {
-    int x, y;
-    int p;
-    double d;
-    Uint32 color;
-    p = find_nearest_player (round(pilot->x), round(pilot->y), plr, &d);
-    if (d < 150 && players[p].ship && pilot->lock == 0) {
-        d = atan2 (players[p].ship->x - pilot->x,
-                   players[p].ship->y - pilot->y);
-        x = rect->x + (pilot->hx) + sin (d) * 12;
-        y = rect->y + (pilot->hy / 2) + cos (d) * 12;
-        color = col_plrs[p];
-    } else {
-        x = rect->x + (pilot->hx) - pilot->attack_vector.x * 12;
-        y = rect->y + pilot->hy / 2 - pilot->attack_vector.y * 12;
-        color = col_white;
-    }
-    putpixel (screen, x, y, color);
-    putpixel (screen, x, y + 2, color);
-    putpixel (screen, x, y - 2, color);
-    putpixel (screen, x - 2, y, color);
-    putpixel (screen, x + 2, y, color);
+    int x = rect->x + (pilot->hx) - pilot->attack_vector.x * 12;
+    int y = rect->y + pilot->hy / 2 - pilot->attack_vector.y * 12;
+    putpixel (screen, x, y, pilot->crosshair_color);
+    putpixel (screen, x, y + 2, pilot->crosshair_color);
+    putpixel (screen, x, y - 2, pilot->crosshair_color);
+    putpixel (screen, x - 2, y, pilot->crosshair_color);
+    putpixel (screen, x + 2, y, pilot->crosshair_color);
 }
 
 void control_pilot (int plr) {
     Pilot *pilot;
     Uint8 solid;
-    double f1;
-    int i2;
+
     pilot = &players[plr].pilot;
-    solid=lev_level.solid[round(pilot->x)+pilot->hx][round(pilot->y)+pilot->hy];
-    if (players[plr].controller.axis[1] > 0) {  /* Walk/glide left */
+    solid=lev_level.solid[Round(pilot->x)+pilot->hx][Round(pilot->y)+pilot->hy];
+    pilot->lock = players[plr].controller.weapon2;
+    if (players[plr].controller.axis[1] > 0) {
+        /* Walk/glide left */
         pilot->walking = -1;
-        pilot->attack_vector.x = 1;
-    } else if (players[plr].controller.axis[1] < 0) {   /* Walk/glide right */
+        if(pilot->attack_vector.x < 0 && pilot->crosshair_color==col_white) {
+            pilot->attack_vector.x = -pilot->attack_vector.x;
+        }
+    } else if (players[plr].controller.axis[1] < 0) {
+        /* Walk/glide right */
         pilot->walking = 1;
-        pilot->attack_vector.x = -1;
+        if(pilot->attack_vector.x > 0 && pilot->crosshair_color==col_white) {
+            pilot->attack_vector.x = -pilot->attack_vector.x;
+        }
     } else {
         pilot->walking = 0;
     }
-    if (players[plr].controller.axis[0] > 0) {  /* Jump/swim/climb up */
+    if (players[plr].controller.axis[0] > 0) {
+        /* Jump/swim/climb up/aim up */
         pilot->updown = -1;
-        pilot->attack_vector.y = 1;
-    } else if (players[plr].controller.axis[0] < 0) {   /* Swim/climb down, shoot rope straight up */
+        if(pilot->lock || pilot->parachuting)
+            rotateVector(&pilot->attack_vector,pilot->attack_vector.x<0?-0.5:0.5);
+    } else if (players[plr].controller.axis[0] < 0) {
+        /* Swim/climb down, shoot rope straight up/aim down */
         pilot->updown = 1;
-        pilot->attack_vector.y = -1;
+        if(pilot->lock || pilot->parachuting)
+            rotateVector(&pilot->attack_vector,pilot->attack_vector.x<0?0.5:-0.5);
     } else {
         pilot->updown = 0;
-        pilot->attack_vector.y = 0;
     }
-    pilot->lock = players[plr].controller.weapon2;
 
-    if (players[plr].controller.weapon2 && pilot->lock_btn2 == 0) {     /* Open parachute / Get off the rope */
+    if (players[plr].controller.weapon2 && pilot->lock_btn2 == 0) {
+        /* Open parachute / Get off the rope */
         pilot->lock_btn2 = 1;
         if (pilot->rope)
             pilot_rope_state (pilot, 0);
@@ -601,15 +627,14 @@ void control_pilot (int plr) {
     } else {
         pilot->lock_btn2 = 0;
     }
-    if (players[plr].controller.weapon1) {/* Shoot / Board another players ship by force */
+    if (players[plr].controller.weapon1) {
+        /* Shoot / Board another players ship by force */
         Projectile *p;
         if (pilot->weap_cooloff > 0)
             return;
-        /* BUG: Should the ship be destroyed while the rope is still attached,
-         * bad things will happen... */
         if (pilot->updown == -1 && pilot->rope_ship
-            && abs (pilot->rope_ship->x - round(pilot->x)) < 8
-            && abs (pilot->rope_ship->y - round(pilot->y) + 4) < 8) {
+            && abs (pilot->rope_ship->x - Round(pilot->x)) < 8
+            && abs (pilot->rope_ship->y - Round(pilot->y) + 4) < 8) {
             int targ = find_player (pilot->rope_ship);
             if (targ < 0)
                 return;
@@ -624,14 +649,6 @@ void control_pilot (int plr) {
         p = make_projectile (pilot->x + pilot->hx, pilot->y + pilot->hy / 2.0,
                              pilot->attack_vector);
         p->ownerteam = player_teams[plr];
-        if (pilot->lock == 0) {
-            i2 = find_nearest_player(round(pilot->x),round(pilot->y), plr, &f1);
-            if (i2>=0 && f1 < 150) {
-                f1 = atan2 (players[i2].ship->x - pilot->x,
-                            players[i2].ship->y - pilot->y);
-                p->vector = makeVector (sin (f1) * -5, cos (f1) * -5);
-            }
-        }
         add_projectile (p);
         pilot->weap_cooloff = 7;
     }
@@ -678,7 +695,7 @@ int find_nearest_pilot(int myX,int myY,int not_this, double *dist) {
         if (not_this >= 0)
             if (player_teams[p] == player_teams[not_this])
                 continue;
-        if (players[p].state==ALIVE || players[p].ship==NULL) {
+        if (players[p].state==ALIVE && players[p].ship==NULL) {
             d = hypot (abs (players[p].pilot.x - myX),
                        abs (players[p].pilot.y - myY));
             if (d < distance) {
