@@ -24,7 +24,7 @@
 #ifndef L_PILOT_H
 #define L_PILOT_H
 
-#include <SDL/SDL.h>
+#include "SDL.h"
 
 #include "game.h"
 
@@ -34,6 +34,7 @@ typedef struct {
     SDL_Surface *sprite[3];     /* Normal,Normal2, Parachute */
     Vector vector;
     Vector attack_vector;
+    Uint32 crosshair_color;
     enum {NOROPE,ROPE_EXTENDING,USEROPE} rope;
     int rope_len;
     int rope_x, rope_y;
@@ -69,5 +70,6 @@ extern void draw_pilots (void);
 extern int hit_pilot (int x, int y,int team);
 extern void control_pilot (int plr);
 extern int find_nearest_pilot(int myX,int myY,int not_this, double *dist);
+extern void pilot_detach_rope(Pilot *pilot);
 
 #endif
