@@ -50,26 +50,22 @@ static SDL_Surface *plr_weaponsel_bg;
 static SDL_Surface *plr_weapons[4];
 /*static SDL_Surface *plr_criticals;*/
 static signed int player_message[4];
-static unsigned char plr_teamc[4];
+static int plr_teamc[4];
 static Uint32 plr_healthbar_col, plr_healthbar_col2, plr_healthbar_col3,
     plr_energybar_col, plr_blankbar_col;
 
 /* Exported globals */
 SDL_Surface *plr_messages[4];
-unsigned char player_teams[4];
-char plr_teams_left;
-char radars_visible;
+int player_teams[4];
+int plr_teams_left;
+int radars_visible;
 Player players[4];
 #ifdef CHEAT_POSSIBLE
 char cheat, cheat1;
 #endif
 
 /* Internally used functions */
-static void player_key_update (unsigned char plr);
-static inline void draw_player_statusbars (void);
-static void draw_player_weaponselection (int plr);
-static void draw_player_message (int plr);
-/*static void draw_player_criticals(int plr);*/
+static void player_key_update(unsigned char plr);
 
 /* Initialize players */
 int init_players (void) {
@@ -102,11 +98,9 @@ int init_players (void) {
 void reinit_players (void) {
     int p, unlucky = -1;
     endgame = -1;
-    plr_teams_left = 0;
     if (strcmp (game_settings.players_in, "    ") == 0) {
-        printf
-            ("Oops ! Entered reinit_players(void) without any players selected !\n");
-        exit (1);
+        printf("Bug! reinit_players(): no players selected!\n");
+        abort();
     }
     if (game_settings.playmode == OutsideShip1) {
         do
@@ -120,6 +114,7 @@ void reinit_players (void) {
             continue;
         }
         players[p].state = ALIVE;
+        players[p].recall_cooloff = 0;
         init_pilot(&players[p].pilot,p);
         memset (&players[p].controller, 0, sizeof (GameController));
         if (plr_weapons[p]) {
@@ -150,7 +145,6 @@ void reinit_players (void) {
             for (c = 0; c < cc; c++)
                 ship_critical (players[p].ship, 0);
         }
-        plr_teamc[p] = 0;
         if (p != unlucky) {
             players[p].ship->x = lev_level.player_def_x[0][p];
             players[p].ship->y = lev_level.player_def_y[0][p];
@@ -168,6 +162,7 @@ void reinit_players (void) {
         }
     }
     /* Count the number of teams and the number of players in each of them */
+    memset(plr_teamc,0,sizeof(int)*4);
     for (p = 0; p < 4; p++)
         if (players[p].state)
             plr_teamc[player_teams[p]]++;       /* Number of players per team */
@@ -178,82 +173,51 @@ void reinit_players (void) {
     if (plr_teams_left == 1) {
         plr_teams_left++;
     }
-    reinit_animation ();
-}
 
-/* All sorts of information for the player */
-void draw_player_hud(void) {
-    int p;
-    draw_player_statusbars ();
-    for (p = 0; p < 4; p++) {
-        if (players[p].state==ALIVE) {
-#if 0
-            if(players[p].ship && players[p].ship->criticals) draw_player_criticals(p);
-#endif
-            if (players[p].weapon_select)
-                draw_player_weaponselection (p);
-            if (player_message[p])
-                draw_player_message (p);
-        }
-    }
+    /* Initialize player screen geometry */
+    reinit_animation ();
+    recalc_geometry ();
+    rearrange_animation ();
 }
 
 /* Draw the player statusbars */
-static inline void draw_player_statusbars (void) {
-    int p, p1, p2;
-    SDL_Rect rect, rect2, rect3;
+static void draw_player_statusbar (int p) {
+    SDL_Rect outline,health,energy;
     Uint32 healthcol;
-    rect.y = 234;
-    rect.w = 280;
-    rect.h = 2;
-    rect2.h = 2;
-    rect2.y = 234;
-    rect3.y = 233;
-    rect3.w = 282;
-    rect3.h = 7;
-    p = 0;
-    for (p1 = 0; p1 < 2; p1++) {
-        rect.x = 20;
-        rect3.x = 19;
-        for (p2 = 0; p2 < 2; p2++) {
-            if (players[p].ship) {
-                rect.w = 280.0 * players[p].ship->health;
-                if (players[p].ship->health > 0.5)
-                    healthcol = plr_healthbar_col;
-                else if (players[p].ship->health > 0.2)
-                    healthcol = plr_healthbar_col2;
-                else
-                    healthcol = plr_healthbar_col3;
-                SDL_FillRect (screen, &rect3, 0);
-                SDL_FillRect (screen, &rect, healthcol);
-                rect2.x = rect.x + rect.w;
-                rect2.w = 280 - rect.w;
-                SDL_FillRect (screen, &rect2, plr_blankbar_col);
-                rect.w = 280.0 * players[p].ship->energy;
-                rect.y += 3;
-                rect2.y += 3;
-                SDL_FillRect (screen, &rect, plr_energybar_col);
-                rect2.x = rect.x + rect.w;
-                rect2.w = 280 - rect.w;
-                SDL_FillRect (screen, &rect2, plr_blankbar_col);
-                rect2.y -= 3;
-                rect.y -= 3;
-            }
-            rect.x += 320;
-            rect3.x += 320;
-            p++;
-        }
-        rect.y += 240;
-        rect2.y += 240;
-        rect3.y += 240;
-    }
+    outline.x = lev_rects[p].x + 19;
+    outline.y = lev_rects[p].y + cam_rects[p].h - 7;
+    outline.w = cam_rects[p].w - 38;
+    outline.h = 7;
+
+    health = outline;
+    health.x++; health.w -= 2;
+    health.y++; health.h = 2;
+
+    energy = health;
+    energy.y += 3;
+    
+    SDL_FillRect (screen, &outline, 0);
+    SDL_FillRect (screen, &health, plr_blankbar_col);
+    SDL_FillRect (screen, &energy, plr_blankbar_col);
+
+    if (players[p].ship->health > 0.5)
+        healthcol = plr_healthbar_col;
+    else if (players[p].ship->health > 0.2)
+        healthcol = plr_healthbar_col2;
+    else
+        healthcol = plr_healthbar_col3;
+
+    health.w *= players[p].ship->health;
+    energy.w *= players[p].ship->energy;
+    SDL_FillRect (screen, &health, healthcol);
+    SDL_FillRect (screen, &energy, plr_energybar_col);
 }
 
 /* Draw the weapon selection screen for player */
 static void draw_player_weaponselection (int plr) {
     SDL_Rect rect = lev_rects[plr];
     SDL_BlitSurface (plr_weaponsel_bg, NULL, screen, &rect);
-    /* Check if the string surface needs an update */
+    /* Update the string surface if necessary */
     if (players[plr].specialWeapon != players[plr].ship->special
         || plr_weapons[plr] == NULL) {
         players[plr].specialWeapon = players[plr].ship->special;
@@ -263,16 +227,17 @@ static void draw_player_weaponselection (int plr) {
                           font_color_green);
     }
     /* Blit text to screen */
-    rect.x += 160 - plr_weapons[plr]->w / 2;
-    rect.y += 120 - plr_weapons[plr]->h / 2;
+    rect.x += cam_rects[plr].w/2 - plr_weapons[plr]->w / 2;
+    rect.y += cam_rects[plr].h/2 - plr_weapons[plr]->h / 2;
     SDL_BlitSurface (plr_weapons[plr], NULL, screen, &rect);
 }
 
+/* Draw player messages */
 static void draw_player_message (int plr)
 {
     SDL_Rect rect;
-    rect.x = (plr == 1 || plr == 3) * 320 + 160 - plr_messages[plr]->w / 2;
-    rect.y = (plr > 1) * 240 + 120 - plr_messages[plr]->h / 2;
+    rect.x = lev_rects[plr].x + cam_rects[plr].w/2 - plr_messages[plr]->w/2;
+    rect.y = lev_rects[plr].y + cam_rects[plr].h/2 - plr_messages[plr]->h/2;
     SDL_BlitSurface (plr_messages[plr], NULL, screen, &rect);
 }
 
@@ -295,6 +260,25 @@ static  void draw_player_criticals(int plr) {
 }
 #endif
 
+/* All sorts of information for the player */
+void draw_player_hud(void) {
+    int p;
+    for (p = 0; p < 4; p++) {
+        if (players[p].state==ALIVE || players[p].state==DEAD) {
+            if(players[p].ship) {
+                draw_player_statusbar(p);
+#if 0
+                if(players[p].ship->criticals) draw_player_criticals(p);
+#endif
+            }
+            if (players[p].weapon_select)
+                draw_player_weaponselection (p);
+            if (player_message[p])
+                draw_player_message (p);
+        }
+    }
+}
+
 void draw_radar (SDL_Rect rect, int plr) {
     double d;
     int p, dx, dy, dx2, dy2;
@@ -330,8 +314,8 @@ void player_cleanup (void) {
 /*** PLAYER ANIMATION ***/
 void animate_players ()
 {
-    int n, newx = -1, newy = -1;
     struct dllist *ships;
+    int n,newx,newy;
     if (plr_teams_left == 0 && endgame == -1)
         endgame = 30;           /* In case of tie game, end it */
     for (n = 0; n < 4; n++) {
@@ -341,6 +325,8 @@ void animate_players ()
         game_status.lifetime[n]++;
         if (player_message[n] > 0)
             player_message[n]--;
+        if(players[n].recall_cooloff>0)
+            players[n].recall_cooloff--;
         /* Weapon selection */
         if (players[n].weapon_select) {
             if (players[n].ship->onbase == 0) {
@@ -371,7 +357,7 @@ void animate_players ()
                     endgame = 30;
             }
             /* Jump into an available ship */
-            ships = getshiplist ();
+            ships = ship_list;
             while (ships) {
                 struct Ship *ship=ships->data;
                 if (players[n].pilot.x >= ship->x - 8
@@ -384,6 +370,7 @@ void animate_players ()
                         if (find_player (ship) < 0
                             && ship->eject_cooloff == 0) {
                             players[n].ship = ship;
+                            players[n].recall_cooloff = 0;
                             if (players[n].ship->color == Grey)
                                 claim_ship (players[n].ship, n);
                         }
@@ -399,11 +386,11 @@ void animate_players ()
         }
         /* Update camera position */
         if (players[n].ship) {
-            newx = round(players[n].ship->x);
-            newy = round(players[n].ship->y);
+            newx = Round(players[n].ship->x);
+            newy = Round(players[n].ship->y);
         } else {
-            newx = round(players[n].pilot.x);
-            newy = round(players[n].pilot.y);
+            newx = Round(players[n].pilot.x);
+            newy = Round(players[n].pilot.y);
             if (players[n].pilot.parachuting) {
                 newy +=
                     players[n].pilot.sprite[2]->h -
@@ -413,18 +400,18 @@ void animate_players ()
                     players[n].pilot.sprite[0]->w / 2;
             }
         }
-        if (newx < 0)
-            continue;
-        cam_rects[n].x = newx - 180;
+        /* BUG: Sometimes with larger videomodes, setting y here will
+         * apparently clobber x */
+        cam_rects[n].x = newx - cam_rects[n].w/2;
+        cam_rects[n].y = newy - cam_rects[n].h/2;
         if (cam_rects[n].x < 0)
             cam_rects[n].x = 0;
-        if (cam_rects[n].x > lev_level.width - 320)
-            cam_rects[n].x = lev_level.width - 320;
-        cam_rects[n].y = newy - 120;
+        else if (cam_rects[n].x > lev_level.width - cam_rects[n].w)
+            cam_rects[n].x = lev_level.width - cam_rects[n].w;
         if (cam_rects[n].y < 0)
             cam_rects[n].y = 0;
-        if (cam_rects[n].y > lev_level.height - 240)
-            cam_rects[n].y = lev_level.height - 240;
+        else if (cam_rects[n].y > lev_level.height - cam_rects[n].h)
+            cam_rects[n].y = lev_level.height - cam_rects[n].h;
     }
 }
 
@@ -509,8 +496,7 @@ void player_joybuttonhandler (SDL_JoyButtonEvent * button)
     }
 }
 
-/* Generic input handling		  */
-/* (this is where the real magic happens) */
+/* Generic input handling (this is where the real magic happens) */
 void player_key_update (unsigned char plr) {
     struct Ship *ship;
     if(players[plr].state!=ALIVE) return;
@@ -543,10 +529,11 @@ void player_key_update (unsigned char plr) {
             if ((int) players[plr].ship->remote_control > 1)
                 ship = players[plr].ship->remote_control;
             else
-                return;         /* If you are being remote controlled, controls are disabled */
+                return; /* If you are being remote controlled, controls are disabled */
         } else
             ship = players[plr].ship;
-        if (players[plr].controller.weapon2 && ship != players[plr].ship) {     /* Disengage remote control */
+        if (players[plr].controller.weapon2 && ship != players[plr].ship) {
+            /* Disengage remote control */
             ship_fire (players[plr].ship, SpecialWeapon);
             return;
         }
@@ -561,7 +548,8 @@ void player_key_update (unsigned char plr) {
             && !(ship->criticals & CRITICAL_STDWEAPON);
         if (ship->fire_special_weapon != players[plr].controller.weapon2
             && !(ship->criticals & CRITICAL_SPECIAL)) {
-            switch (ship->special) {    /* Some weapons need to be shot only once */
+            switch (ship->special) {
+            /* Some weapons need to be fired only once */
             case WCloak:
             case WShield:
             case WGhostShip:
@@ -583,7 +571,8 @@ void player_key_update (unsigned char plr) {
     }
 }
 
-/* If a grenade explodes in the woods and there is nobody around to hear it, does it really make a sound ? */
+/* If a grenade explodes in the woods and there is nobody around to hear it,
+ * does it really make a sound ? */
 int hearme (int x, int y) {
     double dist = 9999, d;
     int nearest=-1;
@@ -604,8 +593,7 @@ int hearme (int x, int y) {
     return -1;
 }
 
-
-void set_player_message (unsigned char plr, FontSize size, SDL_Color color,
+void set_player_message (int plr, FontSize size, SDL_Color color,
                          int dur, const char *msg, ...)
 {
     char buf[256];
@@ -618,14 +606,16 @@ void set_player_message (unsigned char plr, FontSize size, SDL_Color color,
         plr_messages[plr] = NULL;
     }
     if (strlen (buf) && dur) {
-        /*SDL_FillRect(plr_messages[plr],NULL,SDL_MapRGB(plr_messages[plr]->format,0,0,0));
-           centered_string(plr_messages[plr],size,0,buf,color); */
         plr_messages[plr] = renderstring (size, buf, color);
         player_message[plr] = dur;
     } else
         player_message[plr] = 0;
 }
 
+/* Finish off a player (set state to dead, remove ship...) */
+/* This doesn't actually set player state to BURIED. That is set */
+/* in animation.c code after the player has been dead for FADE_STEP frames */
+/* so the fadeout animation can be drawn. */
 void buryplayer (int plr) {
     if(players[plr].state!=ALIVE) {
         printf("Bug! buryplayer(%d): Player is already dead or inactive!\n",plr);
@@ -636,11 +626,11 @@ void buryplayer (int plr) {
         if (plr_teamc[player_teams[plr]] == 0)
             plr_teams_left--;
         set_player_message (plr, Bigfont, font_color_red, -1, "Dead");
-        reinit_animation ();
         kill_plr_screen (plr);
     }
 }
 
+/* Eject the pilot */
 void eject_pilot (int plr) {
     pilot_any_ejected++;
     if (players[plr].ship) {
@@ -677,8 +667,11 @@ void recall_ship (int plr) {
     struct dllist *ships;
     int taken = 0;
     int tx, ty;
+
+    if(players[plr].recall_cooloff>0) return;
+
     /* First, attempt to locate the ship */
-    ships = getshiplist ();
+    ships = ship_list;
     while (ships) {
         if (((struct Ship*)ships->data)->color == Red + plr) {
             if (find_player (ships->data)<0)
@@ -732,6 +725,7 @@ void recall_ship (int plr) {
         jump->link = NULL;
         addspecial (jump);
     }
+    players[plr].recall_cooloff=JPLONGLIFE*2;
 }
 
 #ifdef CHEAT_POSSIBLE
