@@ -30,6 +30,6 @@
 SDL_Surface *sbmp_to_surface (Uint8 * sbmp);
 
 /* This converts an SDL surface into a Small Bitmap binary */
-char *surface_to_sbmp (Uint32 * len, SDL_Surface * surface);
+Uint8 *surface_to_sbmp (Uint32 * len, SDL_Surface * surface);
 
 #endif
