@@ -44,10 +44,11 @@
 /* Exported globals */
 SDL_Surface **ship_gfx[7];      /* 0=grey, 1-4=coloured, 5 = white, 6 =  frozen */
 SDL_Surface **ghost_gfx[4];
+struct dllist *ship_list;
+
 /* Internally used globals */
 static SDL_Surface *shield_gfx[4];      /* 0-3 coloured */
 static SDL_Surface **remocon_gfx;
-static struct dllist *ships;
 static Vector really_weak_gravity, strong_gravity;
 
 /* Internally used functions */
@@ -66,7 +67,7 @@ static void ship_splash (struct Ship * ship);
 int init_ships (void) {
     LDAT *playerdata;
     int r, p;
-    ships = NULL;
+    ship_list = NULL;
     really_weak_gravity = makeVector (0, -WEAP_GRAVITY / 4.0);
     strong_gravity = makeVector (0, -GRAVITY * 2.0);
     /* Load ship graphics */
@@ -127,7 +128,7 @@ int init_ships (void) {
             recolor (shield_gfx[r], 1, 1, 0.6, 1);
             break;
         default:
-            printf ("Warning ! Value of 'r' is %d in init_ships() !\n", r);
+            printf ("Warning ! Value of 'r' is %d in init_ships()!\n", r);
             break;
         }
     }
@@ -145,8 +146,8 @@ int init_ships (void) {
 /* Remove ships */
 void clean_ships (void)
 {
-    dllist_free(ships,free);
-    ships=NULL;
+    dllist_free(ship_list,free);
+    ship_list=NULL;
 }
 
 /* Prepare for a new level */
@@ -212,22 +213,17 @@ struct Ship *create_ship (PlayerColor color, SWeaponType weapon, WeaponType spec
     newship->special = special;
     newship->color = color;
 
-    if(ships)
-        dllist_append(ships,newship);
+    if(ship_list)
+        dllist_append(ship_list,newship);
     else
-        ships=dllist_append(ships,newship);
+        ship_list=dllist_append(ship_list,newship);
     return newship;
 }
 
-/* Get the list of ships */
-struct dllist *getshiplist (void) {
-    return ships;
-}
-
 /* Draw ships on screen */
 void draw_ships (void)
 {
-    struct dllist *current=ships;
+    struct dllist *current=ship_list;
     SDL_Rect rect, rect2;
     int plr, pose;
     struct Ship *ship;
@@ -237,21 +233,21 @@ void draw_ships (void)
             for (plr = 0; plr < 4; plr++) {
                 rect.w = ship_gfx[Grey][0]->w;
                 rect.h = ship_gfx[Grey][0]->h;
-                if (players[plr].state==ALIVE) {
+                if (players[plr].state==ALIVE||players[plr].state==DEAD) {
                     rect.x =
-                        lev_rects[plr].x + round(ship->x) - cam_rects[plr].x - 8;
+                        lev_rects[plr].x + Round(ship->x) - cam_rects[plr].x - 8;
                     rect.y =
-                        lev_rects[plr].y + round(ship->y) - cam_rects[plr].y - 8;
+                        lev_rects[plr].y + Round(ship->y) - cam_rects[plr].y - 8;
                     if (rect.x < lev_rects[plr].x - 16
                         || rect.y < lev_rects[plr].y - 16
-                        || rect.x > lev_rects[plr].x + lev_rects[plr].w
-                        || rect.y > lev_rects[plr].y + lev_rects[plr].h)
+                        || rect.x > lev_rects[plr].x + cam_rects[plr].w
+                        || rect.y > lev_rects[plr].y + cam_rects[plr].h)
                         continue;
                     rect2 =
                         cliprect (rect.x, rect.y, rect.w, rect.h,
                                   lev_rects[plr].x, lev_rects[plr].y,
-                                  lev_rects[plr].x + lev_rects[plr].w,
-                                  lev_rects[plr].y + lev_rects[plr].h);
+                                  lev_rects[plr].x + cam_rects[plr].w,
+                                  lev_rects[plr].y + cam_rects[plr].h);
                     if (rect.x < lev_rects[plr].x)
                         rect.x = lev_rects[plr].x;
                     if (rect.y < lev_rects[plr].y)
@@ -259,7 +255,7 @@ void draw_ships (void)
                     if (players[plr].ship == ship && radars_visible)
                         draw_radar (rect, plr);
                     pose =
-                        roundplus ((35 - ship->angle / (2.0 * M_PI / 35.0)));
+                        Roundplus ((35 - ship->angle / (2.0 * M_PI / 35.0)));
                     if (pose > 35)
                         pose = 35;
                     if (ship->dead)
@@ -277,17 +273,17 @@ void draw_ships (void)
                     if (ship->shieldup) {
                         SDL_Rect sr, tr;
                         tr.x =
-                            lev_rects[plr].x + round(ship->x) - cam_rects[plr].x
+                            lev_rects[plr].x + Round(ship->x) - cam_rects[plr].x
                             - 16;
                         tr.y =
-                            lev_rects[plr].y + round(ship->y) - cam_rects[plr].y
+                            lev_rects[plr].y + Round(ship->y) - cam_rects[plr].y
                             - 16;
                         tr.w = ship->shield->w;
                         tr.h = ship->shield->h;
                         sr = cliprect (tr.x, tr.y, tr.w, tr.h,
                                        lev_rects[plr].x, lev_rects[plr].y,
-                                       lev_rects[plr].x + lev_rects[plr].w,
-                                       lev_rects[plr].y + lev_rects[plr].h);
+                                       lev_rects[plr].x + cam_rects[plr].w,
+                                       lev_rects[plr].y + cam_rects[plr].h);
                         if (tr.x < lev_rects[plr].x)
                             tr.x = lev_rects[plr].x;
                         if (tr.y < lev_rects[plr].y)
@@ -297,17 +293,17 @@ void draw_ships (void)
                     if (ship->remote_control) {
                         SDL_Rect sr, tr;
                         tr.x =
-                            lev_rects[plr].x + round(ship->x) - cam_rects[plr].x -
+                            lev_rects[plr].x + Round(ship->x) - cam_rects[plr].x -
                             16;
                         tr.y =
-                            lev_rects[plr].y + round(ship->y) - cam_rects[plr].y -
+                            lev_rects[plr].y + Round(ship->y) - cam_rects[plr].y -
                             16;
                         tr.w = remocon_gfx[ship->anim]->w;
                         tr.h = remocon_gfx[ship->anim]->h;
                         sr = cliprect (tr.x, tr.y, tr.w, tr.h,
                                        lev_rects[plr].x, lev_rects[plr].y,
-                                       lev_rects[plr].x + lev_rects[plr].w,
-                                       lev_rects[plr].y + lev_rects[plr].h);
+                                       lev_rects[plr].x + cam_rects[plr].w,
+                                       lev_rects[plr].y + cam_rects[plr].h);
                         if (tr.x < lev_rects[plr].x)
                             tr.x = lev_rects[plr].x;
                         if (tr.y < lev_rects[plr].y)
@@ -347,15 +343,55 @@ static void finalize_ship(struct Ship *ship) {
     int num = find_player (ship);
     ship->dead = 2;
     ship->visible = 0;
-    if (num >= 0 && players[num].ship) {
+
+    /* Mark the player as dead if he has not yet ejected */
+    if (num >= 0) {
         buryplayer (num);
     }
-    spawn_clusters (round(ship->x), round(ship->y), 32, Cannon);
+
+    /* Make sure no pilots have roped this ship */
+    for(num=0;num<4;num++) {
+        if(players[num].state == ALIVE && players[num].pilot.rope_ship==ship) {
+            pilot_detach_rope(&players[num].pilot);
+        }
+    }
+
+    /* Explosion and sound effect */
+    spawn_clusters (Round(ship->x), Round(ship->y), 32, Cannon);
+    playwave(WAV_EXPLOSION2);
+}
+
+/** Stop special weapons **/
+static void ship_stop_special(struct Ship *ship) {
+    if (ship->remote_control) {
+        if ((int) ship->remote_control > 1) {
+            ship_fire (ship, SpecialWeapon);
+        } else {
+            struct dllist *tmp2 = ship_list;
+            while (tmp2) {
+                if (((struct Ship*)tmp2->data)->remote_control == ship) {
+                    ship_fire (tmp2->data, SpecialWeapon);
+                    break;
+                }
+                tmp2 = tmp2->next;
+            }
+        }
+    }
+    ship->fire_special_weapon = 0;
+    ship->antigrav = 0;
+    ship->shieldup = 0;
+    ship->afterburn = 1;
+    ship->repairing = 0;
+    ship->visible = 1;
+    if (ship->ghost) {
+        ship->ghost = 0;
+        ghostify (ship, 0);
+    }
 }
 
 /** Ship animation **/
 void animate_ships (void) {
-    struct dllist *current = ships, *tmplist = NULL;
+    struct dllist *current = ship_list, *tmplist = NULL;
     double newx,newy;
     Vector tmpv, sv;
     struct Ship *ship;
@@ -376,6 +412,8 @@ void animate_ships (void) {
                 ship->tagged--;
             if (ship->white_ship > 0)
                 ship->white_ship--;
+            if (ship->no_power > 0)
+                ship->no_power--;
             if (ship->visible < 0) {
                 if (ship->visible == -1)
                     ship->visible = 1;
@@ -384,18 +422,20 @@ void animate_ships (void) {
             }
             if ((ship->criticals & CRITICAL_CARGO) && ship->energy > 0)
                 ship->energy -= 0.001;
-            /* Turn the ship */
-            ship->angle += ship->turn;
-            /* Fire normal weapon */
-            if (ship->fire_weapon && ship->cooloff == 0)
-                ship_fire (ship, NormalWeapon);
-            /* Fire special weapons. */
-            if (ship->fire_special_weapon) {
-                if (ship_fire_special_weapon (ship))
-                    ship_fire (ship, SpecialWeapon2);
-                else if (ship->special != WWatergun
-                         && ship->special != WFlame)
-                    ship_fire (ship, SpecialWeapon);
+            if(ship->no_power == 0) {
+                /* Turn the ship */
+                ship->angle += ship->turn;
+                /* Fire normal weapon */
+                if (ship->fire_weapon && ship->cooloff == 0)
+                    ship_fire (ship, NormalWeapon);
+                /* Fire special weapons. */
+                if (ship->fire_special_weapon) {
+                    if (ship_fire_special_weapon (ship))
+                        ship_fire (ship, SpecialWeapon2);
+                    else if (ship->special != WWatergun
+                             && ship->special != WFlame)
+                        ship_fire (ship, SpecialWeapon);
+                }
             }
             sv = ship->vector;
             /* Player specials like cloaking device and such */
@@ -405,8 +445,8 @@ void animate_ships (void) {
                 ship->angle = 0;
             if (ship->angle < 0)
                 ship->angle = 2 * M_PI;
-            if (ship->thrust || ship->afterburn > 1 || ship->darting
-                || ((ship->criticals & CRITICAL_FUELCONTROL)
+            if ((ship->thrust && ship->no_power==0) || ship->afterburn > 1 ||
+                    ship->darting || ((ship->criticals & CRITICAL_FUELCONTROL)
                     && rand () % 6 == 0)) {
                 if (ship->darting == DARTING)
                     tmpv =
@@ -432,7 +472,7 @@ void animate_ships (void) {
                     ship->health -= 0.001;
                     if (ship->health <= 0.0) {
                         killship (ship);
-                        spawn_clusters (round(ship->x), round(ship->y), 6, Napalm);
+                        spawn_clusters (Round(ship->x), Round(ship->y), 6, Napalm);
                     }
                     for (s = 0; s < 4; s++) {
                         smoke =
@@ -442,7 +482,7 @@ void animate_ships (void) {
                         smoke->vector.y = 3.0 * (rand () % 20) / 10.0;
                     }
                 }
-                solid = hit_solid (round(ship->x), round(ship->y));
+                solid = hit_solid (Round(ship->x), Round(ship->y));
                 sv = addVectors (&ship->vector, &tmpv);
                 if (ship->visible)
                     ship_exhaust (ship, tmpv, solid);
@@ -458,7 +498,7 @@ void animate_ships (void) {
                 ship->angle = M_PI * 2.0;
         }
         /* Check if someone is operating a forcefield nearby */
-        tmplist = ships;
+        tmplist = ship_list;
         while (tmplist) {
             struct Ship *tmpship=tmplist->data;
             if (tmpship != ship && tmpship->shieldup) {
@@ -515,15 +555,15 @@ void animate_ships (void) {
             if (num >= 0) {
                 buryplayer (num);
             }
-            spawn_clusters (round(ship->x), round(ship->y), 32, Cannon);
-            spawn_clusters (round(ship->x), round(ship->y), 32, FireStarter);
+            spawn_clusters (Round(ship->x), Round(ship->y), 32, Cannon);
+            spawn_clusters (Round(ship->x), Round(ship->y), 32, FireStarter);
         }
         solid = 0;
         if (newx < 1) {
             newx = 1;
             ship->vector.x=0;
             solid = 1;
-        } else if (round(newx) >= lev_level.width) {
+        } else if (Round(newx) >= lev_level.width) {
             newx = lev_level.width - 1;
             ship->vector.x=0;
             solid = 1;
@@ -548,23 +588,15 @@ void animate_ships (void) {
             }
         }
         /* Terrain collision detection */
-        if (lev_level.solid[round(newx)][round(newy) + 1] == TER_BASE
-            && lev_level.solid[round(newx)][round(newy)] != TER_BASE && ship->ghost == 0) {
+        if (lev_level.solid[Round(newx)][Round(newy) + 1] == TER_BASE
+            && lev_level.solid[Round(newx)][Round(newy)] != TER_BASE && ship->ghost == 0)
+        {
             ship->x = newx;
             ship->y = newy;
         }
-#if 0
-        else if (lev_level.solid[round(newx)][round(newy) - 1] == TER_BASE
-                   && lev_level.solid[round(newx)][round(newy)] != TER_BASE
-                   && ship->ghost == 0) {
-            /*ship->x=newx;
-            ship->y=newy;*/
-            newy--;
-        }
-#endif
         solid = hit_solid (newx, newy);
         if (solid < 0 && ship->ghost == 0) {    /* Underwater ? */
-            if (lev_level.solid[round(ship->x)][round(ship->y)] == TER_FREE) {
+            if (lev_level.solid[Round(ship->x)][Round(ship->y)] == TER_FREE) {
                 /* Splash when going in to the water */
                 ship_splash (ship);
             }
@@ -600,16 +632,16 @@ void animate_ships (void) {
                 if (ship->darting == SPEARED) {  /* If the ship is speared, it sinks into the terrain a bit */
                     unsigned char tmp_solid;
                     tmp_solid =
-                        lev_level.solid[round(ship->x - ship->vector.x)]
-                                       [round(ship->y-ship->vector.y)];
+                        lev_level.solid[Round(ship->x - ship->vector.x)]
+                                       [Round(ship->y-ship->vector.y)];
                     if (tmp_solid != TER_INDESTRUCT && tmp_solid != TER_BASE) {
                         ship->x -= ship->vector.x;
                         ship->y -= ship->vector.y;
                         if (tmp_solid == TER_EXPLOSIVE)
-                            spawn_clusters (round(ship->x), round(ship->y),
+                            spawn_clusters (Round(ship->x), Round(ship->y),
                                     6, Cannon);
                         else if (tmp_solid == TER_EXPLOSIVE2)
-                            spawn_clusters (round(ship->x), round(ship->y),
+                            spawn_clusters (Round(ship->x), Round(ship->y),
                                     6, Grenade);
                     }
                     ship->health -= 0.05; /* Spears hurt even more than darts ! */
@@ -639,8 +671,8 @@ void animate_ships (void) {
             } else {
                 if (ship->vector.y > 0) {       /* Burrow in snow */
                     int snowx,snowy;
-                    snowx = round(newx);
-                    snowy = round(newy);
+                    snowx = Round(newx);
+                    snowy = Round(newy);
                     if (solid == TER_SNOW) {
                         putpixel (lev_level.terrain, snowx, snowy, col_black);
                         lev_level.solid[snowx][snowy] = TER_FREE;
@@ -649,7 +681,7 @@ void animate_ships (void) {
                                   lev_watercol);
                         lev_level.solid[snowx][snowy] = TER_WATER;
                     }
-                    add_snowflake (round(ship->x), round(ship->y));
+                    add_snowflake (Round(ship->x), Round(ship->y));
                 }
                 ship->vector.y = 0;
                 ship->vector.x /= 2;
@@ -657,7 +689,12 @@ void animate_ships (void) {
             /* Is the player sitting on a base ? */
             if (solid == TER_BASE && ship->dead == 0) {
                 ship->onbase = 1;
-                if (ship->health < 1 && !ship->shieldup) {      /* Repair the ship */
+
+                /* Counter the antigravity weapon */
+                if(ship->special==WAntigrav) ship->vector.y+=gravity.y;
+
+                /* Repair the ship */
+                if (ship->health < 1 && !ship->shieldup) {      
                     Particle *spark;
                     ship->health += 0.0012;
                     /* Special effects */
@@ -685,7 +722,7 @@ void animate_ships (void) {
                         ship_critical (ship, 1);
                 }
                 /* Straighten the ship */
-                if (lev_level.solid[round(newx)][round(newy) + 1] == TER_WATER) {
+                if (lev_level.solid[Round(newx)][Round(newy) + 1] == TER_WATER) {
                     ship->angle = M_PI;
                 } else {
                     ship->angle = 0;
@@ -696,14 +733,14 @@ void animate_ships (void) {
             newx = ship->x;
             newy = ship->y;
         } else {
-            if (hit_solid (round(ship->x), round(ship->y)) < 0) {     /* Splash when coming out of the water */
+            if (hit_solid (Round(ship->x), Round(ship->y)) < 0) {     /* Splash when coming out of the water */
                 ship_splash (ship);
             }
             ship->onbase = 0;
         }
         /* Ship collisions */
         if (game_settings.ship_collisions && !ship->ghost && ship->dead == 0)
-            tmplist = ships;
+            tmplist = ship_list;
         while (tmplist) {
             struct Ship *tmpship=tmplist->data;
             if (tmpship == ship || tmpship->dead) {
@@ -735,8 +772,8 @@ void animate_ships (void) {
             if (p >= 0)
                 players[p].ship = NULL;
             free(ship);
-            if(current==ships)
-                ships=dllist_remove(current);
+            if(current==ship_list)
+                ship_list=dllist_remove(current);
             else
                 dllist_remove(current);
         }
@@ -772,21 +809,9 @@ static void ship_splash (struct Ship * ship)
 /* Ship turns gray and starts falling */
 void killship (struct Ship * ship)
 {
-    /* Stop remote control */
-    if (ship->remote_control) {
-        if ((int) ship->remote_control > 1) {
-            ship_fire (ship, SpecialWeapon);
-        } else {
-            struct dllist *tmp2 = ships;
-            while (tmp2) {
-                if (((struct Ship*)tmp2->data)->remote_control == ship) {
-                    ship_fire (tmp2->data, SpecialWeapon);
-                    break;
-                }
-                tmp2 = tmp2->next;
-            }
-        }
-    }
+    /* Stop special weapons */
+    ship_stop_special(ship);
+
     /* Kill the ship */
     ship->dead = 1;
     ship->frozen = 0;
@@ -795,55 +820,58 @@ void killship (struct Ship * ship)
     ship->visible = 1;
     ship->thrust = 0;
     ship->fire_weapon = 0;
-    ship->fire_special_weapon = 0;
-    ship->antigrav = 0;
+    /* Change the antigravity weapon to something else so the ship can fall */
     if (ship->special == WAntigrav)
-        ship->special = WGrenade;       /* So the ship fill fall when grey */
+        ship->special = WGrenade;
+
     /* This is borrowed to set which way the ship will start to spin */
     if (ship->angle < M_PI)
         ship->turn = 0.6;
     else
         ship->turn = -0.6;
-    if (ship->ghost) {
-        ship->ghost = 0;
-        ghostify (ship, 0);
-    }
+
+    /* Sound effect */
+    playwave(WAV_CRASH);
 }
 
 /* A random critical hit */
-void ship_critical (struct Ship * ship, char repair)
+void ship_critical (struct Ship * ship, int repair)
 {
-    unsigned char c;
-#if CRITICAL_COUNT > 7
-#warning Add more handlers for critical hits !
+    int c;
+#if CRITICAL_COUNT > 8
+#error Add more handlers for critical hits!
 #endif
     c = rand () % CRITICAL_COUNT;
     switch (c) {
-    case 0:
+    case 0: /* Damaged engine */
         c = CRITICAL_ENGINE;
         break;
-    case 1:
+    case 1: /* Malfunctioning fuel control */
         c = CRITICAL_FUELCONTROL;
         break;
-    case 2:
+    case 2: /* Left thruster out of order */
         c = CRITICAL_LTHRUSTER;
         break;
-    case 3:
+    case 3: /* Right thruster out of order */
         c = CRITICAL_RTHRUSTER;
         break;
-    case 4:
+    case 4: /* Leaking cargo bay */
         c = CRITICAL_CARGO;
         break;
-    case 5:
+    case 5: /* Standard weapon out of order */
         c = CRITICAL_STDWEAPON;
         ship->fire_weapon = 0;
         break;
-    case 6:
+    case 6: /* Special weapon out of order */
         c = CRITICAL_SPECIAL;
-        ship->fire_special_weapon = 0;
+        ship_stop_special(ship);
+        break;
+    case 7: /* Temporary power failure */
+        c = CRITICAL_POWER;
+        ship->no_power = 60;
         break;
     default:
-        printf ("Unhandled critical hit !\n");
+        printf ("Bug! ship_critical(): unhandled critical hit!\n");
         return;
     }
     if ((ship->criticals & c) == 0) {
@@ -851,10 +879,10 @@ void ship_critical (struct Ship * ship, char repair)
         if (repair)
             return;
         if (c == CRITICAL_LTHRUSTER && (ship->criticals & CRITICAL_RTHRUSTER))
-            return;             /* We won't leave the ship TOTALLY disabled... */
+            return;           /* We won't leave the ship TOTALLY disabled... */
         if (c == CRITICAL_RTHRUSTER && (ship->criticals & CRITICAL_LTHRUSTER))
             return;
-        ship->criticals |= c;
+        if(c != CRITICAL_POWER) ship->criticals |= c;
         if (plr >= 0)
             set_player_message (plr, Bigfont, font_color_red, 25,
                                 critical2str (c));
@@ -883,14 +911,12 @@ char ship_fire_special_weapon (struct Ship * ship)
     case WZapper:
         fire = 1;
         ship->energy -= 0.0075;
-#if HAVE_LIBSDL_MIXER
         if (ship->repeat_audio == 0) {
             playwave (WAV_ZAP);
             ship->repeat_audio = 9;
         } else
             ship->repeat_audio--;
         break;
-#endif
     case WWatergun:
         if (hit_solid (ship->x, ship->y) < 0) {
             ship->energy += 0.02;
@@ -1051,13 +1077,11 @@ static void ship_specials (struct Ship * ship) {
     }
     /* Afterburner */
     else if (ship->afterburn > 1) {
-#if HAVE_LIBSDL_MIXER
         if (ship->repeat_audio == 0) {
             playwave (WAV_BURN);
             ship->repeat_audio = 15;
         } else
             ship->repeat_audio--;
-#endif
 #ifdef CHEAT_POSSIBLE
         if (!cheat1)
 #endif
@@ -1164,6 +1188,22 @@ void ghostify (struct Ship * ship, char status)
         ship->ship = ship_gfx[ship->color];
 }
 
+/* Check if there is a ship in the specified coordinates and bump
+ * it one pixel upwards if there is. This is used to keep ships
+ * from getting stuck in regenerating bases */
+void bump_ship(int x,int y) {
+    struct dllist *shplst=ship_list;
+    while(shplst) {
+        struct Ship *ship=shplst->data;
+
+        if(Round(ship->x) == x && Round(ship->y-1) == y) {
+            ship->y--;
+            break;
+        }
+        shplst=shplst->next;
+    }
+}
+
 /* Cloaking device engage/disengage effect */
 static void cloaking_device (struct Ship * ship, char status)
 {
@@ -1171,7 +1211,7 @@ static void cloaking_device (struct Ship * ship, char status)
     Particle *part;
     Uint32 *src;
     int x, y, p;
-    p = roundplus ((35 - ship->angle / (2.0 * M_PI / 35.0)));
+    p = Roundplus ((35 - ship->angle / (2.0 * M_PI / 35.0)));
     if (p > 35)
         p = 35;
     surf = ship->ship[p];
@@ -1250,6 +1290,7 @@ void ship_fire (struct Ship * ship, WeaponClass weapon) {
     if (weapon == NormalWeapon) {       /* Standard weapon */
         ship_fire_standard_weapon (ship, p, v, ps);
     } else if (weapon == SpecialWeapon) {       /* Special weapon */
+        sfx = WAV_SPECIALWEAP;
         if (ship->energy <= 0.1) {
             free (p);
             return;
@@ -1330,11 +1371,7 @@ void ship_fire (struct Ship * ship, WeaponClass weapon) {
             p->type = MagMine;
             p->vector.x = sin (ship->angle) * -2.0;
             p->vector.y = cos (ship->angle) * -2.0;
-#if HAVE_LIBSDL_GFX
-            p->color = 0x959EFFFF;
-#else
-            p->color = SDL_MapRGB (screen->format, 149, 158, 255);
-#endif
+            p->color = map_rgba(149,158,255,255);
             p->gravity = NULL;
             p->wind_affects = 0;
             ship->energy -= 0.06;
@@ -1424,7 +1461,7 @@ void ship_fire (struct Ship * ship, WeaponClass weapon) {
                 double f;
                 Projectile *proj;
                 for (f = ship->angle - 0.25; f < ship->angle + 0.25;
-                     f += 0.09) {
+                     f += 0.08) {
                     proj =
                         make_projectile (p->x, p->y,
                                          makeVector (sin (f) * 1.5,
@@ -1540,6 +1577,7 @@ void ship_fire (struct Ship * ship, WeaponClass weapon) {
                 p->vector.y = ship->vector.y + 6.0;
                 ship->special_cooloff = 30;
                 ship->energy -= 0.3;
+                ship->no_power = 60;
                 for (a = 0.1; a < 2 * M_PI; a += 0.1) {
                     Projectile *newp;
                     newp = malloc (sizeof (Projectile));
@@ -1569,8 +1607,8 @@ void ship_fire (struct Ship * ship, WeaponClass weapon) {
                     p->y = i->y;
                 } else {
                     r = (rand () % 628) / 100.0;
-                    p->x += round (cos (r) * 25);
-                    p->y += round (sin (r) * 25);
+                    p->x += Round (cos (r) * 25);
+                    p->y += Round (sin (r) * 25);
                 }
                 playsound = 0;
             }
@@ -1610,10 +1648,8 @@ void ship_fire (struct Ship * ship, WeaponClass weapon) {
     if (ship->energy < 0)
         ship->energy = 0;
     add_projectile (p);
-#if HAVE_LIBSDL_MIXER
     if (playsound)
         playwave (sfx);
-#endif
 #ifdef CHEAT_POSSIBLE
     if (cheat1)
         ship->energy = 1.0;
@@ -1685,9 +1721,7 @@ static char ship_fire_special_noproj (struct Ship * ship)
             ship->fire_weapon = 0;
             if (ship->energy < 0)
                 ship->energy = 0;
-#if HAVE_LIBSDL_MIXER
             playwave (WAV_DART);
-#endif
         }
         return 1;
     case WLandmine:
@@ -1713,7 +1747,7 @@ static char ship_fire_special_noproj (struct Ship * ship)
         return 1;
     case WHelicopter:
         if (ship->energy >= 0.25
-            && lev_level.solid[round(ship->x)][round(ship->y)] == TER_FREE) {
+            && lev_level.solid[Round(ship->x)][Round(ship->y)] == TER_FREE) {
             add_critter (make_critter
                          (Helicopter, ship->x, ship->y, find_player (ship)));
             ship->energy -= 0.25;
@@ -1728,21 +1762,22 @@ static char ship_fire_special_noproj (struct Ship * ship)
             ship->remote_control->thrust = 0;
             ship->remote_control->turn = 0;
             ship->remote_control->fire_weapon = 0;
-            ship->remote_control->fire_special_weapon = 0;
+            ship_stop_special(ship->remote_control);
             ship->remote_control = NULL;
         } else if (ship->energy > 0) {  /* Active remote control */
             struct Ship *targ;
             double d;
             targ = find_nearest_ship (ship->x, ship->y, ship, &d);
             if (targ && d < 150) {
-                ship->remote_control = targ;
-                targ->remote_control = (struct Ship *) 1;      /* To indicate that the ship is being remote controlled */
                 targ->thrust = 0;
                 targ->turn = 0;
                 targ->fire_weapon = 0;
                 ship->thrust = 0;
                 ship->turn = 0;
                 ship->fire_weapon = 0;
+                ship_stop_special(targ);
+                ship->remote_control = targ;
+                targ->remote_control = (struct Ship *) 1; /* To indicate that the ship is being remote controlled */
             }
         }
         return 1;
@@ -1802,7 +1837,6 @@ static void ship_fire_standard_weapon (struct Ship * ship, Projectile * p, Vecto
             Vector vec;
             p->vector.x = v.x + ps.x;
             p->vector.y = v.y + ps.y;
-            ship->cooloff = 3;
 
             proj = make_projectile (p->x, p->y, p->vector);
             proj->vector.x = -v.x + ps.x;
@@ -1825,7 +1859,7 @@ static void ship_fire_standard_weapon (struct Ship * ship, Projectile * p, Vecto
             proj->owner = p->owner;
             add_projectile (proj);
 
-            ship->cooloff = 14;
+            ship->cooloff = 7;
         }
         break;
 
@@ -1921,10 +1955,10 @@ int find_nearest_player (int myX, int myY, int not_this, double *dist) {
 
 /* Note. This is a bit different that find_nearest_player       */
 /* First of all, this returns a pointer to the ship directly    */
-/* Second, not_this is a pointer to a ship, not a player number */
+/* Second, not_this is a pointer to a ship, not a team number */
 struct Ship *find_nearest_ship (int myX, int myY, struct Ship * not_this, double *dist)
 {
-    struct dllist *current = ships;
+    struct dllist *current = ship_list;
     struct Ship *nearest = NULL;
     double distance = 9999999, d;
     while (current) {
@@ -1947,7 +1981,7 @@ struct Ship *find_nearest_ship (int myX, int myY, struct Ship * not_this, double
 }
 
 struct Ship *hit_ship(double x, double y, struct Ship *ignore, double radius) {
-    struct dllist *current = ships;
+    struct dllist *current = ship_list;
     while (current) {
         struct Ship *ship=current->data;
         if (ship->dead == 0 && ship!=ignore) {
@@ -1997,6 +2031,7 @@ signed char ship_damage (struct Ship * ship, Projectile * proj)
         ship->darting = SPEARED;
         ship->fire_weapon = 0;
         ship->fire_special_weapon = 0;
+        ship->turn=0;
         damage = 0.01;
         rval = -1;
         critical = 4;
