@@ -52,6 +52,7 @@ struct Ship {
     char onbase;
     /* Mostly weapons related stuff */
     unsigned char criticals;
+    int no_power;
     int cooloff, special_cooloff, eject_cooloff;
     int fire_weapon;
     int fire_special_weapon;
@@ -90,11 +91,12 @@ extern int find_nearest_player (int myX, int myY, int not_this, double *dist);
 struct Ship *find_nearest_ship (int myX, int myY, struct Ship * not_this, double *dist);
 extern signed char ship_damage (struct Ship * ship, Projectile * proj);
 extern void killship (struct Ship * ship);
-extern struct dllist *getshiplist (void);
 extern void claim_ship (struct Ship * ship, int plr);
-extern void ship_critical (struct Ship * ship, char repair);
+extern void ship_critical (struct Ship * ship, int repair);
+extern void bump_ship(int x,int y);
 
 /* Globals */
 extern SDL_Surface **ship_gfx[7];
+extern struct dllist *ship_list;
 
 #endif
