@@ -37,10 +37,7 @@
 #include "special.h"
 #include "vector.h"
 #include "ship.h"
-
-#if HAVE_LIBSDL_MIXER
 #include "audio.h"
-#endif
 
 /* Internally used globals */
 static struct dllist *specials;
@@ -368,19 +365,19 @@ static inline void draw_special (SpecialObj * special)
         break;
     }
     for (p = 0; p < 4; p++) {
-        if (players[p].state==ALIVE && (showme == p || showme == -1)) {
+        if ((players[p].state==ALIVE||players[p].state==DEAD) && (showme == p || showme == -1)) {
             rect.w = sprite[0]->w;
             rect.h = sprite[0]->h;
             rect.x = special->x - cam_rects[p].x + lev_rects[p].x;
             rect.y = special->y - cam_rects[p].y + lev_rects[p].y;
             if ((rect.x > lev_rects[p].x - rect.w
-                 && rect.x < lev_rects[p].x + lev_rects[p].w)
+                 && rect.x < lev_rects[p].x + cam_rects[p].w)
                 && (rect.y > lev_rects[p].y - rect.h
-                    && rect.y < lev_rects[p].y + lev_rects[p].h)) {
+                    && rect.y < lev_rects[p].y + cam_rects[p].h)) {
                 rect2 =
                     cliprect (rect.x, rect.y, rect.w, rect.h, lev_rects[p].x,
-                              lev_rects[p].y, lev_rects[p].x + lev_rects[p].w,
-                              lev_rects[p].y + lev_rects[p].h);
+                              lev_rects[p].y, lev_rects[p].x + cam_rects[p].w,
+                              lev_rects[p].y + cam_rects[p].h);
                 if (rect.x < lev_rects[p].x)
                     rect.x = lev_rects[p].x;
                 if (rect.y < lev_rects[p].y)
@@ -397,7 +394,7 @@ static void ship_hit_special(struct dllist *list) {
     struct dllist *ships;
     int w;
     w = 32;
-    ships = getshiplist ();
+    ships = ship_list;
     while (ships) {
         struct Ship *ship=ships->data;
         if (ship->x >= obj->x && ship->x <= obj->x + w)
@@ -483,7 +480,7 @@ static void turret_shoot(SpecialObj *turret) {
                 || turret->var1 == 2
                 || ((angle > -M_PI_2 && angle < M_PI_2) && turret->var1 > 2)) {
         if (turret->var1 > 2) {      /* Upside down */
-            targframe=round( (angle+M_PI_2) / M_PI * (TURRET_FRAMES-1) );
+            targframe=Round( (angle+M_PI_2) / M_PI * (TURRET_FRAMES-1) );
         } else {    /* Normal */
             double a2;
             if (angle > -M_PI && angle < -M_PI_2) {
@@ -495,7 +492,7 @@ static void turret_shoot(SpecialObj *turret) {
             } else { /* Missile turret can target ships from below as well */
                 a2=angle+M_PI_2;
             }
-            targframe=round( a2 / M_PI * (TURRET_FRAMES-1) );
+            targframe=Round( a2 / M_PI * (TURRET_FRAMES-1) );
         }
         if(targframe>=TURRET_FRAMES) targframe=TURRET_FRAMES-1;
         else if(targframe<0) targframe=0;
@@ -642,6 +639,7 @@ int projectile_hit_special (Projectile * proj) {
                                             FireStarter);
                             add_explosion (obj->x, obj->y,
                                            Noproj);
+                            playwave_3d(WAV_EXPLOSION2,obj->x,obj->y);
                             free (obj);
                             if(list==specials)
                                 specials=dllist_remove(list);
