@@ -59,6 +59,8 @@ void init_startup_options (void) {
     luola_options.audio_chunks = 256;
     luola_options.sfont = 0;
     luola_options.mbg_anim = 1;
+    luola_options.videomode = VID_640;
+
     /* Load configuration file (if exists) */
     fp = fopen (getfullpath (HOME_DIRECTORY, STARTUP_FILE), "r");
     if (!fp)
@@ -102,34 +104,40 @@ void init_startup_options (void) {
         } else if (strcmp (line, "audiochunks") == 0) {
             fgets (tmps, sizeof (tmps) - 1, fp);
             luola_options.audio_chunks = atoi (tmps);
+        } else if (strcmp (line, "videomode") == 0) {
+            fgets (tmps, sizeof (tmps) - 1, fp);
+            luola_options.videomode = atoi (tmps);
+            if(luola_options.videomode<VID_640)
+                luola_options.videomode=VID_640;
+            else if(luola_options.videomode>VID_1024)
+                luola_options.videomode=VID_1024;
         }
     }
     fclose (fp);
 }
 
 void print_help (void) {
-    printf ("Luola command tmps options:\n");
-    printf ("  --fullscreen	Full screen mode\n");
-    printf ("  --window		Windowed mode\n");
-    printf
-        ("  --hidemouse		Hide the mouse when it's over the window\n");
-    printf
-        ("  --showmouse		Show the mouse when it's over the window\n");
-    printf ("  --pad		Enable joystick\n");
-    printf ("  --nopad		Disable joystick\n");
+    printf ("Luola command line options:\n");
+    printf ("  --fullscreen               Full screen mode\n");
+    printf ("  --window                   Windowed mode\n");
+    printf ("  --videomode <size>         Screen size (640x480,800x600,1024x768)\n");      
+    printf ("  --hidemouse                Hide the mouse cursor when it's over the window\n");
+    printf ("  --showmouse                Show the mouse cursor when it's over the window\n");
+    printf ("  --pad                      Enable joypad\n");
+    printf ("  --nopad                    Disable joypad\n");
 #if HAVE_LIBSDL_MIXER
-    printf ("  --sounds		Enable sounds\n");
-    printf ("  --nosounds		Disable sounds\n");
+    printf ("  --sounds                   Enable sounds\n");
+    printf ("  --nosounds                 Disable sounds\n");
 #endif
-    printf ("  --sfont		Force SFont to be used\n");
-    printf ("  --ttf		Attempt to use SDL_ttf for font rendering\n");
-    printf ("  --menu-animation	Enable menu background animation\n");
-    printf ("  --no-menu-animation	Disable menu background animation\n");
-    printf ("  --audiorate		Set audio sampling frequency\n");
-    printf ("  --audiochannels	Set audio channels (1 or 2)\n");
-    printf ("  --audiochunks	Set audio chunks\n");
-    printf ("  --help		Show this message\n");
-    printf ("  --version		Show version information\n\n");
+    printf ("  --sfont                    Force SFont to be used\n");
+    printf ("  --ttf                      Attempt to use SDL_ttf for font rendering\n");
+    printf ("  --menu-animation           Enable menu background animation\n");
+    printf ("  --no-menu-animation        Disable menu background animation\n");
+    printf ("  --audiorate <rate>         Set audio sampling frequency\n");
+    printf ("  --audiochannels <channels> Set audio channels (1 or 2)\n");
+    printf ("  --audiochunks <chunks>     Set audio chunks\n");
+    printf ("  --help                     Show this message\n");
+    printf ("  --version                  Show version information\n\n");
 }
 
 /* Parse a command tmps argument */
@@ -184,6 +192,23 @@ int parse_argument (int r, int argc, char **argv) {
             printf ("You did not specify number of chunks\n");
             return 0;
         }
+    } else if (strcmp (argv[r], "--videomode") == 0) {
+        if (r + 1 < argc) {
+            r++;
+            if(strcmp(argv[r],"640x480")==0)
+                luola_options.videomode=VID_640;
+            else if(strcmp(argv[r],"800x600")==0)
+                luola_options.videomode=VID_800;
+            else if(strcmp(argv[r],"1024x768")==0)
+                luola_options.videomode=VID_1024;
+            else {
+                printf ("Unknown video mode %s. Supported modes are 640x480,800x600 and 1024x768",argv[r]);
+                return 0;
+            }
+        } else {
+            printf ("You did not specify the video mode\n");
+            return 0;
+        }
     } else {
         printf ("Unrecognized argument: %s\n", argv[r]);
         return 0;
@@ -226,6 +251,7 @@ char save_startup_config (void) {
         fprintf (fp, "menu-animation\n");
     else
         fprintf (fp, "no-menu-animation\n");
+    fprintf(fp, "videomode\n%d\n",luola_options.videomode);
     /* Done. */
     fclose (fp);
     return 0;
