@@ -1,6 +1,6 @@
 /*
  * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2003-2004 Calle Laakkonen
+ * Copyright (C) 2003-2005 Calle Laakkonen
  *
  * File        : startup.h
  * Description : Startup options
@@ -32,8 +32,8 @@ typedef struct {
     int sounds;
     int audio_rate, audio_channels, audio_chunks;
     int sfont;
-    int ttf_mode;              /* Latin1, UTF8 */
     int mbg_anim;
+    enum {VID_640,VID_800,VID_1024} videomode;
 } StartupOptions;
 
 /* The structure used by everyone */
