@@ -204,8 +204,8 @@ void energy_movement (Projectile * proj)
     float p;
     for (p = 1; p <= 3; p++) {
         part =
-            make_particle (round(proj->x - proj->vector.x / p),
-                           round(proj->y - proj->vector.y / p), 30);
+            make_particle (Round(proj->x - proj->vector.x / p),
+                           Round(proj->y - proj->vector.y / p), 30);
         part->color[0] = 255;
         part->color[1] = 255;
         part->color[2] = 0;
@@ -367,7 +367,7 @@ void boomerang_movement (Projectile * proj) {
     if (proj->var1 > 10) {
         struct dllist *ships;
         /* First check that the ship still exists */
-        ships=getshiplist();
+        ships=ship_list;
         while(ships) {
             if(ships->data==proj->owner) break;
             ships=ships->next;
