@@ -1,6 +1,6 @@
 /*
  * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2003 Calle Laakkonen
+ * Copyright (C) 2003-2005 Calle Laakkonen
  *
  * File        : vector.c
  * Description : Vector math
@@ -21,6 +21,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <math.h>
+
 #include "vector.h"
 
 Vector makeVector (double X, double Y)
@@ -38,3 +40,12 @@ Vector oppositeVector (Vector * vec)
     opp.y = -vec->y;
     return opp;
 }
+
+void rotateVector(Vector *vec, double th) {
+    double c = cos(th);
+    double s = sin(th);
+    double tx = c * vec->x - s * vec->y;
+    vec->y = s * vec->x + c * vec->y;
+    vec->x = tx;
+}
+
