@@ -39,9 +39,7 @@
 #include "weather.h"
 #include "ship.h"
 
-#if HAVE_LIBSDL_MIXER
 #include "audio.h"
-#endif
 
 #include "hole.h"
 
@@ -196,9 +194,7 @@ void add_explosion (int x, int y, ProjectileType cluster) {
     struct Explosion *newentry;
     char solid;
     int fx, fy, rx, ry;
-#if HAVE_LIBSDL_MIXER
     playwave_3d (WAV_EXPLOSION, x, y);
-#endif
     if (game_settings.explosions || (cluster != Noproj && cluster != Zap)) {
         newentry = malloc (sizeof (struct Explosion));
         newentry->x = x;
@@ -334,9 +330,9 @@ static void draw_special_weapons (int x, int y, Projectile * proj) {
             for (seg = 0; seg < 4; seg++) {
                 dx = (targx - x + ((rand () % 20) - 10)) / 3;
                 dy = (targy - y + ((rand () % 20) - 10)) / 3;
-                if (x > 0 && x < SCREEN_W && y > 0 && y < SCREEN_H)
-                    if (x + dx > 0 && x + dx < SCREEN_W && y + dy > 0
-                        && y + dy < SCREEN_H)
+                if (x > 0 && x < screen->w && y > 0 && y < screen->h)
+                    if (x + dx > 0 && x + dx < screen->w && y + dy > 0
+                        && y + dy < screen->h)
                         draw_line (screen, x, y, x + dx, y + dy, col_yellow);
                 x += dx;
                 y += dy;
@@ -344,10 +340,10 @@ static void draw_special_weapons (int x, int y, Projectile * proj) {
         }
         break;
     case Boomerang:
-        putpixel (screen, x - round (cos (proj->angle - 0.5)),
-                  y - round (sin (proj->angle - 0.5)), proj->color);
-        putpixel (screen, x - round (cos (proj->angle + 0.5)),
-                  y - round (sin (proj->angle + 0.5)), proj->color);
+        putpixel (screen, x - Round (cos (proj->angle - 0.5)),
+                  y - Round (sin (proj->angle - 0.5)), proj->color);
+        putpixel (screen, x - Round (cos (proj->angle + 0.5)),
+                  y - Round (sin (proj->angle + 0.5)), proj->color);
         break;
     default:
         break;
@@ -362,12 +358,30 @@ static inline void draw_projectile (Projectile * projectile)
         return;
     /* Draw the projectile */
     for (p = 0; p < 4; p++) {
-        if (players[p].state==ALIVE) {
-            x = round(projectile->x) - cam_rects[p].x;
-            y = round(projectile->y) - cam_rects[p].y;
-            if ((x > 0 && x < 320) && (y > 0 && y < 240)) {
+        if (players[p].state==ALIVE || players[p].state==DEAD) {
+            x = Round(projectile->x) - cam_rects[p].x;
+            y = Round(projectile->y) - cam_rects[p].y;
+            if ((x > 0 && x < cam_rects[p].w) && (y > 0 && y < cam_rects[p].h)) {
                 putpixel (screen, x + lev_rects[p].x, y + lev_rects[p].y,
                           projectile->color);
+                if (game_settings.large_bullets) {
+                    if (x + lev_rects[p].x + 1 < cam_rects[p].w) {
+                        putpixel (screen, x + lev_rects[p].x + 1, y + lev_rects[p].y,
+                                  projectile->color);
+                    }
+                    if (x + lev_rects[p].x - 1 > 0) {
+                        putpixel (screen, x + lev_rects[p].x - 1, y + lev_rects[p].y,
+                                  projectile->color);
+                    }
+                    if (y + lev_rects[p].y + 1 < cam_rects[p].h) {
+                        putpixel (screen, x + lev_rects[p].x, y + lev_rects[p].y + 1,
+                                  projectile->color);
+                    }
+                    if (y + lev_rects[p].y - 1 > 0) {
+                        putpixel (screen, x + lev_rects[p].x, y + lev_rects[p].y - 1,
+                                  projectile->color);
+                    }
+                }
                 draw_special_weapons (x + lev_rects[p].x, y + lev_rects[p].y,
                                       projectile);
             }
@@ -388,10 +402,10 @@ static inline void draw_explosions (void) {
                 rect.x = lev_rects[p].x + expl->x - cam_rects[p].x - 8;
                 rect.y = lev_rects[p].y + expl->y - cam_rects[p].y - 8;
                 if ((rect.x > lev_rects[p].x
-                     && rect.x < lev_rects[p].x + lev_rects[p].w - tmpsurf->w)
+                     && rect.x < lev_rects[p].x + cam_rects[p].w - tmpsurf->w)
                     && (rect.y > lev_rects[p].y
                         && rect.y <
-                        lev_rects[p].y + lev_rects[p].h - tmpsurf->h))
+                        lev_rects[p].y + cam_rects[p].h - tmpsurf->h))
                     SDL_BlitSurface (tmpsurf, NULL, screen, &rect);
             }
         }
@@ -415,7 +429,7 @@ void animate_weapons (void)
             expl_cluster = Noproj;
         else
             expl_cluster = Zap;
-  /*** Collision detection ***/
+        /* Collision detection */
         if (proj->primed == 0) {
             explode = 0;
             /* Collide with a ship ? */
@@ -428,21 +442,22 @@ void animate_weapons (void)
                 if (explode)
                     explode = ship_damage ((struct Ship *) explode, proj);
                 if (explode) {
-                    if (explode == -2) {    /* Did the ship just swallow the projectile ? */
+                    if (explode == -2) {
+                        /* Did the ship just swallow the projectile? */
                         plist=remove_projectile(plist);
                         continue;
                     }
                 }
-                /* Collide with a level special ? (Same types of projectiles that collide with ships) */
+                /* Collide with a level special? (Same types of projectiles that collide with ships) */
                 if (explode == 0)
                     explode = projectile_hit_special (proj);
             }
-            /* Collide with a critter or a pilot ? Also check if the projectile is underwater */
+            /* Collide with a critter or a pilot? Also check if the projectile is underwater */
             if (explode == 0 && proj->type != GravityWell
                 && proj->type != Energy && proj->type != MagWave
                 && proj->type != FireStarter) {
                 if (proj->type != Decor)
-                    explode = hit_critter (round(proj->x), round(proj->y), proj->type);
+                    explode = hit_critter (Round(proj->x), Round(proj->y), proj->type);
                 if (explode
                     && (proj->type == Spear || proj->type == FireStarter
                         || proj->type == Acid))
@@ -456,7 +471,7 @@ void animate_weapons (void)
                         proj->vector.x *= -1;
                 }
                 if (explode == 0 && (proj->type != Decor && proj->type!=FireStarter))
-                    explode = hit_pilot (round(proj->x), round(proj->y),proj->ownerteam);
+                    explode = hit_pilot (Round(proj->x), Round(proj->y),proj->ownerteam);
             }
             /* Check if the projectile goes outside its boundaries */
             if (proj->x <= 1)
@@ -468,7 +483,7 @@ void animate_weapons (void)
             else if (proj->y >= lev_level.height - 1)
                 explode = -1;
             /* Mines collide with other projectiles */
-            if (explode == 0 && (proj->type == DividingMine || proj->type == Mine || proj->type == MagMine)) {      /* Mines detonate when they hit other projectiles */
+            if (explode == 0 && (proj->type == DividingMine || proj->type == Mine || proj->type == MagMine)) {/* Mines detonate when they hit other projectiles */
                 plist3 = projectiles;
                 while (plist3) {
                     Projectile *proj3=plist3->data;
@@ -494,8 +509,8 @@ void animate_weapons (void)
             }
             /* A hack to make Claybomb react with things other than ground */
             if (explode && proj->type == Claybomb) {
-                lev_level.solid[round(proj->x)][round(proj->y)] =
-                    (lev_level.solid[round(proj->x)][round(proj->y)] ==
+                lev_level.solid[Round(proj->x)][Round(proj->y)] =
+                    (lev_level.solid[Round(proj->x)][Round(proj->y)] ==
                      TER_WATER) ? TER_UNDERWATER : TER_GROUND;
                 explode = 0;
             }
@@ -504,11 +519,11 @@ void animate_weapons (void)
                 && proj->type != GravityWell && proj->type != MagWave) {
                 int tmpx,tmpy;
                 solid =
-                    hit_solid_line (round(proj->x), round(proj->y),
-                                    round(proj->x - proj->vector.x),
-                                    round(proj->y - proj->vector.y),
+                    hit_solid_line (Round(proj->x), Round(proj->y),
+                                    Round(proj->x - proj->vector.x),
+                                    Round(proj->y - proj->vector.y),
                                     &tmpx, &tmpy);
-                if (solid < 0) {    /* Hit water */
+                if (solid < 0) { /* Hit water */
                     switch (proj->type) {
                         case DividingMine:
                         case Tag:
@@ -528,13 +543,11 @@ void animate_weapons (void)
                             proj->x=tmpx; proj->y=tmpy;
                             alter_level (tmpx, tmpy,
                                      solid == TER_GROUND ? 30 : 15, Ice);
-#if HAVE_LIBSDL_MIXER
                             playwave_3d (WAV_FREEZE, proj->x, proj->y);
-#endif
                         default:
                             break;
                     }
-                } else if(solid>0) {    /* Hit ground */
+                } else if(solid>0) { /* Hit ground */
                     proj->x=tmpx; proj->y=tmpy;
                     if (solid == TER_EXPLOSIVE)
                         expl_cluster = Cannon;
@@ -547,7 +560,7 @@ void animate_weapons (void)
                                  || solid == TER_COMBUSTABL2)
                                 || (solid == TER_EXPLOSIVE
                                     || solid == TER_EXPLOSIVE2)) {
-                                start_burning (round(proj->x), round(proj->y));
+                                start_burning (Round(proj->x), Round(proj->y));
                                 explode = -1;
                             }
                             break;
@@ -560,41 +573,33 @@ void animate_weapons (void)
                                      || solid == TER_COMBUSTABL2
                                      || solid == TER_EXPLOSIVE
                                      || solid == TER_EXPLOSIVE2)
-                                start_burning (round(proj->x), round(proj->y));
+                                start_burning (Round(proj->x), Round(proj->y));
                             proj->vector.x = 0;
                             proj->vector.y = 0;
                             proj->wind_affects = 0;
                             break;
                         case Plastique:
                             explode = -1;
-                            alter_level (round(proj->x), round(proj->y),
+                            alter_level (Round(proj->x), Round(proj->y),
                                          5 + 3 * (game_settings.holesize),
                                          Explosive);
-#if HAVE_LIBSDL_MIXER
                             playwave_3d (WAV_SWOOSH, proj->x, proj->y);
-#endif
                             break;
                         case Claybomb:
                             explode = -1;
-                            alter_level (round(proj->x), round(proj->y), 15, Earth);
-#if HAVE_LIBSDL_MIXER
+                            alter_level (Round(proj->x), Round(proj->y), 15, Earth);
                             playwave_3d (WAV_SWOOSH, proj->x, proj->y);
-#endif
                             break;
                         case Snowball:
                             explode = -1;
-                            alter_level (round(proj->x), round(proj->y),
+                            alter_level (Round(proj->x), Round(proj->y),
                                          solid == TER_GROUND ? 30 : 15, Ice);
-#if HAVE_LIBSDL_MIXER
                             playwave_3d (WAV_FREEZE, proj->x, proj->y);
-#endif
                             break;
                         case Acid:
                             explode = -1;
-                            start_melting (round(proj->x), round(proj->y), 10);
-#if HAVE_LIBSDL_MIXER
+                            start_melting (Round(proj->x), Round(proj->y), 10);
                             playwave_3d (WAV_STEAM, proj->x, proj->y);
-#endif
                             break;
                         case Waterjet:
                         case Decor:
@@ -604,8 +609,8 @@ void animate_weapons (void)
                                 int dx, dy;
                                 dx = sin (proj->angle) * 6;
                                 dy = cos (proj->angle) * 6;
-                                draw_line (lev_level.terrain, round(proj->x) - dx,
-                                           proj->y - dy, round(proj->x) + dx,
+                                draw_line (lev_level.terrain, Round(proj->x) - dx,
+                                           proj->y - dy, Round(proj->x) + dx,
                                            proj->y + dy, proj->color);
                                 explode = -1;
                             } break;
@@ -619,64 +624,66 @@ void animate_weapons (void)
             /* Larger explosions */
             if (explode) {
                 if (explode > 0)
-                    add_explosion (round(proj->x), round(proj->y), expl_cluster);
+                    add_explosion (Round(proj->x), Round(proj->y), expl_cluster);
                 switch (proj->type) {
                     case Grenade:
-                        spawn_clusters (round(proj->x), round(proj->y), 16, Cannon);
-                        spawn_clusters (round(proj->x), round(proj->y), 16, FireStarter);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16, Cannon);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16, FireStarter);
                         break;
                     case MegaBomb:
-                        spawn_clusters (round(round(proj->x)), round(round(proj->y)),
+                        spawn_clusters (Round(proj->x), Round(proj->y),
                                         8, Grenade);
-                        spawn_clusters (round(proj->x), round(proj->y), 16, FireStarter);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16, FireStarter);
+                        playwave_3d(WAV_EXPLOSION2,proj->x,proj->y);
                         break;
                     case Missile:
-                        spawn_clusters (round(round(proj->x)), round(round(proj->y)),
+                        spawn_clusters (Round(proj->x), Round(proj->y),
                                         10, Cannon);
-                        spawn_clusters (round(proj->x), round(proj->y), 16,
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16,
                                         FireStarter);
                         break;
                     case Boomerang:
-                        spawn_clusters (round(proj->x), round(proj->y), 4,  Cannon);
-                        spawn_clusters (round(proj->x), round(proj->y), 16, FireStarter);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 4,  Cannon);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16, FireStarter);
                         break;
                     case Rocket:
-                        spawn_clusters (round(proj->x), round(proj->y), 3, Grenade);
-                        spawn_clusters (round(proj->x), round(proj->y), 6, Cannon);
-                        spawn_clusters (round(proj->x), round(proj->y), 16, FireStarter);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 3, Grenade);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 6, Cannon);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16, FireStarter);
+                        playwave_3d(WAV_EXPLOSION2,proj->x,proj->y);
                         break;
                     case DividingMine:
-                        spawn_clusters (round(proj->x), round(proj->y), 4, Cannon);
-                        spawn_clusters (round(proj->x), round(proj->y), 16, FireStarter);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 4, Cannon);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16, FireStarter);
                         break;
                     case MagMine:
                     case Mine:
-                        spawn_clusters (round(proj->x), round(proj->y), 8, Cannon);
-                        spawn_clusters (round(proj->x), round(proj->y), 16, FireStarter);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 8, Cannon);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16, FireStarter);
                         break;
                     case GravityWell:
                         gravity_weapon_count--;
                         break;
                     case Mush:
-                        spawn_clusters (round(proj->x), round(proj->y), 5, Cannon);
-                        spawn_clusters (round(proj->x), round(proj->y), 16, FireStarter);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 5, Cannon);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16, FireStarter);
                         break;
                     case Ember:
-                        spawn_clusters (round(proj->x), round(proj->y),
+                        spawn_clusters (Round(proj->x), Round(proj->y),
                                         3 * (proj->var1 + 1), Napalm);
-                        spawn_clusters (round(proj->x), round(proj->y), 16,
+                        spawn_clusters (Round(proj->x), Round(proj->y), 16,
                                         FireStarter);
                         break;
                     case Mirv:
                         if (proj->var1 == 3)
-                            spawn_clusters (round(proj->x), round(proj->y), 3, Grenade);
+                            spawn_clusters (Round(proj->x), Round(proj->y), 3, Grenade);
                         else if (proj->var1 == 2)
-                            spawn_clusters (round(proj->x), round(proj->y), 32, Cannon);
+                            spawn_clusters (Round(proj->x), Round(proj->y), 32, Cannon);
                         else if (proj->var1 == 1)
-                            spawn_clusters (round(proj->x), round(proj->y), 12, Cannon);
+                            spawn_clusters (Round(proj->x), Round(proj->y), 12, Cannon);
                         else
-                            spawn_clusters (round(proj->x), round(proj->y), 6, Cannon);
-                        spawn_clusters (round(proj->x), round(proj->y), 8, FireStarter);
+                            spawn_clusters (Round(proj->x), Round(proj->y), 6, Cannon);
+                        spawn_clusters (Round(proj->x), Round(proj->y), 8, FireStarter);
                         break;
                     case Zap:
                         proj->primed = proj->life;
@@ -739,7 +746,7 @@ void animate_weapons (void)
             mush_movement (proj);
             break;
         case Zap:
-            expl_cluster = Zap;     /* This is to tell add_explosion to char the ground */
+            expl_cluster = Zap; /* This is to tell add_explosion to char the ground */
             break;
         case Mirv:
             mirv_movement (proj);
@@ -753,7 +760,7 @@ void animate_weapons (void)
         /* End weapon specific movement/fx code */
         /* Check if someone is operating a force field nearby */
         if (proj->type != GravityWell) {
-            struct dllist *ships = getshiplist ();
+            struct dllist *ships = ship_list;
             while (ships) {
                 struct Ship *ship=ships->data;
                 if (ship->shieldup) {
@@ -784,7 +791,7 @@ void animate_weapons (void)
         }
         /* Is this is a gravity weapon ? */
         if (proj->type == GravityWell && proj->primed == 0) {
-            struct dllist *ships = getshiplist ();
+            struct dllist *ships = ship_list;
             plist2 = projectiles;   /* Attract other projectiles */
             while (plist2) {
                 Projectile *proj2=plist2->data;
@@ -828,23 +835,23 @@ void animate_weapons (void)
         /* Common movement code */
         proj->x -= proj->vector.x;
         proj->y -= proj->vector.y;
-        if (round(proj->x) < 0)
+        if (Round(proj->x) < 0)
             proj->x = 0;
-        else if (round(proj->x) >= lev_level.width)
+        else if (Round(proj->x) >= lev_level.width)
             proj->x = lev_level.width - 1;
-        if (round(proj->y) < 0)
+        if (Round(proj->y) < 0)
             proj->y = 0;
-        else if (round(proj->y) >= lev_level.height)
+        else if (Round(proj->y) >= lev_level.height)
             proj->y = lev_level.height - 1;
         if (proj->wind_affects
-            && lev_level.solid[round(proj->x)][round(proj->y)] == TER_FREE) {
+            && lev_level.solid[Round(proj->x)][Round(proj->y)] == TER_FREE) {
             proj->vector.x -= weather_wind_vector / 50.0;
         }
         if (proj->gravity) {
             proj->vector = addVectors (&proj->vector, proj->gravity);
             proj->vector.x /= 1.005;
             proj->vector.y /= 1.005;
-            switch (lev_level.solid[round(proj->x)][round(proj->y)]) {
+            switch (lev_level.solid[Round(proj->x)][Round(proj->y)]) {
                 case TER_FREE:
                     if(proj->wind_affects) proj->vector.x -= weather_wind_vector / 50.0;
                     break;
